/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: addressbook.c
 *         Address book handling functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>

#include "eucalyptus.h"

AddrEntryItem addr_layout_defaults[] =
{
	{ AEIT_STRING, N_("Name"), "", N_("Name"), 0, },
	{ AEIT_EMAIL, N_("Email"), "", N_("Email address"), 0, },
	{ -1, 0, },
};

gboolean addresses_read(void)
{
	gchar *fname;
	gboolean success = TRUE;
	FILE *fp;
	
	fname = path_build(g_get_home_dir(), ".eucalyptus", ".addresses", NULL);
	if(fp = fopen(fname, "r"))
	{
		gchar buf[WORKLEN];
		
		while(fgets(buf, sizeof(buf), fp))
		{
		}
	
		fclose(fp);
	}
	
	g_free(fname);
	
	return success;
}

gboolean addresses_read_layout(void)
{
	GList *node;
	glong i;
	gchar *fname = NULL;
	gboolean success = TRUE, done = FALSE;
	FILE *fp;
	
	/* Clear out the old layout.
	 */
	while(node = g_list_first(euc.addr.Items.List))
	{
		AddrEntryItem *aei = (AddrEntryItem *)node->data;
		
		euc.addr.Items.List = list_remove(&euc.addr.Items, aei);
		address_entry_item_free(aei);
	}
	 
	/* Read the new one, both the common
	 * and the custom layout.
	 */
	for(i=0; !done; i++)
	{
		switch(i)
		{
			/* From the share directory.
			 */
			case 0:
				fname = path_build(/* FIXME */ "/usr/local/share", "eucalyptus", "addr-layout", NULL);
				break;
				
			/* From the user's home directory.
			 */
			case 1:
				g_free(fname);
				fname = path_build(g_get_home_dir(), ".eucalyptus", ".addr-layout", NULL);
				break;

			/* Done.
			 */
			default:
				g_free(fname);
				done = TRUE;
				continue;
		}

		/* Open the file.
		 */
		if(fp = fopen(fname, "r"))
		{
			gchar buf[WORKLEN];
			
			while(fgets(buf, sizeof(buf) - 1, fp))
			{
			}
		
			fclose(fp);
		}
	}

	/* Check that we have some layout.
	 * If not, put some default items in.
	 */
	if(!g_list_length(euc.addr.Items.List))
	{
		for(i=0; addr_layout_defaults[i].Type != -1; i++)
		{
			AddrEntryItem *aei;
			
			if(aei = address_entry_item_new())
			{
				aei->Type = addr_layout_defaults[i].Type;
				aei->Item = g_strdup(addr_layout_defaults[i].Item);
				aei->Value = g_strdup(addr_layout_defaults[i].Value);
				aei->Label = g_strdup(addr_layout_defaults[i].Label);
			
				list_append(&euc.addr.Items, aei);
			}
		}
	}
	
	return success;
}

#if 0
void __saveds AddressBookThread(void)
{
     AddressBookControl *control;
     struct Node *node;
     LONG rxsig, error = 0;
     BOOL done = FALSE;

     if(!(control = euc.AddressBookControl = (AddressBookControl *)AllocVec(sizeof(AddressBookControl), MEMF_CLEAR)))
          goto thatsit;

     if(!(control->IDCMPPort = CreateMsgPort()))
          goto thatsit;

     if(!(control->AppPort = CreateMsgPort()))
          goto thatsit;

     if(!strlen(control->PortName))
          strncpy(control->PortName, "EUCALYPTUS_ADDR", sizeof(control->PortName) - 1);

#ifdef EUCALYPTUS_PRO
     if(!(control->ARexxO = NewObject(AREXX_GetClass(), NULL,
                    AREXX_HostName,     control->PortName,
                    AREXX_Commands,     RexxAddrCommands,
                    AREXX_NoSlot,       FALSE,
                    AREXX_ReplyHook,    &euc.RexxReplyHook,
                    TAG_END)))
     {
          euc.Flags |= EGF_REXXERROR;
          goto thatsit;
     }
#endif

     NewList(&control->Addresses);
     if(BuildAddressBook(&control->Addresses, BABF_KEEPSEL | BABF_COLUMNS))
          goto thatsit;

     CA_SetUpHook(control->IDCMPHook, AddressBookIDCMP, NULL);
     CA_SetUpHook(control->AppHook, AddressBookAppMsgs, NULL);

     if(oprefs->AddressBookColumns[0] == 0xFF)
     {
          memset(control->ColumnInfo, 0, sizeof(control->ColumnInfo));
          control->ColumnInfo[0].ci_Title = LocaleString(ETXT_LBTITLE_NAME);
          control->ColumnInfo[0].ci_Width = 50;
          control->ColumnInfo[0].ci_Flags = CIF_WEIGHTED | CIF_DRAGGABLE | CIF_SORTABLE;
          control->ColumnInfo[1].ci_Title = LocaleString(ETXT_LBTITLE_ADDRESS);
          control->ColumnInfo[1].ci_Width = 50;
          control->ColumnInfo[1].ci_Flags = CIF_WEIGHTED | CIF_DRAGGABLE | CIF_SORTABLE;
          control->ColumnInfo[2].ci_Width = -1;
     }
     else
     {
          LONG i = 0;
          LONG tab_titles[] = {ETXT_LBTITLE_NAME, ETXT_LBTITLE_ADDRESS, ETXT_LBTITLE_ORGANIZATION, ETXT_LBTITLE_TITLE,
                               ETXT_LBTITLE_STREETADDRESS, ETXT_LBTITLE_WORKPHONE, ETXT_LBTITLE_HOMEPHONE, ETXT_LBTITLE_FAX,
                               ETXT_LBTITLE_KEYID, ETXT_LBTITLE_COMMENTS, ETXT_LBTITLE_NICKNAME, ETXT_LBTITLE_HOMEPAGE, };

          memset(control->ColumnInfo, 0, sizeof(control->ColumnInfo));

          while(i < 5)
          {
               if(oprefs->AddressBookColumns[i] == 0xFF)
                    break;

               control->ColumnInfo[i].ci_Title = LocaleString(tab_titles[oprefs->AddressBookColumns[i]]);
               control->ColumnInfo[i].ci_Width = 0;
               control->ColumnInfo[i].ci_Flags = CIF_DRAGGABLE | CIF_SORTABLE;

               i++;
          }

          control->ColumnInfo[i].ci_Width = -1;
     }

     if(!(control->WindowO = NewObject(WINDOW_GetClass(), NULL,
               (sprefs->AddrWindowLeft == -1) ? WINDOW_Position : TAG_IGNORE,    WPOS_CENTERSCREEN,
               (sprefs->AddrWindowLeft == -1) ? TAG_IGNORE : WA_Left,           sprefs->AddrWindowLeft,
               (sprefs->AddrWindowLeft == -1) ? TAG_IGNORE : WA_Top,            sprefs->AddrWindowTop,
               WA_Width,                (sprefs->AddrWindowLeft == -1) ? 450 : sprefs->AddrWindowWidth,
               (sprefs->AddrWindowLeft == -1) ? TAG_IGNORE : WA_Height,         sprefs->AddrWindowHeight,

               WA_Activate,             TRUE,
               WA_CloseGadget,          TRUE,
               WA_DragBar,              TRUE,
               WA_DepthGadget,          TRUE,
               WA_SizeGadget,           TRUE,
               WA_SizeBRight,           TRUE,
               WA_RMBTrap,              TRUE,
               WINDOW_BackFillName,     sprefs->Backfill,

               WINDOW_AppPort,          control->AppPort,
               WINDOW_IconifyGadget,    TRUE,
               WINDOW_Icon,             GetDiskObject("PROGDIR:Eucalyptus/def_addressbook"),
               WINDOW_AppMsgHook,       &control->AppHook,

               WINDOW_SharedPort,       control->IDCMPPort,
               WINDOW_IDCMPHook,        &control->IDCMPHook,
               WINDOW_IDCMPHookBits,    IDCMP_IDCMPUPDATE,
               WA_IDCMP,                IDCMP_RAWKEY,

               WA_PubScreen,            euc.Screen,
               WA_Title,                LocaleString(ETXT_TITLE_ADDRESSBOOK),
               WA_ScreenTitle,          eucsctitle,

               WINDOW_Layout,           control->Layout = VGroupObject,
                    LAYOUT_DeferLayout, TRUE,
                    LAYOUT_SpaceOuter,  TRUE,
                    StartHGroup,
                         LAYOUT_VertAlignment,    LALIGN_TOP,
                         StartMember,        control->ListBrowser = ListBrowserObject,
                              LISTBROWSER_Labels,      &control->Addresses,
                              LISTBROWSER_AutoFit,     TRUE,
                              LISTBROWSER_ColumnInfo,  &control->ColumnInfo,
                              LISTBROWSER_ColumnTitles,TRUE,
                              LISTBROWSER_TitleClickable,   TRUE,
                              LISTBROWSER_Separators,  TRUE,
                              LISTBROWSER_Hierarchical,TRUE,
                              LISTBROWSER_HorizontalProp,    TRUE,
                              LISTBROWSER_ShowSelected,TRUE,
                              LISTBROWSER_MultiSelect, TRUE,
                              LISTBROWSER_FastRender,  TRUE,
                              GA_ID,                   GAD_ADDRESSBOOK_LISTBROWSER,
                              GA_RelVerify,            TRUE,
                         EndMember,
                         StartVGroup,
                              StartMember,        control->Top = ButtonObject,
                                   GA_Image,           GetButtonImage(IMAGE_TOP),
                                   GA_ID,              GAD_ADDRESSBOOK_TOP,
                                   GA_RelVerify,       TRUE,
                                   GA_Disabled,        TRUE,
                              EndMember,
                              CHILD_WeightedHeight,    0,
                              StartMember,        control->Up = ButtonObject,
                                   GA_Image,           GetButtonImage(IMAGE_UP),
                                   GA_ID,              GAD_ADDRESSBOOK_UP,
                                   GA_RelVerify,       TRUE,
                                   GA_Disabled,        TRUE,
                              EndMember,
                              CHILD_WeightedHeight,    0,
                              StartMember,        control->Down = ButtonObject,
                                   GA_Image,           GetButtonImage(IMAGE_DOWN),
                                   GA_ID,              GAD_ADDRESSBOOK_DOWN,
                                   GA_RelVerify,       TRUE,
                                   GA_Disabled,        TRUE,
                              EndMember,
                              CHILD_WeightedHeight,    0,
                              StartMember,        control->Bottom = ButtonObject,
                                   GA_Image,           GetButtonImage(IMAGE_BOTTOM),
                                   GA_ID,              GAD_ADDRESSBOOK_BOTTOM,
                                   GA_RelVerify,       TRUE,
                                   GA_Disabled,        TRUE,
                              EndMember,
                              CHILD_WeightedHeight,    0,
                         EndGroup,
                         CHILD_WeightedWidth,     0,
                         StartVGroup,
                              StartImage,    BevelObject,
                                   BEVEL_Style,        BVS_SBAR_HORIZ,
                              EndImage,
                         EndGroup,
                         CHILD_WeightedWidth,     0,
                         StartVGroup,
                              LAYOUT_SpaceOuter,  FALSE,
                              StartMember,   control->Edit = ButtonObject,
                                   GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_EDIT),
                                   GA_ID,         GAD_ADDRESSBOOK_EDIT,
                                   GA_RelVerify,  TRUE,
                                   GA_Disabled,   TRUE,
                              EndMember,
                              StartHGroup,
                                   StartImage,    BevelObject,
                                        BEVEL_Style,   BVS_SBAR_VERT,
                                   EndImage,
                              EndGroup,
                              StartMember,   ButtonObject,
                                   GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_NEW),
                                   GA_ID,         GAD_ADDRESSBOOK_NEW,
                                   GA_RelVerify,  TRUE,
                              EndMember,
                              StartMember,   control->NewMember = ButtonObject,
                                   GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_NEWMEMBER),
                                   GA_ID,         GAD_ADDRESSBOOK_NEWMEMBER,
                                   GA_RelVerify,  TRUE,
                                   GA_Disabled,   TRUE,
                              EndMember,
                              StartMember,   control->Delete = ButtonObject,
                                   GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_DELETE),
                                   GA_ID,         GAD_ADDRESSBOOK_DELETE,
                                   GA_RelVerify,  TRUE,
                                   GA_Disabled,   TRUE,
                              EndMember,
                              StartMember,   control->Clear = ButtonObject,
                                   GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_CLEAR),
                                   GA_ID,         GAD_ADDRESSBOOK_CLEAR,
                                   GA_RelVerify,  TRUE,
                              EndMember,
                              StartHGroup,
                                   StartImage,    BevelObject,
                                        BEVEL_Style,   BVS_SBAR_VERT,
                                   EndImage,
                              EndGroup,
                              StartMember,   control->Sort = ButtonObject,
                                   GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_SORT),
                                   GA_ID,         GAD_ADDRESSBOOK_SORT,
                                   GA_RelVerify,  TRUE,
                              EndMember,
                              StartMember,   control->SortGroup = ButtonObject,
                                   GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_SORTGROUP),
                                   GA_ID,         GAD_ADDRESSBOOK_SORTGROUP,
                                   GA_RelVerify,  TRUE,
                                   GA_Disabled,   TRUE,
                              EndMember,
                              StartHGroup,
                                   StartImage,    BevelObject,
                                        BEVEL_Style,   BVS_SBAR_VERT,
                                   EndImage,
                              EndGroup,
                              StartMember,   control->MoveTo = ButtonObject,
                                   GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_MOVETO),
                                   GA_ID,         GAD_ADDRESSBOOK_MOVETO,
                                   GA_RelVerify,  TRUE,
                                   GA_Disabled,   TRUE,
                              EndMember,
                              StartMember,   control->CopyTo = ButtonObject,
                                   GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_COPYTO),
                                   GA_ID,         GAD_ADDRESSBOOK_COPYTO,
                                   GA_RelVerify,  TRUE,
                                   GA_Disabled,   TRUE,
                              EndMember,
                              StartHGroup,
                                   StartImage,    BevelObject,
                                        BEVEL_Style,   BVS_SBAR_VERT,
                                   EndImage,
                              EndGroup,
                              StartMember,   control->SendTo = ButtonObject,
                                   GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_SENDTO),
                                   GA_ID,         GAD_ADDRESSBOOK_SENDTO,
                                   GA_RelVerify,  TRUE,
                                   GA_Disabled,   TRUE,
                              EndMember,
                              StartHGroup,
                                   StartImage,    BevelObject,
                                        BEVEL_Style,   BVS_SBAR_VERT,
                                   EndImage,
                              EndGroup,
                              StartMember,   control->SelectAll = ButtonObject,
                                   GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_SELECTALL),
                                   GA_ID,         GAD_ADDRESSBOOK_SELECTALL,
                                   GA_RelVerify,  TRUE,
                              EndMember,
                              StartMember,   control->SelectNone = ButtonObject,
                                   GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_SELECTNONE),
                                   GA_ID,         GAD_ADDRESSBOOK_SELECTNONE,
                                   GA_RelVerify,  TRUE,
                              EndMember,
                         EndGroup,
                         CHILD_WeightedWidth,     0,
                         CHILD_WeightedHeight,    0,
                    EndGroup,
#ifdef EUCALYPTUS_PRO
                    StartHGroup,
                         StartMember,   control->PrintForm = GetFileObject,
                              GETFILE_TitleText,  LocaleString(ETXT_TITLE_SELECTFORM),
                              GETFILE_FullFile,   strlen(oprefs->AddressForm) ? (STRPTR)oprefs->AddressForm : (STRPTR)"PROGDIR:Forms/AddressBook",
                              GETFILE_ReadOnly,   FALSE,
                              GA_ID,              GAD_ADDRESSBOOK_PRINTFORM,
                              GA_RelVerify,       TRUE,
                              GA_Disabled,        TRUE,
                              ICA_TARGET,         ICTARGET_IDCMP,
                         EndMember,
                         MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_PRINTFORM)),
                         StartMember,   control->Print = ButtonObject,
                              GA_Text,       LocaleString(ETXT_GAD_ADDRESSBOOK_PRINT),
                              GA_ID,         GAD_ADDRESSBOOK_PRINT,
                              GA_RelVerify,  TRUE,
                              GA_Disabled,   TRUE,
                         EndMember,
                         CHILD_WeightedWidth,     20,
                    EndGroup,
                    CHILD_WeightedHeight,    0,
#endif
               End,
               TAG_END)))
          goto thatsit;

     /* open window
     if(!(control->Window = CA_OpenWindow(control->WindowO)))
          goto thatsit;

/*   Signal(euc.ThisProcess, SIG_HANDSHAKE);

     GetAttr(AREXX_SigMask, control->ARexxO, (ULONG *)&rxsig);

     while(!done)
     {
          LONG signals;

          signals = Wait((1L << control->IDCMPPort->mp_SigBit) |
                         (1L << control->AppPort->mp_SigBit) |
                         rxsig |
                         SIG_KILL |
                         SIG_SHOW |
                         SIG_CHECK |
                         SIG_UPDATE);

          if(signals & SIG_KILL)
          {
               Signal(FindTask(NULL), SIG_KILL);
               break;
          }

          if(signals & SIG_SHOW)
          {
               if(!control->Window)
               {
                    control->Window = CA_OpenWindow(control->WindowO);
               }
               else
                    WindowToFront(control->Window);
          }

          if(signals & SIG_UPDATE)
          {
               SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                         LISTBROWSER_Labels,      ~0,
                         TAG_END);

               BuildAddressBook(&control->Addresses, BABF_KEEPSEL | BABF_COLUMNS);

               if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                         LISTBROWSER_Labels,      &control->Addresses,
                         TAG_END))
                    if(control->Window)
                         RefreshGList(control->ListBrowser, control->Window, NULL, 1);
          }

          if(signals & SIG_CHECK)
          {
               AddressEntry *ae;

               ObtainSemaphore(&euc.Locks.NewAddrs);
               ae = (AddressEntry *)RemHead(&euc.NewAddrs);
               ReleaseSemaphore(&euc.Locks.NewAddrs);

               if(ae)
               {
                    if(!AddressInfo(control, ae))
                    {
                         ObtainSemaphore(&euc.Locks.Addresses);
                         AddTail(&euc.Addresses, (struct Node *)ae);
                         ReleaseSemaphore(&euc.Locks.Addresses);
                         Signal(euc.AddressBookProcess, SIG_UPDATE);
                    }
               }

               if(!IsListEmpty(&euc.NewAddrs))
                    Signal(euc.AddressBookProcess, SIG_CHECK);
          }

          if(signals & rxsig)
          {
               CA_HandleRexx(control->ARexxO);
          }

          if((signals & (1L << control->IDCMPPort->mp_SigBit)) ||
             (signals & (1L << control->AppPort->mp_SigBit)))
          {
               LONG result, code = 0;

               while(WMHI_LASTMSG != (result = CA_HandleInput(control->WindowO, code)))
               {
                    switch(result & WMHI_CLASSMASK)
                    {
                         case WMHI_CLOSEWINDOW:
                              done = TRUE;
                              break;
                         case WMHI_ICONIFY:
                              if(CA_Iconify(control->WindowO))
                              {
                                   control->Window = NULL;
                              }
                              break;
                         case WMHI_UNICONIFY:
                              if(control->Window = CA_OpenWindow(control->WindowO))
                              {
                              }
                              else
                              {
                                   DisplayBeep(euc.Screen);
                              }
                              break;
                         case WMHI_GADGETUP:
                              switch(result & WMHI_GADGETMASK)
                              {
                                   case GAD_ADDRESSBOOK_PRINTFORM:
                                        CA_RequestFile(control->PrintForm, control->Window);
                                        break;
                              }
                              break;
                    }
               }
          }
     }

     WriteAddresses(control->Window);

     thatsit:
     if(error)
     {
          IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_UNABLETOOPENADDRBOOK),
                    TAG_END);
     }

     if(control)
     {
          if(control->ARexxO)
               DisposeObject(control->ARexxO);

          if(control->WindowO)
               DisposeObject(control->WindowO);

          if(control->InfoWindowO)
               DisposeObject(control->InfoWindowO);

          if(control->IDCMPPort)
               DeleteMsgPort(control->IDCMPPort);

          if(control->AppPort)
               DeleteMsgPort(control->AppPort);

          while(node = RemHead(&control->Addresses))
               FreeListBrowserNode(node);

          FreeVec(control);
          euc.AddressBookControl = NULL;
     }

     /* clean up & signal
     Forbid();
     euc.AddressBookProcess = NULL;
     Signal(euc.ThisProcess, SIG_HANDSHAKE);
}

void __saveds __asm AddressBookIDCMP(register __a0 struct Hook *hook,
                                     register __a2 Object *o,
                                     register __a1 struct IntuiMessage *msg)
{
     AddressBookControl *control;

     if(!(control = euc.AddressBookControl))
          return;

     switch(msg->Class)
     {
          struct TagItem *tags;
          STRPTR text;

          case IDCMP_IDCMPUPDATE:
               tags = (struct TagItem *)msg->IAddress;
               while(tags->ti_Tag != TAG_END)
               {
                    BOOL moveme = FALSE;

                    switch(tags->ti_Tag)
                    {
                         case GA_ID:
                              text = NULL;
                              switch(tags->ti_Data)
                              {
                                   AddressEntry *ae, *aenew, *aeg, *aenext;
                                   struct Node *aemove;
                                   struct Composition *comp;
                                   LONG val;
                                   STRPTR text;
                                   BOOL asked;

                                   case GAD_ADDRESSBOOK_LISTBROWSER:
                                        SetMarkedAddresses();
                                        AdjustAddressButtons();
                                        break;
                                   case GAD_ADDRESSBOOK_TOP:
                                        if(control->NumSelected != 1)
                                             break;
                                        SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      ~0,
                                                  TAG_END);
                                        ObtainSemaphore(&euc.Locks.Addresses);
                                        for(ae = (AddressEntry *)euc.Addresses.lh_Head;
                                            ae->Node.ln_Succ;
                                            ae = (AddressEntry *)ae->Node.ln_Succ)
                                        {
                                             if(ae->Flags & AEF_MARKED)
                                             {
                                                  aeg = GetGroupHeader(ae);
                                                  Remove((struct Node *)ae);
                                                  if(aeg && aeg->Members)
                                                       AddHead(aeg->Members, (struct Node *)ae);
                                                  else
                                                       AddHead(&euc.Addresses, (struct Node *)ae);
                                                  break;
                                             }

                                             if(ae->Members)
                                             {
                                                  AddressEntry *aem;

                                                  for(aem = (AddressEntry *)ae->Members->lh_Head;
                                                      aem->Node.ln_Succ;
                                                      aem = (AddressEntry *)aem->Node.ln_Succ)
                                                  {
                                                       if(aem->Flags & AEF_MARKED)
                                                       {
                                                            aeg = GetGroupHeader(aem);
                                                            Remove((struct Node *)aem);
                                                            if(aeg && aeg->Members)
                                                                 AddHead(aeg->Members, (struct Node *)aem);
                                                            else
                                                                 AddHead(&euc.Addresses, (struct Node *)aem);
                                                            break;
                                                       }
                                                  }
                                             }
                                        }
                                        ReleaseSemaphore(&euc.Locks.Addresses);
                                        BuildAddressBook(&control->Addresses, BABF_KEEPSEL | BABF_COLUMNS);
                                        val = MatchAddressNum(control->Current);
                                        if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      &control->Addresses,
                                                  LISTBROWSER_MakeVisible, val,
                                                  TAG_END))
                                             if(control->Window)
                                                  RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                        break;
                                   case GAD_ADDRESSBOOK_UP:
                                        if(control->NumSelected != 1)
                                             break;
                                        SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      ~0,
                                                  TAG_END);
                                        ObtainSemaphore(&euc.Locks.Addresses);
                                        for(ae = (AddressEntry *)euc.Addresses.lh_Head;
                                            ae->Node.ln_Succ;
                                            ae = (AddressEntry *)ae->Node.ln_Succ)
                                        {
                                             if(ae->Flags & AEF_MARKED)
                                             {
                                                  if(ae->Flags & AEF_ISMEMBER)
                                                  {
                                                       aemove = ae->Node.ln_Pred;
                                                       if(aemove->ln_Pred)
                                                       {
                                                            Remove((struct Node *)ae);
                                                            Insert(ae->Members, (struct Node *)ae, aemove->ln_Pred);
                                                       }
                                                  }
                                                  else
                                                  {
                                                       aemove = ae->Node.ln_Pred;
                                                       if(aemove->ln_Pred)
                                                       {
                                                            Remove((struct Node *)ae);
                                                            Insert(&euc.Addresses, (struct Node *)ae, aemove->ln_Pred);
                                                       }
                                                  }

                                                  break;
                                             }

                                             if(ae->Members)
                                             {
                                                  AddressEntry *aem;

                                                  for(aem = (AddressEntry *)ae->Members->lh_Head;
                                                      aem->Node.ln_Succ;
                                                      aem = (AddressEntry *)aem->Node.ln_Succ)
                                                  {
                                                       if(aem->Flags & AEF_MARKED)
                                                       {
                                                            if(aem->Flags & AEF_ISMEMBER)
                                                            {
                                                                 aemove = aem->Node.ln_Pred;
                                                                 if(aemove->ln_Pred)
                                                                 {
                                                                      Remove((struct Node *)aem);
                                                                      Insert(aem->Members, (struct Node *)aem, aemove->ln_Pred);
                                                                 }
                                                            }
                                                            else
                                                            {
                                                                 aemove = aem->Node.ln_Pred;
                                                                 if(aemove->ln_Pred)
                                                                 {
                                                                      Remove((struct Node *)aem);
                                                                      Insert(&euc.Addresses, (struct Node *)aem, aemove->ln_Pred);
                                                                 }
                                                            }

                                                            break;
                                                       }
                                                  }
                                             }
                                        }
                                        ReleaseSemaphore(&euc.Locks.Addresses);
                                        BuildAddressBook(&control->Addresses, BABF_KEEPSEL | BABF_COLUMNS);
                                        val = MatchAddressNum(control->Current);
                                        if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      &control->Addresses,
                                                  LISTBROWSER_MakeVisible, val,
                                                  TAG_END))
                                             if(control->Window)
                                                  RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                        break;
                                   case GAD_ADDRESSBOOK_DOWN:
                                        if(control->NumSelected != 1)
                                             break;
                                        SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      ~0,
                                                  TAG_END);
                                        ObtainSemaphore(&euc.Locks.Addresses);
                                        for(ae = (AddressEntry *)euc.Addresses.lh_Head;
                                            ae->Node.ln_Succ;
                                            ae = (AddressEntry *)ae->Node.ln_Succ)
                                        {
                                             if(ae->Flags & AEF_MARKED)
                                             {
                                                  if(ae->Flags & AEF_ISMEMBER)
                                                  {
                                                       aemove = ae->Node.ln_Succ;
                                                       Remove((struct Node *)ae);
                                                       if(aemove->ln_Succ)
                                                            Insert(ae->Members, (struct Node *)ae, aemove);
                                                       else
                                                            AddTail(ae->Members, (struct Node *)ae);
                                                  }
                                                  else
                                                  {
                                                       aemove = ae->Node.ln_Succ;
                                                       Remove((struct Node *)ae);
                                                       if(aemove->ln_Succ)
                                                            Insert(&euc.Addresses, (struct Node *)ae, aemove);
                                                       else
                                                            AddTail(&euc.Addresses, (struct Node *)ae);
                                                  }

                                                  break;
                                             }

                                             if(ae->Members)
                                             {
                                                  AddressEntry *aem;

                                                  for(aem = (AddressEntry *)ae->Members->lh_Head;
                                                      aem->Node.ln_Succ;
                                                      aem = (AddressEntry *)aem->Node.ln_Succ)
                                                  {
                                                       if(aem->Flags & AEF_MARKED)
                                                       {
                                                            if(aem->Flags & AEF_ISMEMBER)
                                                            {
                                                                 aemove = aem->Node.ln_Succ;
                                                                 Remove((struct Node *)aem);
                                                                 if(aemove->ln_Succ)
                                                                      Insert(aem->Members, (struct Node *)aem, aemove);
                                                                 else
                                                                      AddTail(aem->Members, (struct Node *)aem);
                                                            }
                                                            else
                                                            {
                                                                 aemove = aem->Node.ln_Succ;
                                                                 Remove((struct Node *)aem);
                                                                 if(aemove->ln_Succ)
                                                                      Insert(&euc.Addresses, (struct Node *)aem, aemove);
                                                                 else
                                                                      AddTail(&euc.Addresses, (struct Node *)aem);
                                                            }

                                                            break;
                                                       }
                                                  }
                                             }
                                        }
                                        ReleaseSemaphore(&euc.Locks.Addresses);
                                        BuildAddressBook(&control->Addresses, BABF_KEEPSEL | BABF_COLUMNS);
                                        val = MatchAddressNum(control->Current);
                                        if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      &control->Addresses,
                                                  LISTBROWSER_MakeVisible, val,
                                                  TAG_END))
                                             if(control->Window)
                                                  RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                        break;
                                   case GAD_ADDRESSBOOK_BOTTOM:
                                        if(control->NumSelected != 1)
                                             break;
                                        SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      ~0,
                                                  TAG_END);
                                        ObtainSemaphore(&euc.Locks.Addresses);
                                        for(ae = (AddressEntry *)euc.Addresses.lh_Head;
                                            ae->Node.ln_Succ;
                                            ae = (AddressEntry *)ae->Node.ln_Succ)
                                        {
                                             if(ae->Flags & AEF_MARKED)
                                             {
                                                  aeg = GetGroupHeader(ae);
                                                  Remove((struct Node *)ae);
                                                  if(aeg)
                                                       AddTail(aeg->Members, (struct Node *)ae);
                                                  else
                                                       AddTail(&euc.Addresses, (struct Node *)ae);
                                                  break;
                                             }

                                             if(ae->Members)
                                             {
                                                  AddressEntry *aem;

                                                  for(aem = (AddressEntry *)ae->Members->lh_Head;
                                                      aem->Node.ln_Succ;
                                                      aem = (AddressEntry *)aem->Node.ln_Succ)
                                                  {
                                                       if(aem->Flags & AEF_MARKED)
                                                       {
                                                            aeg = GetGroupHeader(aem);
                                                            Remove((struct Node *)aem);
                                                            if(aeg)
                                                                 AddTail(aeg->Members, (struct Node *)aem);
                                                            else
                                                                 AddTail(&euc.Addresses, (struct Node *)aem);
                                                            break;
                                                       }
                                                  }
                                             }
                                        }
                                        ReleaseSemaphore(&euc.Locks.Addresses);
                                        BuildAddressBook(&control->Addresses, BABF_KEEPSEL | BABF_COLUMNS);
                                        val = MatchAddressNum(control->Current);
                                        if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      &control->Addresses,
                                                  LISTBROWSER_MakeVisible, val,
                                                  TAG_END))
                                             if(control->Window)
                                                  RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                        break;
                                   case GAD_ADDRESSBOOK_MOVETO:
                                        moveme = TRUE;
                                   case GAD_ADDRESSBOOK_COPYTO:
                                        if(!control->NumSelected)
                                             break;
                                        if(aeg = SelectAddressGroup())
                                        {
                                             AddressEntry *aenext;

                                             SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                       LISTBROWSER_Labels,      ~0,
                                                       TAG_END);
                                             ObtainSemaphore(&euc.Locks.Addresses);
                                             ae = (AddressEntry *)euc.Addresses.lh_Head;
                                             while(aenext = (AddressEntry *)ae->Node.ln_Succ)
                                             {
                                                  if(ae->Flags & AEF_MARKED)
                                                  {
                                                       CopyAddressEntry(ae, aeg, moveme);
                                                  }

                                                  if(ae->Members)
                                                  {
                                                       AddressEntry *aem, *aemnext;

                                                       aem = (AddressEntry *)ae->Members->lh_Head;
                                                       while(aemnext = (AddressEntry *)aem->Node.ln_Succ)
                                                       {
                                                            if(aem->Flags & AEF_MARKED)
                                                            {
                                                                 CopyAddressEntry(aem, aeg, moveme);
                                                            }

                                                            aem = aemnext;
                                                       }
                                                  }

                                                  ae = aenext;
                                             }
                                             ReleaseSemaphore(&euc.Locks.Addresses);
                                             BuildAddressBook(&control->Addresses, BABF_KEEPSEL | BABF_COLUMNS);
                                             val = MatchAddressNum(control->Current);
                                             if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                       LISTBROWSER_Labels,      &control->Addresses,
                                                       TAG_END))
                                                  if(control->Window)
                                                       RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                        }
                                        break;
                                   case GAD_ADDRESSBOOK_SENDTO:
                                        if(!control->NumSelected)
                                             break;
                                        if(comp = (struct Composition *)AllocVec(sizeof(struct Composition), MEMF_CLEAR | MEMF_PUBLIC))
                                        {
                                             InitComposition(comp);

                                             for(ae = (AddressEntry *)euc.Addresses.lh_Head;
                                                 ae->Node.ln_Succ;
                                                 ae = (AddressEntry *)ae->Node.ln_Succ)
                                             {
                                                  if(ae->Flags & AEF_MARKED)
                                                  {
                                                       UBYTE namebuf[101];

                                                       strncpy(namebuf, ae->FirstName, sizeof(namebuf) - 1);
                                                       strncat(namebuf, " ", sizeof(namebuf) - 1);
                                                       strncat(namebuf, ae->LastName, sizeof(namebuf) - 1);

                                                       AddCompAddress(comp, namebuf, ACAFT_NAME, CAT_TO);
                                                  }

                                                  if(ae->Members)
                                                  {
                                                       AddressEntry *aem;

                                                       for(aem = (AddressEntry *)ae->Members->lh_Head;
                                                           aem->Node.ln_Succ;
                                                           aem = (AddressEntry *)aem->Node.ln_Succ)
                                                       {
                                                            if(aem->Flags & AEF_MARKED)
                                                            {
                                                                 UBYTE namebuf[101];

                                                                 strncpy(namebuf, aem->FirstName, sizeof(namebuf) - 1);
                                                                 strncat(namebuf, " ", sizeof(namebuf) - 1);
                                                                 strncat(namebuf, aem->LastName, sizeof(namebuf) - 1);

                                                                 AddCompAddress(comp, namebuf, ACAFT_NAME, CAT_TO);
                                                            }
                                                       }
                                                  }
                                             }
                                             AddCompose(comp);
                                        }
                                        break;
                                   case GAD_ADDRESSBOOK_EDIT:
                                        if(!control->NumSelected)
                                             break;
                                        for(ae = (AddressEntry *)euc.Addresses.lh_Head;
                                            ae->Node.ln_Succ;
                                            ae = (AddressEntry *)ae->Node.ln_Succ)
                                        {
                                             if(ae->Flags & AEF_MARKED)
                                             {
                                                  AddressInfo(control, ae);
                                                  control->Current = ae;
                                             }

                                             if(ae->Members)
                                             {
                                                  AddressEntry *aem;

                                                  for(aem = (AddressEntry *)ae->Members->lh_Head;
                                                      aem->Node.ln_Succ;
                                                      aem = (AddressEntry *)aem->Node.ln_Succ)
                                                  {
                                                       if(aem->Flags & AEF_MARKED)
                                                       {
                                                            AddressInfo(control, aem);
                                                            control->Current = aem;
                                                       }
                                                  }
                                             }
                                        }

                                        SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      ~0,
                                                  TAG_END);
                                        if(oprefs->KeepAddressesSorted)
                                             SortAddresses(&euc.Addresses);
                                        HandleGroups(&euc.Addresses);
                                        BuildAddressBook(&control->Addresses, BABF_KEEPSEL | BABF_COLUMNS);
/*                                      val = MatchAddressNum(control->Current);
                                        if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      &control->Addresses,
/*                                                LISTBROWSER_MakeVisible, val,
                                                  TAG_END))
                                             if(control->Window)
                                                  RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                        AdjustAddressButtons();
                                        break;
                                   case GAD_ADDRESSBOOK_NEW:
                                        if(aenew = control->Current = (AddressEntry *)AllocVec(sizeof(AddressEntry), MEMF_CLEAR))
                                        {
                                             if(!AddressInfo(control, aenew))
                                             {
                                                  SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                            LISTBROWSER_Labels,      ~0,
                                                            TAG_END);
                                                  ObtainSemaphore(&euc.Locks.Addresses);
                                                  AddTail(&euc.Addresses, (struct Node *)aenew);
                                                  ReleaseSemaphore(&euc.Locks.Addresses);
                                                  if(oprefs->KeepAddressesSorted)
                                                       SortAddresses(&euc.Addresses);
                                                  HandleGroups(&euc.Addresses);
                                                  BuildAddressBook(&control->Addresses, BABF_COLUMNS);
/*                                                val = MatchAddressNum(aenew);
                                                  if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                            LISTBROWSER_Labels,      &control->Addresses,
/*                                                          LISTBROWSER_MakeVisible, val,
                                                            TAG_END))
                                                       if(control->Window)
                                                            RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                                  AdjustAddressButtons();
                                             }
                                        }
                                        break;
                                   case GAD_ADDRESSBOOK_NEWMEMBER:
                                        if(control->NumSelected != 1)
                                             break;
                                        if(!(control->Current = GetGroupHeader(control->Current)))
                                             break;
                                        if(!control->Current->Members)
                                        {
                                             if(!IsenRequest(euc.Screen,
                                                       ISENREQUEST_Text,        LocaleString(ETXT_NOTAGROUP),
                                                       ISENREQUEST_Button,      LocaleString(ETXT_REQ_YES_NO)))
                                                  break;

                                             if(!(control->Current->Members = (struct List *)AllocVec(sizeof(struct List), MEMF_CLEAR)))
                                             {
                                                  IsenRequest(euc.Screen,
                                                            ISENREQUEST_Text, LocaleString(ETXT_UNABLETOMAKEGROUP),
                                                            TAG_END);
                                                  break;
                                             }
                                        }
                                        if(aenew = (AddressEntry *)AllocVec(sizeof(AddressEntry), MEMF_CLEAR))
                                        {
                                             if(!AddressInfo(control, aenew))
                                             {
                                                  aenew->Flags |= AEF_ISMEMBER;
                                                  SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                            LISTBROWSER_Labels,      ~0,
                                                            TAG_END);
                                                  ObtainSemaphore(&euc.Locks.Addresses);
                                                  AddTail(control->Current->Members, (struct Node *)aenew);
                                                  ReleaseSemaphore(&euc.Locks.Addresses);
                                                  if(oprefs->KeepAddressesSorted)
                                                       SortAddresses(control->Current->Members);
                                                  HandleGroups(control->Current->Members);
                                                  BuildAddressBook(&control->Addresses, BABF_COLUMNS);
/*                                                val = MatchAddressNum(aenew);
                                                  if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                            LISTBROWSER_Labels,      &control->Addresses,
/*                                                          LISTBROWSER_MakeVisible, val,
                                                            TAG_END))
                                                       if(control->Window)
                                                            RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                                  control->Current = aenew;
                                                  AdjustAddressButtons();
                                             }
                                             else
                                             {
                                                  FreeAddressEntry(aenew);
                                             }
                                        }
                                        break;
                                   case GAD_ADDRESSBOOK_DELETE:
                                        asked = FALSE;
                                        if(!control->NumSelected)
                                             break;

                                        if(!IsenRequest(euc.Screen,
                                                  ISENREQUEST_Text,   LocaleString(ETXT_DELETEADDRESS),
                                                  ISENREQUEST_Button, LocaleString(ETXT_REQ_YES_NO),
                                                  TAG_END))
                                             break;

                                        SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      ~0,
                                                  TAG_END);
                                        ObtainSemaphore(&euc.Locks.Addresses);
                                        ae = (AddressEntry *)euc.Addresses.lh_Head;
                                        while(aenext = (AddressEntry *)ae->Node.ln_Succ)
                                        {
                                             if(ae->Flags & AEF_MARKED)
                                             {
                                                  if(ae->Members &&
                                                     !IsListEmpty(ae->Members))
                                                  {
                                                       if(!IsenRequest(euc.Screen,
                                                                 ISENREQUEST_Text,   LocaleString(ETXT_DELETEGROUP),
                                                                 ISENREQUEST_Button, LocaleString(ETXT_REQ_YES_NO),
                                                                 TAG_END))
                                                            break;
                                                  }

                                                  Remove(ae);
                                                  FreeAddressEntry(ae);
                                             }

                                             if(ae->Members)
                                             {
                                                  AddressEntry *aem, *aemnext;

                                                  aem = (AddressEntry *)ae->Members->lh_Head;
                                                  while(aemnext = (AddressEntry *)aem->Node.ln_Succ)
                                                  {
                                                       if(aem->Flags & AEF_MARKED)
                                                       {
                                                            if(aem->Members &&
                                                               !IsListEmpty(aem->Members))
                                                            {
                                                                 if(!IsenRequest(euc.Screen,
                                                                           ISENREQUEST_Text,   LocaleString(ETXT_DELETEGROUP),
                                                                           ISENREQUEST_Button, LocaleString(ETXT_REQ_YES_NO),
                                                                           TAG_END))
                                                                      break;
                                                            }

                                                            Remove(aem);
                                                            FreeAddressEntry(aem);
                                                       }

                                                       aem = aemnext;
                                                  }
                                             }

                                             ae = aenext;
                                        }
                                        ReleaseSemaphore(&euc.Locks.Addresses);
                                        BuildAddressBook(&control->Addresses, BABF_COLUMNS);
                                        if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      &control->Addresses,
                                                  LISTBROWSER_Selected,    ~0,
                                                  TAG_END))
                                             if(control->Window)
                                                  RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                        ClearMarkedAddresses();
                                        AdjustAddressButtons();
                                        break;
                                   case GAD_ADDRESSBOOK_SORT:
                                        SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      ~0,
                                                  TAG_END);
                                        ObtainSemaphore(&euc.Locks.Addresses);
                                        SortAddresses(&euc.Addresses);
                                        HandleGroups(&euc.Addresses);
                                        ReleaseSemaphore(&euc.Locks.Addresses);
                                        BuildAddressBook(&control->Addresses, BABF_KEEPSEL | BABF_COLUMNS);
                                        if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      &control->Addresses,
                                                  LISTBROWSER_Selected,    ~0,
                                                  TAG_END))
                                             if(control->Window)
                                                  RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                        ClearMarkedAddresses();
                                        AdjustAddressButtons();
                                        break;
                                   case GAD_ADDRESSBOOK_SORTGROUP:
                                        if(control->NumSelected != 1)
                                             break;
                                        if(control->Current->Flags & AEF_ISMEMBER)
                                             if(!(control->Current = GetGroupHeader(control->Current)))
                                                  break;
                                        SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      ~0,
                                                  TAG_END);
                                        ObtainSemaphore(&euc.Locks.Addresses);
                                        SortAddresses(control->Current->Members);
                                        HandleGroups(control->Current->Members);
                                        ReleaseSemaphore(&euc.Locks.Addresses);
                                        BuildAddressBook(&control->Addresses, BABF_KEEPSEL | BABF_COLUMNS);
                                        if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      &control->Addresses,
                                                  LISTBROWSER_Selected,    ~0,
                                                  TAG_END))
                                             if(control->Window)
                                                  RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                        ClearMarkedAddresses();
                                        AdjustAddressButtons();
                                        break;
                                   case GAD_ADDRESSBOOK_SELECTALL:
                                        SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      ~0,
                                                  TAG_END);
                                        ListBrowserSelectAll(&control->Addresses);
                                        if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      &control->Addresses,
                                                  LISTBROWSER_Selected,    ~0,
                                                  TAG_END))
                                             if(control->Window)
                                                  RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                        SetMarkedAddresses();
                                        break;
                                   case GAD_ADDRESSBOOK_SELECTNONE:
                                        if(SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Selected,    ~0,
                                                  TAG_END))
                                             if(control->Window)
                                                  RefreshGList(control->ListBrowser, control->Window, NULL, 1);
                                        ClearMarkedAddresses();
                                        break;
#ifdef EUCALYPTUS_PRO
                                   case GAD_ADDRESSBOOK_PRINT:
                                        GetAttr(GETFILE_FullFile, control->PrintForm, (ULONG *)&text);
                                        switch(IsenRequest(euc.Screen,
                                                  ISENREQUEST_Text,        LocaleString(ETXT_PRINTORSAVEADDRESSBOOK),
                                                  ISENREQUEST_Button,      LocaleString(ETXT_REQ_PRINT_SAVE_CANCEL),
                                                  TAG_END))
                                        {
                                             Object *getfile;
                                             STRPTR file;

                                             case 1:   /* print
                                                  PrintAddressBook("PRT:", text);
                                                  break;
                                             case 2:   /* save
                                                  if(getfile = NewObject(GETFILE_GetClass(), NULL,
                                                            GETFILE_TitleText,       LocaleString(ETXT_TITLE_SELECTTEXTFILE),
                                                            GETFILE_Drawer,          oprefs->DefaultSavePath,
                                                            GETFILE_DoSaveMode,      TRUE,
                                                            TAG_END))
                                                  {
                                                       if(CA_RequestFile(getfile, control->Window))
                                                       {
                                                            GetAttr(GETFILE_FullFile, getfile, (ULONG *)&file);

                                                            PrintAddressBook(file, text);
                                                       }

                                                       DisposeObject(getfile);
                                                  }
                                                  break;
                                        }
                                        break;
                                   case GAD_ADDRESSBOOK_PRINTFORM:
                                        break;
#endif

                                   case GAD_ADDRESSBOOK_ISGROUP:
                                        if(!control->Current)
                                             break;
                                        GetAttr(GA_Selected, control->IsGroup, (ULONG *)&val);
                                        SetGadgetAttrs(control->ListBrowser, control->Window, NULL,
                                                  LISTBROWSER_Labels,      ~0,
                                                  TAG_END);
                                        if(!control->Current->Members && val)
                                        {
                                             if(!(control->Current->Members = (struct List *)AllocVec(sizeof(struct List), MEMF_CLEAR)))
                                             {
                                                  IsenRequest(euc.Screen, ISENREQUEST_Text, LocaleString(ETXT_UNABLETOMAKEGROUP), NULL);
                                                  if(SetGadgetAttrs(control->IsGroup, control->Window, NULL,
                                                            GA_Selected,        FALSE,
                                                            TAG_END))
                                                       if(control->Window)
                                                            RefreshGList(control->IsGroup, control->Window, NULL, 1);
                                             }
                                             else
                                             {
                                                  NewList(control->Current->Members);
                                                  AdjustAddressButtons();
                                             }
                                        }
                                        else if(control->Current->Members && !val)
                                        {
                                             BOOL okay = TRUE;

                                             if(!IsListEmpty(control->Current->Members))
                                             {
                                                  switch(IsenRequest(euc.Screen,
                                                            ISENREQUEST_Text, LocaleString(ETXT_GROUPCONTAINSMEMBERS),
                                                            ISENREQUEST_Button, LocaleString(ETXT_REQ_CONTINUE_INDIV_CANCEL),
                                                            TAG_END))
                                                  {
                                                       AddressEntry *aem, *aei;

                                                       case 1:   /* obliterate
                                                            ObtainSemaphore(&euc.Locks.Addresses);
                                                            while(aem = (AddressEntry *)RemHead(control->Current->Members))
                                                                 FreeAddressEntry(aem);
                                                            ReleaseSemaphore(&euc.Locks.Addresses);
                                                            break;
                                                       case 2:
                                                            aei = control->Current;
                                                            ObtainSemaphore(&euc.Locks.Addresses);
                                                            while(aem = (AddressEntry *)RemHead(control->Current->Members))
                                                            {
                                                                 Insert(&euc.Addresses, aem, aei);
                                                                 aei = aem;
                                                            }
                                                            ReleaseSemaphore(&euc.Locks.Addresses);
                                                            break;
                                                       default:
                                                            okay = FALSE;
                                                            if(SetGadgetAttrs(control->IsGroup, control->Window, NULL,
                                                                      GA_Selected,        TRUE,
                                                                      TAG_END))
                                                                 if(control->Window)
                                                                      RefreshGList(control->IsGroup, control->Window, NULL, 1);
                                                            break;
                                                  }
                                             }

                                             if(okay)
                                             {
                                                  FreeVec(control->Current->Members);
                                                  control->Current->Members = NULL;
                                                  AdjustAddressButtons();
                                             }
                                        }
                                        BuildAddressBook(&control->Addresses, BABF_KEEPSEL | BABF_COLUMNS);
/*                                      val = MatchAddressNum(control->Current);
                                        if(SetPageGadgetAttrs(control->ListBrowser, (Object *)control->Page3, control->Window, NULL,
                                                  LISTBROWSER_Labels,      &control->Addresses,
/*                                                LISTBROWSER_MakeVisible, val,
                                                  TAG_END))
                                             if(control->Window)
                                                  RefreshPageGadget(control->ListBrowser, (Object *)control->Page3, control->Window, NULL);
                                        if(SetPageGadgetAttrs(control->IsML, (Object *)control->Page3, control->InfoWindow, NULL,
                                                  GA_Selected,             control->Current->Members ? (control->Current->Flags & AEF_ISML) : FALSE,
                                                  GA_Disabled,             !control->Current->Members,
                                                  TAG_END))
                                             if(control->Window)
                                                  RefreshPageGadget(control->IsML, (Object *)control->Page3, control->InfoWindow, NULL);
                                        break;
                                   case GAD_ADDRESSBOOK_ISML:
                                        if(!control->Current)
                                             break;
                                        if(!control->Current->Members)
                                             break;
                                        GetAttr(GA_Selected, control->IsML, (ULONG *)&val);
                                        if(val)
                                             control->Current->Flags |= AEF_ISML;
                                        else
                                             control->Current->Flags &= ~AEF_ISML;
                                        break;
                                   case GAD_ADDRESSBOOK_FORM:
                                        if(CA_RequestFile(control->Form, control->InfoWindow))
                                        {
                                             GetAttr(GETFILE_FullFile, control->Form, (ULONG *)&text);
                                             if(text)
                                                  strncpy(control->Current->Form, text, sizeof(control->Current->Form) - 1);
                                        }
                                        break;
                                   case GAD_ADDRESSBOOK_SIGNATURE:
                                        if(CA_RequestFile(control->SigFile, control->InfoWindow))
                                        {
                                             GetAttr(GETFILE_FullFile, control->SigFile, (ULONG *)&text);
                                             if(text)
                                                  strncpy(control->Current->SigFile, text, sizeof(control->Current->SigFile) - 1);
                                        }
                                        break;
                                   case GAD_ADDRESSBOOK_SIGN:
                                        if(!control->Current)
                                             break;
                                        GetAttr(GA_Selected, control->Sign, (ULONG *)&val);
                                        if(val)
                                             control->Current->Flags |= AEF_SIGN;
                                        else
                                             control->Current->Flags &= ~AEF_SIGN;
                                        break;
                                   case GAD_ADDRESSBOOK_ENCRYPT:
                                        if(!control->Current)
                                             break;
                                        GetAttr(GA_Selected, control->Encrypt, (ULONG *)&val);
                                        if(val)
                                             control->Current->Flags |= AEF_ENCRYPT;
                                        else
                                             control->Current->Flags &= ~AEF_ENCRYPT;
                                        break;
                              }
                              break;
                    }

                    tags++;
               }
               break;
     }
}

void AdjustAddressButtons(void)
{
     AddressBookControl *control;
     AddressEntry *ae;
     struct Node *node;

     if(!(control = euc.AddressBookControl))
          return;

     /* determine
     GetAttr(LISTBROWSER_NumSelected, control->ListBrowser, (ULONG *)&control->NumSelected);
     control->NumGroups       =
     control->NumMembers      = 0;
     ObtainSemaphore(&euc.Locks.Addresses);
     for(ae = (AddressEntry *)euc.Addresses.lh_Head;
         ae->Node.ln_Succ;
         ae = (AddressEntry *)ae->Node.ln_Succ)
     {
          AddressEntry *aem;
          LONG gen, flags;
          BOOL sel;

          if((ae->Flags & AEF_MARKED) &&
             ae->Members)
               control->NumGroups++;

          if(ae->Members)
          {
               for(aem = (AddressEntry *)ae->Members->lh_Head;
                   aem->Node.ln_Succ;
                   aem = (AddressEntry *)aem->Node.ln_Succ)
               {
                    if(aem->Flags & AEF_MARKED)
                         control->NumMembers++;
               }
          }
     }
     ReleaseSemaphore(&euc.Locks.Addresses);
/*
debugf("Selected: total: %ld; groups: %ld; members: %ld",
          control->NumSelected,
          control->NumGroups,
          control->NumMembers);
*/

     /* set the buttons
     if(SetGadgetAttrs(control->Top, control->Window, NULL,
               GA_Disabled,        !control->NumSelected,
               TAG_END))
          if(control->Window)
               RefreshGList(control->Top, control->Window, NULL, 1);
     if(SetGadgetAttrs(control->Up, control->Window, NULL,
               GA_Disabled,        !control->NumSelected,
               TAG_END))
          if(control->Window)
               RefreshGList(control->Up, control->Window, NULL, 1);
     if(SetGadgetAttrs(control->Down, control->Window, NULL,
               GA_Disabled,        !control->NumSelected,
               TAG_END))
          if(control->Window)
               RefreshGList(control->Down, control->Window, NULL, 1);
     if(SetGadgetAttrs(control->Bottom, control->Window, NULL,
               GA_Disabled,        !control->NumSelected,
               TAG_END))
          if(control->Window)
               RefreshGList(control->Bottom, control->Window, NULL, 1);
     if(SetGadgetAttrs(control->NewMember, control->Window, NULL,
               GA_Disabled,        (((control->NumGroups == 0) && (control->NumMembers > 0)) || ((control->NumGroups == 1) && (control->NumMembers == 0))) ? FALSE : TRUE,
               TAG_END))
          if(control->Window)
               RefreshGList(control->NewMember, control->Window, NULL, 1);
     if(SetGadgetAttrs(control->Delete, control->Window, NULL,
               GA_Disabled,        !control->NumSelected,
               TAG_END))
          if(control->Window)
               RefreshGList(control->Delete, control->Window, NULL, 1);
     if(SetGadgetAttrs(control->Edit, control->Window, NULL,
               GA_Disabled,        !control->NumSelected,
               TAG_END))
          if(control->Window)
               RefreshGList(control->Edit, control->Window, NULL, 1);
     if(SetGadgetAttrs(control->SortGroup, control->Window, NULL,
               GA_Disabled,        ((control->NumMembers > 0) || (control->NumGroups > 0)) ? FALSE : TRUE,
               TAG_END))
          if(control->Window)
               RefreshGList(control->SortGroup, control->Window, NULL, 1);
     if(SetGadgetAttrs(control->MoveTo, control->Window, NULL,
               GA_Disabled,        !control->NumSelected,
               TAG_END))
          if(control->Window)
               RefreshGList(control->MoveTo, control->Window, NULL, 1);
     if(SetGadgetAttrs(control->CopyTo, control->Window, NULL,
               GA_Disabled,        !control->NumSelected,
               TAG_END))
          if(control->Window)
               RefreshGList(control->CopyTo, control->Window, NULL, 1);
     if(SetGadgetAttrs(control->SendTo, control->Window, NULL,
               GA_Disabled,        !control->NumSelected,
               TAG_END))
          if(control->Window)
               RefreshGList(control->SendTo, control->Window, NULL, 1);
     if(SetGadgetAttrs(control->Print, control->Window, NULL,
               GA_Disabled,        !control->NumSelected,
               TAG_END))
          if(control->Window)
               RefreshGList(control->Print, control->Window, NULL, 1);
     if(SetGadgetAttrs((struct Gadget *)control->PrintForm, control->Window, NULL,
               GA_Disabled,        !control->NumSelected,
               TAG_END))
          if(control->Window)
               RefreshGList((struct Gadget *)control->PrintForm, control->Window, NULL, 1);
}

LONG BuildAddressBook(struct List *list, WORD flags)
{
     AddressEntry *ae;
     struct Node *node;
     LONG error = 0, i = 0;
     UBYTE namebuf[101], cols = 0;

     while(node = RemHead(list))
          FreeListBrowserNode(node);

     if(flags & BABF_COLUMNS)
     {
          i = 0;
          while(i < 5)
          {
               if(oprefs->AddressBookColumns[i] == 0xFF)
                    break;

               cols = ++i;
          }
     }

     if(!cols)
     {
          cols = 2;
          flags &= ~BABF_COLUMNS;
     }

     ObtainSemaphoreShared(&euc.Locks.Addresses);
     for(ae = (AddressEntry *)euc.Addresses.lh_Head;
         ae->Node.ln_Succ;
         ae = (AddressEntry *)ae->Node.ln_Succ)
     {
          switch(oprefs->AddressBookSort)
          {
               case ABST_FIRSTNAME:
               case ABST_ADDRESS:
                    strncpy(namebuf, ae->FirstName, sizeof(namebuf) - 1);
                    strncat(namebuf, " ", sizeof(namebuf) - 1);
                    strncat(namebuf, ae->LastName, sizeof(namebuf) - 1);
                    break;
               case ABST_LASTNAME:
                    strncpy(namebuf, ae->LastName, sizeof(namebuf) - 1);
                    if(*namebuf)
                         strncat(namebuf, ", ", sizeof(namebuf) - 1);
                    strncat(namebuf, ae->FirstName, sizeof(namebuf) - 1);
                    break;
          }

          if(node = AllocListBrowserNode(cols,
                    LBNA_Generation,    1,
                    LBNA_Selected,      (flags & BABF_KEEPSEL) ? (ae->Flags & AEF_MARKED) : FALSE,
                    LBNA_Flags,         ae->Members ? (LBFLG_HASCHILDREN | ((ae->Flags & AEF_HIDE) ? NULL : LBFLG_SHOWCHILDREN)) : NULL,
                    TAG_END))
          {
               if(flags & BABF_COLUMNS)
               {
                    for(i=0; i<cols; i++)
                    {
                         *namebuf = 0;
                         switch(oprefs->AddressBookColumns[i])
                         {
                              case ABCT_NAME:
                                   switch(oprefs->AddressBookSort)
                                   {
                                        case ABST_FIRSTNAME:
                                        case ABST_ADDRESS:
                                             strncpy(namebuf, ae->FirstName, sizeof(namebuf) - 1);
                                             strncat(namebuf, " ", sizeof(namebuf) - 1);
                                             strncat(namebuf, ae->LastName, sizeof(namebuf) - 1);
                                             break;
                                        case ABST_LASTNAME:
                                             strncpy(namebuf, ae->LastName, sizeof(namebuf) - 1);
                                             if(*namebuf)
                                                  strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                             strncat(namebuf, ae->FirstName, sizeof(namebuf) - 1);
                                             break;
                                   }
                                   break;
                              case ABCT_EMAIL:
                                   strncpy(namebuf, ae->Address, sizeof(namebuf) - 1);
                                   break;
                              case ABCT_ORG:
                                   strncpy(namebuf, ae->Organization, sizeof(namebuf) - 1);
                                   break;
                              case ABCT_TITLE:
                                   strncpy(namebuf, ae->Title, sizeof(namebuf) - 1);
                                   break;
                              case ABCT_ADDRESS:
                                   strncpy(namebuf, ae->StreetAddress1, sizeof(namebuf) - 1);
                                   strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                   strncat(namebuf, ae->StreetAddress2, sizeof(namebuf) - 1);
                                   strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                   strncat(namebuf, ae->City, sizeof(namebuf) - 1);
                                   strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                   strncat(namebuf, ae->State, sizeof(namebuf) - 1);
                                   strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                   strncat(namebuf, ae->ZIP, sizeof(namebuf) - 1);
                                   strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                   strncat(namebuf, ae->Country, sizeof(namebuf) - 1);
                                   break;
                              case ABCT_WORKPH:
                                   strncpy(namebuf, ae->WorkPhone, sizeof(namebuf) - 1);
                                   break;
                              case ABCT_HOMEPH:
                                   strncpy(namebuf, ae->HomePhone, sizeof(namebuf) - 1);
                                   break;
                              case ABCT_FAX:
                                   strncpy(namebuf, ae->Fax, sizeof(namebuf) - 1);
                                   break;
                              case ABCT_KEYID:
                                   strncpy(namebuf, ae->KeyID, sizeof(namebuf) - 1);
                                   break;
                              case ABCT_COMMENTS:
                                   strncpy(namebuf, ae->Comments, sizeof(namebuf) - 1);
                                   break;
                              case ABCT_NICK:
                                   strncpy(namebuf, ae->Nickname, sizeof(namebuf) - 1);
                                   break;
                              case ABCT_HOMEPAGE:
                                   strncpy(namebuf, ae->Homepage, sizeof(namebuf) - 1);
                                   break;
                         }

                         SetListBrowserNodeAttrs(node,
                                   LBNA_Column,        i,
                                        LBNCA_CopyText,     TRUE,
                                        LBNCA_Text,         namebuf,
                                        LBNCA_Justification,LCJ_LEFT,
                                   TAG_END);
                    }
               }
               else
               {
                    SetListBrowserNodeAttrs(node,
                              LBNA_Column,        0,
                                   LBNCA_CopyText,     TRUE,
                                   LBNCA_Text,         namebuf,
                                   LBNCA_Justification,LCJ_LEFT,
                              LBNA_Column,        1,
                                   LBNCA_CopyText,     TRUE,
                                   LBNCA_Text,         ae->Address,
                                   LBNCA_Justification,LCJ_LEFT,
                              TAG_END);
               }

               AddTail(list, node);

               if(ae->Members)
               {    /* it's a group
                    AddressEntry *aem;

                    ae->Count = 0;

                    for(aem = (AddressEntry *)ae->Members->lh_Head;
                        aem->Node.ln_Succ;
                        aem = (AddressEntry *)aem->Node.ln_Succ)
                    {
                         struct Node *cnode;

                         switch(oprefs->AddressBookSort)
                         {
                              case ABST_FIRSTNAME:
                              case ABST_ADDRESS:
                                   strncpy(namebuf, aem->FirstName, sizeof(namebuf) - 1);
                                   strncat(namebuf, " ", sizeof(namebuf) - 1);
                                   strncat(namebuf, aem->LastName, sizeof(namebuf) - 1);
                                   break;
                              case ABST_LASTNAME:
                                   strncpy(namebuf, aem->LastName, sizeof(namebuf) - 1);
                                   if(*namebuf)
                                        strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                   strncat(namebuf, aem->FirstName, sizeof(namebuf) - 1);
                                   break;
                         }

                         if(cnode = AllocListBrowserNode(cols,
                                   LBNA_Generation,    2,
                                   LBNA_Selected,      (flags & BABF_KEEPSEL) ? (aem->Flags & AEF_MARKED) : FALSE,
                                   TAG_END))
                         {
                              if(flags & BABF_COLUMNS)
                              {
                                   for(i=0; i<cols; i++)
                                   {
                                        *namebuf = 0;
                                        switch(oprefs->AddressBookColumns[i])
                                        {
                                             case ABCT_NAME:
                                                  switch(oprefs->AddressBookSort)
                                                  {
                                                       case ABST_FIRSTNAME:
                                                       case ABST_ADDRESS:
                                                            strncpy(namebuf, aem->FirstName, sizeof(namebuf) - 1);
                                                            strncat(namebuf, " ", sizeof(namebuf) - 1);
                                                            strncat(namebuf, aem->LastName, sizeof(namebuf) - 1);
                                                            break;
                                                       case ABST_LASTNAME:
                                                            strncpy(namebuf, aem->LastName, sizeof(namebuf) - 1);
                                                            if(*namebuf)
                                                                 strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                                            strncat(namebuf, aem->FirstName, sizeof(namebuf) - 1);
                                                            break;
                                                  }
                                                  break;
                                             case ABCT_EMAIL:
                                                  strncpy(namebuf, aem->Address, sizeof(namebuf) - 1);
                                                  break;
                                             case ABCT_ORG:
                                                  strncpy(namebuf, aem->Organization, sizeof(namebuf) - 1);
                                                  break;
                                             case ABCT_TITLE:
                                                  strncpy(namebuf, aem->Title, sizeof(namebuf) - 1);
                                                  break;
                                             case ABCT_ADDRESS:
                                                  strncpy(namebuf, aem->StreetAddress1, sizeof(namebuf) - 1);
                                                  strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                                  strncat(namebuf, aem->StreetAddress2, sizeof(namebuf) - 1);
                                                  strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                                  strncat(namebuf, aem->City, sizeof(namebuf) - 1);
                                                  strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                                  strncat(namebuf, aem->State, sizeof(namebuf) - 1);
                                                  strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                                  strncat(namebuf, aem->ZIP, sizeof(namebuf) - 1);
                                                  strncat(namebuf, ", ", sizeof(namebuf) - 1);
                                                  strncat(namebuf, aem->Country, sizeof(namebuf) - 1);
                                                  break;
                                             case ABCT_WORKPH:
                                                  strncpy(namebuf, aem->WorkPhone, sizeof(namebuf) - 1);
                                                  break;
                                             case ABCT_HOMEPH:
                                                  strncpy(namebuf, aem->HomePhone, sizeof(namebuf) - 1);
                                                  break;
                                             case ABCT_FAX:
                                                  strncpy(namebuf, aem->Fax, sizeof(namebuf) - 1);
                                                  break;
                                             case ABCT_KEYID:
                                                  strncpy(namebuf, aem->KeyID, sizeof(namebuf) - 1);
                                                  break;
                                             case ABCT_COMMENTS:
                                                  strncpy(namebuf, aem->Comments, sizeof(namebuf) - 1);
                                                  break;
                                             case ABCT_NICK:
                                                  strncpy(namebuf, aem->Nickname, sizeof(namebuf) - 1);
                                                  break;
                                             case ABCT_HOMEPAGE:
                                                  strncpy(namebuf, aem->Homepage, sizeof(namebuf) - 1);
                                                  break;
                                        }

                                        SetListBrowserNodeAttrs(cnode,
                                                  LBNA_Column,        i,
                                                       LBNCA_CopyText,     TRUE,
                                                       LBNCA_Text,         namebuf,
                                                       LBNCA_Justification,LCJ_LEFT,
                                                  TAG_END);
                                   }
                              }
                              else
                              {
                                   SetListBrowserNodeAttrs(cnode,
                                             LBNA_Column,        0,
                                                  LBNCA_CopyText,     TRUE,
                                                  LBNCA_Text,         namebuf,
                                                  LBNCA_Justification,LCJ_LEFT,
                                             LBNA_Column,        1,
                                                  LBNCA_CopyText,     TRUE,
                                                  LBNCA_Text,         aem->Address,
                                                  LBNCA_Justification,LCJ_LEFT,
                                             TAG_END);
                              }

                              AddTail(list, cnode);
                              ae->Count++;
                         }
                         else
                         {
                              IsenRequest(euc.Screen,
                                        ISENREQUEST_Text,        LocaleString(ETXT_UNABLETOCREATEENTRY),
                                        TAG_END);
                              error = err_INSUFFICIENTMEMORY;
                              goto thatsit;
                         }
                    }

                    if(!(flags & BABF_COLUMNS) ||
                       (cols >= 2))
                    {
                         sprintf(ae->Address, LocaleString(ETXT_ADDRESSBOOK_COUNT), ae->Count);
                         SetListBrowserNodeAttrs(node,
                                   LBNA_Column,        1,
                                        LBNCA_CopyText,     TRUE,
                                        LBNCA_Text,         ae->Address,
                                   TAG_END);
                    }

                    if(ae->Flags & AEF_HIDE)
                         HideListBrowserNodeChildren(node);
               }
          }
          else
          {
               IsenRequest(euc.Screen,
                         ISENREQUEST_Text,        LocaleString(ETXT_UNABLETOCREATEENTRY),
                         TAG_END);
               error = err_INSUFFICIENTMEMORY;
               goto thatsit;
          }
     }

     thatsit:
     ReleaseSemaphore(&euc.Locks.Addresses);

     return(error);
}

void FreeAddressBook(void)
{
     AddressEntry *ae;

     ObtainSemaphore(&euc.Locks.Addresses);
     while(ae = (AddressEntry *)RemHead(&euc.Addresses))
     {
          FreeAddressEntry(ae);
     }
     ReleaseSemaphore(&euc.Locks.Addresses);
}

void FreeAddressEntry(AddressEntry *ae)
{
     if(!ae)
          return;

     if(ae->Members)
     {
          AddressEntry *aem;

          while(aem = (AddressEntry *)RemHead(ae->Members))
               FreeAddressEntry(aem);

          FreeVec(ae->Members);
     }

     FreeVec(ae);
}

LONG InitAddressBook(void)
{
     LONG error = 0;

     if(euc.AddressBookProcess)
     {
          Signal(euc.AddressBookProcess, SIG_SHOW);
          return(0);
     }

     Forbid();
     euc.AddressBookProcess = CreateNewProcTags(
               NP_Entry,           AddressBookThread,
               NP_Name,            LocaleString(ETXT_PROCNAME_ADDRESSBOOK),
               NP_StackSize,       EUCALYPTUSSTACKSIZE,
               TAG_END);
     Permit();

     if(!euc.AddressBookProcess)
     {
          DisplayBeep(euc.Screen);
          return(err_UNABLETOCREATETHREAD);
     }

     return(error);
}

LONG WriteAddresses(struct Window *window)
{
     LONG error = 0;
     BPTR fp, dir;

     if(dir = CurrentDir(euc.lock))
     {
          if(!strlen(uprefs->AddressBook))
          {
               struct List *list;

               if(list = GetFiles(LocaleString(ETXT_TITLE_SELECTADDRESSBOOK), window,
                         ASLFR_InitialDrawer,     euc.WorkingDir,
                         ASLFR_InitialFile,       ".addresses",
                         TAG_END))
               {
                    strncpy(uprefs->AddressBook, list->lh_Head->ln_Name, sizeof(uprefs->AddressBook) - 1);

                    FreeFiles(list);
               }
          }

          ObtainSemaphoreShared(&euc.Locks.Addresses);
          if(fp = Open(uprefs->AddressBook, MODE_NEWFILE))
          {
               AddressEntry *ae;

               FPrintf(fp, "# $VER: .addresses 1.1 (21.6.98)\n");

               for(ae = (AddressEntry *)euc.Addresses.lh_Head;
                   ae->Node.ln_Succ;
                   ae = (AddressEntry *)ae->Node.ln_Succ)
               {
                    RemoveWhitespace(ae->FirstName, RWSF_ONLYCRLF);
                    RemoveQuotes(ae->FirstName);

                    if(ae->Members)
                    {
                         AddressEntry *aem;

                         FPrintf(fp, "%%Group %s\n", ae->FirstName);

                         FPrintf(fp, "Options ");
                         if(ae->Flags & AEF_HIDE)
                              FPrintf(fp, "Hide; ");
                         if(ae->Flags & AEF_ISML)
                              FPrintf(fp, "ML; ");
                         if(ae->Flags & AEF_ENCRYPT)
                              FPrintf(fp, "Encrypt; ");
                         if(ae->Flags & AEF_SIGN)
                              FPrintf(fp, "Sign; ");
                         FPrintf(fp, "\n");

                         FPrintf(fp, "Comments %s\n", ae->Comments);

                         if(!IsListEmpty(ae->Members))
                         {
                              BYTE s;

                              for(aem = (AddressEntry *)ae->Members->lh_Head, s = 0;
                                  aem->Node.ln_Succ;
                                  aem = (AddressEntry *)aem->Node.ln_Succ, s = 1)
                              {
                                   RemoveWhitespace(aem->FirstName, RWSF_ONLYCRLF);
                                   RemoveQuotes(aem->FirstName);
                                   RemoveWhitespace(aem->LastName, RWSF_ONLYCRLF);
                                   RemoveQuotes(aem->LastName);
                                   RemoveWhitespace(aem->Address, RWSF_ONLYCRLF);
                                   RemoveQuotes(aem->Address);
                                   RemoveWhitespace(aem->Comments, RWSF_ONLYCRLF);
                                   RemoveQuotes(aem->Comments);
                                   RemoveWhitespace(aem->KeyID, RWSF_ONLYCRLF);
                                   RemoveQuotes(aem->KeyID);

                                   WriteUserAddressEntry(fp, aem);
                              }
                         }
                         FPrintf(fp, "%%EndGroup\n");
                    }
                    else
                    {
                         RemoveWhitespace(ae->Address, RWSF_ONLYCRLF);
                         RemoveQuotes(ae->Address);
                         RemoveWhitespace(ae->Comments, RWSF_ONLYCRLF);
                         RemoveQuotes(ae->Comments);
                         RemoveWhitespace(ae->KeyID, RWSF_ONLYCRLF);
                         RemoveQuotes(ae->KeyID);
                         WriteUserAddressEntry(fp, ae);
                    }
               }

               Close(fp);
          }
          ReleaseSemaphore(&euc.Locks.Addresses);

          CurrentDir(dir);
     }

     return(error);
}

LONG WriteUserAddressEntry(BPTR fp, AddressEntry *ae)
{
     LONG error = 0;

     if(!fp ||
        !ae)
          return(0);

     FPrintf(fp, "%%User%s\n", "");
     FPrintf(fp, "FirstName %s\n", ae->FirstName);
     FPrintf(fp, "LastName %s\n", ae->LastName);
     FPrintf(fp, "E-mail %s\n", ae->Address);
     FPrintf(fp, "Organization\n", ae->Organization);
     FPrintf(fp, "Title %s\n", ae->Title);
     FPrintf(fp, "Nickname %s\n", ae->Nickname);
     FPrintf(fp, "Comments %s\n", ae->Comments);
     FPrintf(fp, "Address1 %s\n", ae->StreetAddress1);
     FPrintf(fp, "Address2 %s\n", ae->StreetAddress2);
     FPrintf(fp, "City %s\n", ae->City);
     FPrintf(fp, "State %s\n", ae->State);
     FPrintf(fp, "ZIP %s\n", ae->ZIP);
     FPrintf(fp, "Country %s\n", ae->Country);
     FPrintf(fp, "WorkPhone %s\n", ae->WorkPhone);
     FPrintf(fp, "HomePhone %s\n", ae->HomePhone);
     FPrintf(fp, "Fax %s\n", ae->Fax);
     FPrintf(fp, "Form %s\n", ae->Form);
     FPrintf(fp, "Signature %s\n", ae->SigFile);
     FPrintf(fp, "Options ");
     if(ae->Flags & AEF_ENCRYPT)
          FPrintf(fp, "Encrypt; ");
     if(ae->Flags & AEF_SIGN)
          FPrintf(fp, "Sign; ");
     FPrintf(fp, "\n");
     FPrintf(fp, "&\n");

     return(error);
}

LONG ReadAddresses(void)
{
     LONG error = 0;
     UBYTE *work;
     BPTR fp;

     ObtainSemaphore(&euc.Locks.Addresses);
     if(work = AllocVec(WORKLEN, MEMF_CLEAR))
     {
          if(fp = Open(uprefs->AddressBook, MODE_OLDFILE))
          {
               AddressEntry *ae = NULL, *aeg = NULL;
               struct List *list;
               LONG format = 0;
               BOOL usercontext = FALSE;

               list = &euc.Addresses;

               while(!error &&
                     FGets(fp, work, WORKLEN - 1))
               {
                    UBYTE *c, *n;

                    if(c = strchr(work, '\n'))
                         *c = '\0';
                    if(c = strchr(work, '\r'))
                         *c = '\0';

                    if(work[0] == '#')
                    {
                         if(strstr(work, "$VER:") &&
                            strstr(work, " 1.1 "))
                              format = 1;
                         continue;
                    }

                    if(!ae)
                         ae = (AddressEntry *)AllocVec(sizeof(AddressEntry), MEMF_CLEAR | MEMF_PUBLIC);

                    if(!ae)
                         continue;

                    switch(format)
                    {
                         UBYTE *data, *c;

                         case 1:   /* 1.1 */
                              data = NULL;
                              if(c = strchr(work, ' '))
                              {
                                   *c = '\0';
                                   data = c + 1;
                              }

                              if(work[0] == '%')
                              {
                                   if(!stricmp(work + 1, "Group") && !aeg)
                                   {
                                        if(data)
                                        {
                                             strncpy(ae->FirstName, data, sizeof(ae->FirstName) - 1);
                                        }

                                        aeg = ae;
                                        ae = NULL;
                                        if(!(list = aeg->Members = (struct List *)AllocVec(sizeof(struct List), MEMF_CLEAR)))
                                        {
                                             error = err_INSUFFICIENTMEMORY;
                                             break;
                                        }
                                        NewList(aeg->Members);
                                   }
                                   else if(!stricmp(work + 1, "EndGroup"))
                                   {
                                        AddTail(&euc.Addresses, (struct Node *)aeg);

                                        if(usercontext && ae)
                                        {
                                             AddTail(list, (struct Node *)ae);
                                        }
                                        aeg = ae = NULL;
                                        usercontext = FALSE;
                                        list = &euc.Addresses;
                                   }
                                   else if(!stricmp(work + 1, "User"))
                                   {
                                        usercontext = TRUE;
                                   }
                              }
                              else if(work[0] == '&')
                              {
                                   AddTail(list, (struct Node *)ae);
                                   ae = NULL;
                                   usercontext = FALSE;
                              }
                              else
                              {
                                   if(usercontext && ae && data)
                                   {
                                        if(!stricmp(work, "Name"))
                                        {
                                             UBYTE *c;

                                             if(c = strrchr(data, ' '))
                                             {
                                                  *c = '\0';
                                                  strncpy(ae->FirstName, data, sizeof(ae->FirstName) - 1);
                                                  strncpy(ae->LastName, c + 1, sizeof(ae->LastName) - 1);
                                             }
                                             else
                                                  strncpy(ae->FirstName, data, sizeof(ae->FirstName) - 1);
                                        }
                                        else if(!stricmp(work, "FirstName"))
                                             strncpy(ae->FirstName, data, sizeof(ae->FirstName) - 1);
                                        else if(!stricmp(work, "LastName"))
                                             strncpy(ae->LastName, data, sizeof(ae->LastName) - 1);
                                        else if(!stricmp(work, "E-mail"))
                                             strncpy(ae->Address, data, sizeof(ae->Address) - 1);
                                        else if(!stricmp(work, "Organization"))
                                             strncpy(ae->Organization, data, sizeof(ae->Organization) - 1);
                                        else if(!stricmp(work, "Title"))
                                             strncpy(ae->Title, data, sizeof(ae->Title) - 1);
                                        else if(!stricmp(work, "Nickname"))
                                             strncpy(ae->Nickname, data, sizeof(ae->Nickname) - 1);
                                        else if(!stricmp(work, "Comments"))
                                             strncpy(ae->Comments, data, sizeof(ae->Comments) - 1);
                                        else if(!stricmp(work, "Address1"))
                                             strncpy(ae->StreetAddress1, data, sizeof(ae->StreetAddress1) - 1);
                                        else if(!stricmp(work, "Address2"))
                                             strncpy(ae->StreetAddress2, data, sizeof(ae->StreetAddress2) - 1);
                                        else if(!stricmp(work, "City"))
                                             strncpy(ae->City, data, sizeof(ae->City) - 1);
                                        else if(!stricmp(work, "State"))
                                             strncpy(ae->State, data, sizeof(ae->State) - 1);
                                        else if(!stricmp(work, "ZIP"))
                                             strncpy(ae->ZIP, data, sizeof(ae->ZIP) - 1);
                                        else if(!stricmp(work, "Country"))
                                             strncpy(ae->Country, data, sizeof(ae->Country) - 1);
                                        else if(!stricmp(work, "WorkPhone"))
                                             strncpy(ae->WorkPhone, data, sizeof(ae->WorkPhone) - 1);
                                        else if(!stricmp(work, "HomePhone"))
                                             strncpy(ae->HomePhone, data, sizeof(ae->HomePhone) - 1);
                                        else if(!stricmp(work, "Fax"))
                                             strncpy(ae->Fax, data, sizeof(ae->Fax) - 1);
                                        else if(!stricmp(work, "Form"))
                                             strncpy(ae->Form, data, sizeof(ae->Form) - 1);
                                        else if(!stricmp(work, "Signature"))
                                             strncpy(ae->SigFile, data, sizeof(ae->SigFile) - 1);
                                        else if(!stricmp(work, "Options"))
                                        {
                                             if(strstr(data, "Hide;"))
                                                  ae->Flags |= AEF_HIDE;
                                             if(strstr(data, "ML;"))
                                                  ae->Flags |= AEF_ISML;
                                             if(strstr(data, "Sign;"))
                                                  ae->Flags |= AEF_SIGN;
                                             if(strstr(data, "Encrypt;"))
                                                  ae->Flags |= AEF_ENCRYPT;
                                        }
                                   }
                                   else
                                   {
                                        if(!stricmp(work, "Comments"))
                                        {
                                             if(aeg)
                                                  strncpy(aeg->Comments, data, sizeof(aeg->Comments) - 1);
                                        }
                                        else if(!stricmp(work, "Options"))
                                        {
                                             if(aeg)
                                             {
                                                  if(strstr(data, "Hide;"))
                                                       aeg->Flags |= AEF_HIDE;
                                                  if(strstr(data, "ML;"))
                                                       aeg->Flags |= AEF_ISML;
                                                  if(strstr(data, "Sign;"))
                                                       aeg->Flags |= AEF_SIGN;
                                                  if(strstr(data, "Encrypt;"))
                                                       aeg->Flags |= AEF_ENCRYPT;
                                             }
                                        }
                                   }
                              }
                              break;
                         default:  /* 1.0 -- */
                              n = work;
                              if(c = strchr(n, ':'))
                              {
                                   *c = '\0';
                                   strncpy(ae->FirstName, n, sizeof(ae->FirstName) - 1);
                                   n = c + 1;
                              }

                              while((*n == ' ') ||
                                    (*n == '\t'))
                                   n++;

                              if(*n == '{')
                              {
                                   AddressEntry *aem;

                                   if(!(ae->Members = (struct List *)AllocVec(sizeof(struct List), MEMF_CLEAR)))
                                   {
                                        error = err_INSUFFICIENTMEMORY;
                                        break;
                                   }
                                   NewList(ae->Members);

                                   /* process group directives
                                   if(strstr(n + 1, "Hide;"))
                                        ae->Flags |= AEF_HIDE;
                                   else
                                        ae->Flags &= ~AEF_HIDE;

                                   if(strstr(n + 1, "ML;"))
                                        ae->Flags |= AEF_ISML;
                                   else
                                        ae->Flags &= ~AEF_ISML;

                                   while(FGets(fp, work, WORKLEN - 1))
                                   {
                                        UBYTE *b, *c, *e;

                                        if(work[0] == '}')
                                             break;

                                        n = work;
                                        while((*n == ' ') ||
                                              (*n == '\t'))
                                             n++;

                                        if(!strlen(n))
                                             continue;

                                        if(c = strchr(n, ':'))
                                        {
                                             if(!(aem = (AddressEntry *)AllocVec(sizeof(AddressEntry), MEMF_CLEAR)))
                                             {
                                                  error = err_INSUFFICIENTMEMORY;
                                                  break;
                                             }

                                             *c = '\0';
                                             strncpy(aem->FirstName, n, sizeof(aem->FirstName) - 1);
                                             RemoveWhitespace(aem->FirstName, 0);
                                             n = c + 1;

                                             if(b = strchr(n, ';'))
                                             {
                                                  *b = '\0';
                                                  strncpy(aem->Address, n + 1, sizeof(aem->Address) - 1);
                                                  RemoveWhitespace(aem->Address, 0);

                                                  if(e = strchr(b + 1, '\"'))
                                                  {
                                                       UBYTE *ee;

                                                       *e = '\0';
                                                       if(ee = strchr(e + 1, '\"'))
                                                       {    /* get key ID */
                                                            *ee = '\0';
                                                            strncpy(aem->KeyID, ee + 1, sizeof(aem->KeyID) - 1);
                                                            RemoveWhitespace(aem->KeyID, 0);
                                                       }

                                                       strncpy(aem->Comments, e + 1, sizeof(aem->Comments) - 1);
                                                  }
                                             }
                                        }

                                        aem->Flags |= AEF_ISMEMBER;
                                        AddTail(ae->Members, (struct Node *)aem);
                                   }

                                   if(oprefs->KeepAddressesSorted)
                                        SortAddresses(ae->Members);
                                   HandleGroups(ae->Members);

                                   n = c + 1;
                              }

                              if(c = strchr(n, ';'))
                              {
                                   *c = '\0';
                                   strncpy(ae->Address, n, sizeof(ae->Address) - 1);
                                   n = c + 1;
                              }

                              if(c = strchr(n, '\"'))
                              {
                                   UBYTE *e;

                                   if(e = strchr(c + 1, '\"'))
                                   {
                                        *e = '\0';
                                        strncpy(ae->KeyID, e + 1, sizeof(ae->KeyID) - 1);
                                        RemoveWhitespace(ae->KeyID, 0);
                                   }

                                   strncpy(ae->Comments, c + 1, sizeof(ae->Comments) - 1);
                              }

                              AddTail(&euc.Addresses, (struct Node *)ae);
                              ae = NULL;
                              break;
                    }
               }

               Close(fp);

               if(ae)
                    FreeVec(ae);
               if(aeg)
                    FreeVec(aeg);
          }

          FreeVec(work);
     }

     if(oprefs->KeepAddressesSorted)
          SortAddresses(&euc.Addresses);
     HandleGroups(&euc.Addresses);

     ReleaseSemaphore(&euc.Locks.Addresses);

     return(error);
}

#endif

