/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: addressbook_support.c
 *         Support functions for the address book
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>

#include "eucalyptus.h"

#if 0
LONG SortAddresses(struct List *list)
{
     AddressEntry *ae;
     LONG error = 0, n, i, j;

     if(IsListEmpty(list))
          return(0);

     // count the nodes
     for(ae = (AddressEntry *)list->lh_Head, n = 1;
         ae->Node.ln_Succ;
         ae = (AddressEntry *)ae->Node.ln_Succ, n++)
          ;

     if(n < 2)
          return(0);

     /* sort 'em */
     for(i=0; i<n; i++)
     {
          BOOL flag;

          flag = FALSE;

          for(j=0; j<(n-i); j++)
          {
               AddressEntry *ae1, *ae2;

               if(!(ae1 = (AddressEntry *)GetListNum(list, j)))
                    continue;
               if(!(ae2 = (AddressEntry *)GetListNum(list, j + 1)))
                    continue;

               switch(oprefs->AddressBookSort)
               {
                    case ABST_FIRSTNAME:
                         if(stricmp(ae1->FirstName, ae2->FirstName) > 0)
                         {    /* swap 'em */
                              Remove((struct Node *)ae1);
                              Insert(list, ae1, ae2);
                         }
                         break;
                    case ABST_LASTNAME:
                         if(stricmp(ae1->LastName, ae2->LastName) > 0)
                         {    /* swap 'em */
                              Remove((struct Node *)ae1);
                              Insert(list, ae1, ae2);
                         }
                         break;
                    case ABST_ADDRESS:
                         if(stricmp(ae1->Address, ae2->Address) > 0)
                         {    /* swap 'em */
                              Remove((struct Node *)ae1);
                              Insert(list, ae1, ae2);
                         }
                         break;
               }
          }
     }

     return(error);
}

LONG HandleGroups(struct List *list)
{
     AddressEntry *ae, *aen;
     struct List newlist;
     LONG error = 0, n, i, j;

     if(IsListEmpty(list))
          return(0);

     if(oprefs->SortGroupsAs == SGAT_MIXED)
          return(0);

     // init a temp list
     NewList(&newlist);

     // remove any groups
     ae = (AddressEntry *)list->lh_Head;
     while(aen = (AddressEntry *)ae->Node.ln_Succ)
     {
          if(ae->Members)     /* it's a group */
          {
               Remove((struct Node *)ae);
               AddTail(&newlist, (struct Node *)ae);
          }

          ae = aen;
     }

     // place the groups at the right place
     while(!IsListEmpty(&newlist))
     {
          switch(oprefs->SortGroupsAs)
          {
               case SGAT_FIRST:
                    if(ae = (AddressEntry *)RemTail(&newlist))
                         AddHead(list, ae);
                    break;
               case SGAT_LAST:
                    if(ae = (AddressEntry *)RemHead(&newlist))
                         AddTail(list, ae);
                    break;
          }
     }

     return(error);
}

void GetMsgAddress(struct MailMsg *msg, AddressEntry *aeg, BOOL autoadd)
{
     struct List *list = NULL;
     struct HdrNode *node = NULL, *rnode, *fnode;
     BOOL ourown = FALSE;

     if(!(list = msg->Headers))
     {
          struct AsyncFile *fp;

          if(fp = OpenAsync(msg->Index.Filename, MODE_READ, ASYNCIO_BUF, SysBase, DOSBase))
          {
               list = GetMsgHeader(GMHT_ASYNCIO, (APTR)fp, NULL);
               ourown = TRUE;
               CloseAsync(fp);
          }
     }

     if(!list)
          return;

     node = fnode = FindHeader("From", list, 0);
     if(rnode = FindHeader("Reply-To", list, 0))
          node = rnode;

     if(fnode && rnode)
     {
          node = autoadd ? rnode : fnode;
          if((stricmp(fnode->Data, rnode->Data) &&
              !strstr(fnode->Data, rnode->Data)) &&
             !autoadd)
          {
               UBYTE work[301];

               sprintf(work, LocaleString(ETXT_GETMSGADDRESS_FROMREPLYTO), fnode->Data, rnode->Data);
               switch(IsenRequest(euc.Screen,
                         ISENREQUEST_Text,        work,
                         ISENREQUEST_Button,      LocaleString(ETXT_REQ_FROM_REPLYTO_CANCEL),
                         TAG_END))
               {
                    case 1:   // from
                         node = fnode;
                         break;
                    case 2:   // reply-to
                         node = rnode;
                         break;
                    default:
                         node = NULL;
                         goto out;
               }
          }
     }

     if(node)
     {
          AddressEntry *ae, *aem;
          UBYTE name[101];

          if(ae = (AddressEntry *)AllocVec(sizeof(AddressEntry), MEMF_CLEAR))
          {
               UBYTE *c;

               ParseAddress(node->Data, name, sizeof(name) - 1, ae->Address, sizeof(ae->Address) - 1, FALSE);
               if(c = strrchr(name, ' '))
               {
                    *c = '\0';
                    strncpy(ae->FirstName, name, sizeof(ae->FirstName) - 1);
                    strncpy(ae->LastName, c + 1, sizeof(ae->LastName) - 1);
               }
               else
                    strncpy(ae->FirstName, name, sizeof(ae->FirstName) - 1);

               if(autoadd)
               {
                    if(aem = CheckDuplicateAddress(ae->Address))
                         goto out;

                    ObtainSemaphore(&euc.Locks.Addresses);
                    if(aeg == AEG_ROOT)
{
D(bug("Add to root address list\n"));
                         AddTail(&euc.Addresses, (struct Node *)ae);
}
                    else
{
D(bug("Add to group: %s\n", aeg->FirstName));
                         AddTail(aeg->Members, (struct Node *)ae);
}
                    ReleaseSemaphore(&euc.Locks.Addresses);
                    if(euc.AddressBookProcess)
                         Signal(euc.AddressBookProcess, SIG_UPDATE);
               }
               else
               {
                    if(aem = CheckDuplicateAddress(ae->Address))
                    {
                         UBYTE work[201];

                         sprintf(work, LocaleString(ETXT_DUPLICATEADDRESS), ae->Address);
                         switch(IsenRequest(euc.Screen,
                                   ISENREQUEST_Text,        work,
                                   ISENREQUEST_Button,      LocaleString(ETXT_REQ_REPLACE_ADD_CANCEL),
                                   TAG_END))
                         {
                              case 1:   // replace
                                   ObtainSemaphore(&euc.Locks.Addresses);
                                   Remove((struct Node *)aem);
                                   if(aeg == AEG_ROOT)
{
D(bug("Add to root address list\n"));
                                        AddTail(&euc.Addresses, (struct Node *)ae);
}
                                   else
{
D(bug("Add to group: %s\n", aeg->FirstName));
                                        AddTail(aeg->Members, (struct Node *)ae);
}
                                   ReleaseSemaphore(&euc.Locks.Addresses);
                                   if(euc.AddressBookProcess)
                                        Signal(euc.AddressBookProcess, SIG_UPDATE);
                                   break;
                              case 2:   // add
                                   AddNewAddress(ae);
                                   break;
                         }

                         goto out;
                    }

                    AddNewAddress(ae);
               }
          }
          else
          {    // insufficient memory
               IsenRequest(euc.Screen,
                         ISENREQUEST_Text,        LocaleString(ETXT_INSUFFICIENTMEMORY),
                         TAG_END);
          }
     }
     else
     {    // unable to find address
          IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_UNABLETOFINDADDRESS),
                    TAG_END);
     }

     out:
     if(list && ourown)
          FreeMsgHeader(list);
}

AddressEntry *CheckDuplicateAddress(UBYTE *address)
{
     AddressEntry *ae, *aematch = NULL;
     LONG count = 0;

     if(!address)
          return(NULL);

     ObtainSemaphore(&euc.Locks.Addresses);
     for(ae = (AddressEntry *)euc.Addresses.lh_Head;
         ae->Node.ln_Succ;
         ae = (AddressEntry *)ae->Node.ln_Succ)
     {
          if(!stricmp(ae->Address, address))
          {
               count++;
               aematch = ae;
               break;
          }

          if(ae->Members)
          {
               AddressEntry *aem;

               for(aem = (AddressEntry *)ae->Members->lh_Head;
                   aem->Node.ln_Succ;
                   aem = (AddressEntry *)aem->Node.ln_Succ)
               {
                    if(!stricmp(aem->Address, address))
                    {
                         count++;
                         aematch = aem;
                         break;
                    }
               }
          }
     }
     ReleaseSemaphore(&euc.Locks.Addresses);

     return(aematch);
}

void PrepCompose(struct Composition *comp, AddressEntry *ae)
{
     UBYTE name[101];

     if(!comp ||
        !ae)
          return;

     if(strlen(ae->Form))
          strncpy(comp->FormFile, ae->Form, sizeof(comp->FormFile) - 1);
     if(strlen(ae->SigFile))
          strncpy(comp->SignatureFile, ae->SigFile, sizeof(comp->SignatureFile) - 1);

     if(ae->Flags & AEF_ENCRYPT)
          comp->Flags |= CF_ENCRYPT;
     if(ae->Flags & AEF_SIGN)
          comp->Flags |= CF_SIGN;

     comp->MsgFormat = ae->MailFormat;

     strncpy(name, ae->FirstName, sizeof(name) - 1);
     strncat(name, " ", sizeof(name) - 1);
     strncat(name, ae->LastName, sizeof(name) - 1);
     AddCompAddress(comp, name, ACAFT_NAME, CAT_TO);
}

LONG AddressInfo(AddressBookControl *control, AddressEntry *ae)
{
     struct Gadget *layout;
     struct Node *node;
     static struct TagItem tab2page[] =
     {
          {CLICKTAB_Current, PAGE_Current,},
          {TAG_END,},
     };
     LONG error = 0, x, TabIDs[] = {ETXT_TAB_NAME, ETXT_TAB_CONTACT, ETXT_TAB_MISC, -1,};
     BOOL done = FALSE;

     if(!control)
          return(err_UNABLETOOPENWINDOW);

     if(!control->InfoWindowO)
     {
          NewList(&control->TabList);
          for(x=0; TabIDs[x] != -1; x++)
          {
               struct Node *node;

               if(node = AllocClickTabNode(
                         TNA_Text,      LocaleString(TabIDs[x]),
                         TNA_Number,    x,
                         TAG_END))
               {
                    AddTail(&control->TabList, node);
               }
               else
                    return(err_UNABLETOOPENWINDOW);
          }

          if(!(control->InfoWindowO = NewObject(WINDOW_GetClass(), NULL,
                    WINDOW_Position,         WPOS_CENTERSCREEN,

                    WA_Activate,             TRUE,
                    WA_CloseGadget,          TRUE,
                    WA_DragBar,              TRUE,
                    WA_DepthGadget,          TRUE,
                    WA_SizeGadget,           TRUE,
                    WA_SizeBRight,           TRUE,
                    WA_RMBTrap,              TRUE,
                    WINDOW_BackFillName,     sprefs->Backfill,

                    WINDOW_SharedPort,       control->IDCMPPort,
                    WINDOW_IDCMPHook,        &control->IDCMPHook,
                    WINDOW_IDCMPHookBits,    IDCMP_IDCMPUPDATE,
                    WA_IDCMP,                IDCMP_RAWKEY,

                    WA_PubScreen,            euc.Screen,
                    WA_Title,                LocaleString(ETXT_TITLE_ADDRESSINFO),
                    WA_ScreenTitle,          eucsctitle,

                    WINDOW_Layout,           layout = VGroupObject,
                         LAYOUT_DeferLayout, TRUE,
                         StartMember,   SpaceObject,
                              SPACE_Transparent,  TRUE,
                         EndMember,
                         CHILD_WeightedHeight,    0,
                         StartMember,   control->Tabs = ClickTabObject,
                              CLICKTAB_Labels,         &control->TabList,
                              CLICKTAB_Current,        0,
                              ICA_MAP,                 tab2page,
                         EndMember,
                         CHILD_WeightedHeight,    0,
                         StartMember,   control->Pager = PageObject,
                              PAGE_Add, control->Page1 = VGroupObject,  // name
                                   LAYOUT_SpaceOuter,  TRUE,
                                   LAYOUT_VertAlignment,    LALIGN_TOP,
                                   LAYOUT_HorizAlignment,   LALIGN_CENTER,
                                   StartMember,        control->FirstName = StringObject,
                                        STRINGA_TextVal,    ae->FirstName,
                                        STRINGA_MaxChars,   sizeof(ae->FirstName) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_FIRSTNAME,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_FIRSTNAME)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->LastName = StringObject,
                                        STRINGA_TextVal,    ae->LastName,
                                        STRINGA_MaxChars,   sizeof(ae->LastName) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_LASTNAME,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_LASTNAME)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->Organization = StringObject,
                                        STRINGA_TextVal,    ae->Organization,
                                        STRINGA_MaxChars,   sizeof(ae->Organization) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_ORGANIZATION,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_ORGANIZATION)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->Title = StringObject,
                                        STRINGA_TextVal,    ae->Title,
                                        STRINGA_MaxChars,   sizeof(ae->Title) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_TITLE,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_TITLE)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->Address = StringObject,
                                        STRINGA_TextVal,    ae->Address,
                                        STRINGA_MaxChars,   sizeof(ae->Address) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_ADDRESS,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_ADDRESS)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->Homepage = StringObject,
                                        STRINGA_TextVal,    ae->Homepage,
                                        STRINGA_MaxChars,   sizeof(ae->Homepage) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_HOMEPAGE,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_HOMEPAGE)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->Nickname = StringObject,
                                        STRINGA_TextVal,    ae->Nickname,
                                        STRINGA_MaxChars,   sizeof(ae->Nickname) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_NICKNAME,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_NICKNAME)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->Comments = StringObject,
                                        STRINGA_TextVal,    ae->Comments,
                                        STRINGA_MaxChars,   sizeof(ae->Comments) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_COMMENTS,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_COMMENTS)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,   SpaceObject,
                                        SPACE_Transparent,       TRUE,
                                   EndMember,
                              End,
                              PAGE_Add, control->Page2 = VGroupObject,  // contact
                                   LAYOUT_SpaceOuter,  TRUE,
                                   LAYOUT_VertAlignment,    LALIGN_TOP,
                                   LAYOUT_HorizAlignment,   LALIGN_CENTER,
                                   StartMember,        control->StreetAddress1 = StringObject,
                                        STRINGA_TextVal,    ae->StreetAddress1,
                                        STRINGA_MaxChars,   sizeof(ae->StreetAddress1) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_STREETADDRESS1,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_STREETADDRESS1)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->StreetAddress2 = StringObject,
                                        STRINGA_TextVal,    ae->StreetAddress2,
                                        STRINGA_MaxChars,   sizeof(ae->StreetAddress2) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_STREETADDRESS2,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_STREETADDRESS2)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->City = StringObject,
                                        STRINGA_TextVal,    ae->City,
                                        STRINGA_MaxChars,   sizeof(ae->City) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_CITY,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_CITY)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->State = StringObject,
                                        STRINGA_TextVal,    ae->State,
                                        STRINGA_MaxChars,   sizeof(ae->State) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_STATE,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_STATE)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->ZIP = StringObject,
                                        STRINGA_TextVal,    ae->ZIP,
                                        STRINGA_MaxChars,   sizeof(ae->ZIP) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_ZIP,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_ZIP)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->Country = StringObject,
                                        STRINGA_TextVal,    ae->Country,
                                        STRINGA_MaxChars,   sizeof(ae->Country) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_COUNTRY,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_COUNTRY)),
                                   CHILD_WeightedHeight,    0,
                                   StartHGroup,
                                        StartMember,   BevelObject,
                                             BEVEL_Style,   BVS_SBAR_VERT,
                                        EndMember,
                                   EndGroup,
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->WorkPhone = StringObject,
                                        STRINGA_TextVal,    ae->WorkPhone,
                                        STRINGA_MaxChars,   sizeof(ae->WorkPhone) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_WORKPHONE,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_WORKPHONE)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->HomePhone = StringObject,
                                        STRINGA_TextVal,    ae->HomePhone,
                                        STRINGA_MaxChars,   sizeof(ae->HomePhone) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_HOMEPHONE,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_HOMEPHONE)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->Fax = StringObject,
                                        STRINGA_TextVal,    ae->Fax,
                                        STRINGA_MaxChars,   sizeof(ae->Fax) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_FAX,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_FAX)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,   SpaceObject,
                                        SPACE_Transparent,       TRUE,
                                   EndMember,
                              End,
                              PAGE_Add, control->Page3 = VGroupObject,  // misc
                                   LAYOUT_SpaceOuter,  TRUE,
                                   LAYOUT_VertAlignment,    LALIGN_TOP,
                                   LAYOUT_HorizAlignment,   LALIGN_CENTER,
                                   StartHGroup,
                                        StartMember,   control->IsGroup = CheckBoxObject,
                                             GA_Text,                 LocaleString(ETXT_GAD_ADDRESSBOOK_ISGROUP),
                                             CHECKBOX_TextPlace,      PLACETEXT_RIGHT,
                                             GA_Selected,             ae->Members,
                                             GA_ID,                   GAD_ADDRESSBOOK_ISGROUP,
                                             GA_RelVerify,            TRUE,
                                        EndMember,
                                        StartMember,   control->IsML = CheckBoxObject,
                                             GA_Text,                 LocaleString(ETXT_GAD_ADDRESSBOOK_ISML),
                                             CHECKBOX_TextPlace,      PLACETEXT_RIGHT,
                                             GA_Selected,             (ae->Flags & AEF_ISML),
                                             GA_ID,                   GAD_ADDRESSBOOK_ISML,
                                             GA_RelVerify,            TRUE,
                                        EndMember,
                                   EndGroup,
                                   CHILD_WeightedHeight,    0,
                                   StartHGroup,
                                        StartMember,   BevelObject,
                                             BEVEL_Style,   BVS_SBAR_VERT,
                                        EndMember,
                                   EndGroup,
                                   CHILD_WeightedHeight,    0,
                                   StartMember,   control->Form = GetFileObject,
                                        GETFILE_TitleText,       LocaleString(ETXT_TITLE_SELECTFORM),
                                        GETFILE_FullFile,        ae->Form,
                                        GETFILE_ReadOnly,        FALSE,
                                        GA_ID,                   GAD_ADDRESSBOOK_FORM,
                                        GA_RelVerify,            TRUE,
                                        GA_TabCycle,             TRUE,
                                        ICA_TARGET,              ICTARGET_IDCMP,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_FORM)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,   control->SigFile = GetFileObject,
                                        GETFILE_TitleText,       LocaleString(ETXT_TITLE_SELECTSIGNATURE),
                                        GETFILE_FullFile,        ae->SigFile,
                                        GETFILE_ReadOnly,        FALSE,
                                        GA_ID,                   GAD_ADDRESSBOOK_SIGNATURE,
                                        GA_RelVerify,            TRUE,
                                        GA_TabCycle,             TRUE,
                                        ICA_TARGET,              ICTARGET_IDCMP,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_SIGNATURE)),
                                   CHILD_WeightedHeight,    0,
                                   StartHGroup,
                                        StartMember,   BevelObject,
                                             BEVEL_Style,   BVS_SBAR_VERT,
                                        EndMember,
                                   EndGroup,
                                   CHILD_WeightedHeight,    0,
                                   StartMember,        control->KeyID = StringObject,
                                        STRINGA_TextVal,    ae->KeyID,
                                        STRINGA_MaxChars,   sizeof(ae->KeyID) - 1,
                                        GA_RelVerify,       TRUE,
                                        GA_TabCycle,        TRUE,
                                        GA_ID,              GAD_ADDRESSBOOK_KEYID,
                                   EndMember,
                                   MemberLabel(LocaleString(ETXT_GAD_ADDRESSBOOK_KEYID)),
                                   CHILD_WeightedHeight,    0,
                                   StartMember,   SpaceObject,
                                        SPACE_Transparent,       TRUE,
                                   EndMember,
                              End,
                         EndMember,
                    End,
                    TAG_END)))
               goto thatsit;

          SetAttrs(control->Tabs,
                    ICA_TARGET,         control->Pager,
                    TAG_END);
     }

     // open window
     if(!(control->InfoWindow = CA_OpenWindow(control->InfoWindowO)))
          goto thatsit;

     ActivateLayoutGadget(layout, control->InfoWindow, NULL, control->FirstName);

     while(!done)
     {
          LONG signals;

          signals = Wait((1L << control->IDCMPPort->mp_SigBit) |
                         SIG_KILL |
                         SIG_SHOW);

          if(signals & SIG_KILL)
          {
               Signal(FindTask(NULL), SIG_KILL);
               break;
          }

          if(signals & SIG_SHOW)
          {
               WindowToFront(control->Window);
               WindowToFront(control->InfoWindow);
          }

          if(signals & (1L << control->IDCMPPort->mp_SigBit))
          {
               LONG result, code = 0;

               while(WMHI_LASTMSG != (result = CA_HandleInput(control->InfoWindowO, code)))
               {
                    switch(result & WMHI_CLASSMASK)
                    {
                         STRPTR text;

                         case WMHI_CLOSEWINDOW:
                              GetAttr(STRINGA_TextVal, control->FirstName, (ULONG *)&text);
                              strncpy(ae->FirstName, text, sizeof(ae->FirstName) - 1);
                              GetAttr(STRINGA_TextVal, control->LastName, (ULONG *)&text);
                              strncpy(ae->LastName, text, sizeof(ae->LastName) - 1);
                              GetAttr(STRINGA_TextVal, control->Address, (ULONG *)&text);
                              strncpy(ae->Address, text, sizeof(ae->Address) - 1);
                              GetAttr(STRINGA_TextVal, control->Comments, (ULONG *)&text);
                              strncpy(ae->Comments, text, sizeof(ae->Comments) - 1);
                              GetAttr(STRINGA_TextVal, control->KeyID, (ULONG *)&text);
                              strncpy(ae->KeyID, text, sizeof(ae->KeyID) - 1);
                              GetAttr(STRINGA_TextVal, control->Organization, (ULONG *)&text);
                              strncpy(ae->Organization, text, sizeof(ae->Organization) - 1);
                              GetAttr(STRINGA_TextVal, control->Homepage, (ULONG *)&text);
                              strncpy(ae->Homepage, text, sizeof(ae->Homepage) - 1);
                              GetAttr(STRINGA_TextVal, control->Title, (ULONG *)&text);
                              strncpy(ae->Title, text, sizeof(ae->Title) - 1);
                              GetAttr(STRINGA_TextVal, control->StreetAddress1, (ULONG *)&text);
                              strncpy(ae->StreetAddress1, text, sizeof(ae->StreetAddress1) - 1);
                              GetAttr(STRINGA_TextVal, control->StreetAddress2, (ULONG *)&text);
                              strncpy(ae->StreetAddress2, text, sizeof(ae->StreetAddress2) - 1);
                              GetAttr(STRINGA_TextVal, control->City, (ULONG *)&text);
                              strncpy(ae->City, text, sizeof(ae->City) - 1);
                              GetAttr(STRINGA_TextVal, control->State, (ULONG *)&text);
                              strncpy(ae->State, text, sizeof(ae->State) - 1);
                              GetAttr(STRINGA_TextVal, control->ZIP, (ULONG *)&text);
                              strncpy(ae->ZIP, text, sizeof(ae->ZIP) - 1);
                              GetAttr(STRINGA_TextVal, control->Country, (ULONG *)&text);
                              strncpy(ae->Country, text, sizeof(ae->Country) - 1);
                              GetAttr(STRINGA_TextVal, control->HomePhone, (ULONG *)&text);
                              strncpy(ae->HomePhone, text, sizeof(ae->HomePhone) - 1);
                              GetAttr(STRINGA_TextVal, control->WorkPhone, (ULONG *)&text);
                              strncpy(ae->WorkPhone, text, sizeof(ae->WorkPhone) - 1);
                              GetAttr(STRINGA_TextVal, control->Fax, (ULONG *)&text);
                              strncpy(ae->Fax, text, sizeof(ae->Fax) - 1);
                              GetAttr(GETFILE_FullFile, control->Form, (ULONG *)&text);
                              strncpy(ae->Form, text, sizeof(ae->Form) - 1);
                              GetAttr(GETFILE_FullFile, control->SigFile, (ULONG *)&text);
                              strncpy(ae->SigFile, text, sizeof(ae->SigFile) - 1);

                              done = TRUE;
                              break;
                    }
               }
          }
     }

     thatsit:
     if(control->InfoWindowO)
          DisposeObject(control->InfoWindowO);
          control->InfoWindowO = NULL;
          control->InfoWindow = NULL;
          control->FirstName = NULL;
          control->LastName = NULL;
          control->Address = NULL;
          control->Comments = NULL;
          control->Title = NULL;
          control->Organization = NULL;
          control->Homepage = NULL;
          control->StreetAddress1 = NULL;
          control->StreetAddress2 = NULL;
          control->City = NULL;
          control->State = NULL;
          control->ZIP = NULL;
          control->Country = NULL;
          control->HomePhone = NULL;
          control->WorkPhone = NULL;
          control->Fax = NULL;
          control->Form = NULL;
          control->SigFile = NULL;
          control->MailFormat = NULL;
          control->Sign = NULL;
          control->Encrypt = NULL;

     while(node = RemHead(&control->TabList))
          FreeClickTabNode(node);

     return(error);
}

void AddNewAddress(AddressEntry *ae)
{
     if(!ae)
          return;

     ObtainSemaphore(&euc.Locks.NewAddrs);
     AddTail(&euc.NewAddrs, (struct Node *)ae);
     ReleaseSemaphore(&euc.Locks.NewAddrs);

     if(InitAddressBook())
     {
          IsenRequest(euc.Screen,
                    ISENREQUEST_Text, LocaleString(ETXT_UNABLETOOPENWINDOW),
                    TAG_END);
     }
     else
     {
          if(euc.AddressBookProcess)
               Signal(euc.AddressBookProcess, SIG_CHECK);
     }
}

void SetMarkedAddresses(void)
{
     AddressBookControl *control;
     AddressEntry *aeh;
     struct Node *node;

     if(!(control = euc.AddressBookControl))
          return;

     control->NumSelected     =
     control->NumGroups       =
     control->NumMembers      = 0;

     for(node = control->Addresses.lh_Head;
         node->ln_Succ;
         node = node->ln_Succ)
     {
          LONG sel, flags;

          GetListBrowserNodeAttrs(node,
                    LBNA_Selected,      &sel,
                    LBNA_Flags,         &flags,
                    TAG_END);

          if(control->Current = (AddressEntry *)MatchAddress(GetListNode(&control->Addresses, node)))
          {
               if(control->Current->Members)
                    aeh = control->Current;
               else
               {
                    if(!(control->Current->Flags & AEF_ISMEMBER))
                         aeh = NULL;
               }
          }

          if(sel)
          {
               control->NumSelected++;
               if(control->Current)
               {
                    control->Current->Flags |= AEF_MARKED;

                    if(control->Current->Members)
                         control->NumGroups++;

                    if(control->Current->Flags & AEF_ISMEMBER)
                         control->NumMembers++;

                    if(flags & LBFLG_SHOWCHILDREN)
                         control->Current->Flags &= ~AEF_HIDE;
                    else
                         control->Current->Flags |= AEF_HIDE;
               }
          }
          else
          {
               if(control->Current)
                    control->Current->Flags &= ~AEF_MARKED;
          }
     }
}

void ClearMarkedAddresses(void)
{
     AddressBookControl *control;
     AddressEntry *ae;

     if(!(control = euc.AddressBookControl))
          return;

     control->NumSelected = 0;
     control->Current = NULL;

     ObtainSemaphore(&euc.Locks.Addresses);
     for(ae = (AddressEntry *)euc.Addresses.lh_Head;
         ae->Node.ln_Succ;
         ae = (AddressEntry *)ae->Node.ln_Succ)
     {
          ae->Flags &= ~AEF_MARKED;

          if(ae->Members)
          {
               AddressEntry *aem;

               for(aem = (AddressEntry *)ae->Members->lh_Head;
                   aem->Node.ln_Succ;
                   aem = (AddressEntry *)aem->Node.ln_Succ)
               {
                    aem->Flags &= ~AEF_MARKED;
               }
          }
     }
     ReleaseSemaphore(&euc.Locks.Addresses);
}

void CopyAddressEntry(AddressEntry *ae, AddressEntry *aeg, BOOL moveme)
{
     AddressBookControl *control;

     if(!(control = euc.AddressBookControl))
          return;

     ae->Flags &= ~AEF_MARKED;

     if(moveme)
     {
          if(ae->Members)
          {
               AddressEntry *aem;

               while(aem = (AddressEntry *)RemHead(ae->Members))
               {
                    if(aeg == (AddressEntry *)1)
                         AddTail(&euc.Addresses, (struct Node *)aem);
                    else
                    {
                         AddTail(aeg->Members, (struct Node *)aem);
                         aem->Flags |= AEF_ISMEMBER;
                    }
               }
          }
          else
          {
               Remove((struct Node *)ae);
               if(aeg == (AddressEntry *)1)
                    AddTail(&euc.Addresses, (struct Node *)ae);
               else
               {
                    AddTail(aeg->Members, (struct Node *)ae);
                    ae->Flags |= AEF_ISMEMBER;
               }
          }
     }
     else
     {
          AddressEntry *aenew;

          if(ae->Members)
          {
               AddressEntry *aem;

               for(aem = (AddressEntry *)ae->Members->lh_Head;
                   aem->Node.ln_Succ;
                   aem = (AddressEntry *)aem->Node.ln_Succ)
               {
                    if(!(aenew = (AddressEntry *)AllocVec(sizeof(AddressEntry), MEMF_CLEAR)))
                         break;
                    memcpy(aenew, aem, sizeof(AddressEntry));
                    if(aeg == (AddressEntry *)1)
                         AddTail(&euc.Addresses, (struct Node *)aenew);
                    else
                    {
                         AddTail(aeg->Members, (struct Node *)aenew);
                         aenew->Flags |= AEF_ISMEMBER;
                    }
                    control->Current = aenew;
               }
          }
          else
          {
               if(!(aenew = (AddressEntry *)AllocVec(sizeof(AddressEntry), MEMF_CLEAR)))
                    return;
               memcpy(aenew, ae, sizeof(AddressEntry));
               if(aeg == (AddressEntry *)1)
                    AddTail(&euc.Addresses, (struct Node *)aenew);
               else
               {
                    AddTail(aeg->Members, (struct Node *)aenew);
                    aenew->Flags |= AEF_ISMEMBER;
               }
               control->Current = aenew;
          }
     }
}

#ifdef EUCALYPTUS_PRO
LONG PrintAddressBook(UBYTE *dest, UBYTE *form)
{
     AddressBookControl *control;
     AddressEntry *ae;
     LONG error = 0;
     BPTR fp;

     if(!(control = euc.AddressBookControl))
          return(0);

     if(!dest ||
        !form)
          return(0);

     ObtainSemaphore(&euc.Locks.Addresses);
     if(fp = Open(dest, MODE_NEWFILE))
     {
          BusyPointer(control->Window, TRUE);

          for(ae = (AddressEntry *)euc.Addresses.lh_Head;
              ae->Node.ln_Succ && !error;
              ae = (AddressEntry *)ae->Node.ln_Succ)
          {
               // print/save it
               if(ae->Flags & AEF_MARKED)
                    if(error = OutputAddressData(fp, form, ae))
                         break;

               if(ae->Members)
               {
                    AddressEntry *aem;

                    for(aem = (AddressEntry *)ae->Members->lh_Head;
                        aem->Node.ln_Succ;
                        aem = (AddressEntry *)aem->Node.ln_Succ)
                    {
                         // print/save it
                         if(aem->Flags & AEF_MARKED)
                              if(error = OutputAddressData(fp, form, aem))
                                   break;
                    }
               }
          }

          BusyPointer(control->Window, FALSE);

          Close(fp);
     }
     ReleaseSemaphore(&euc.Locks.Addresses);

     return(error);
}

LONG OutputAddressData(BPTR fp, UBYTE *form, AddressEntry *ae)
{
     LONG error = 0;
     UBYTE *buf;
     BPTR form_fp;

     if(!fp ||
        !form ||
        !ae)
          return(0);

     if(buf = (UBYTE *)AllocVec(WORKLEN, MEMF_CLEAR | MEMF_PUBLIC))
     {
          if(form_fp = Open(form, MODE_OLDFILE))
          {
               LONG i = 0;

               while(FGets(form_fp, buf, WORKLEN - 1))
               {
                    i = 0;

                    if(buf[0] == '#')
                         continue;

                    while(buf[i])
                    {
                         switch(buf[i])
                         {
                              LONG l;
                              UBYTE *p;

                              case '\n':
                              case '\r':
                                   break;
                              case '&': // style codes
                                   switch(buf[++i])
                                   {
                                        case 'b': // bold
                                             FPuts(fp, "\033[1m");
                                             break;
                                        case 'B': // bold off
                                             FPuts(fp, "\033[22m");
                                             break;
                                        case 'i': // italic
                                             FPuts(fp, "\033[3m");
                                             break;
                                        case 'I': // italic off
                                             FPuts(fp, "\033[23m");
                                             break;
                                        case 'u': // underline
                                             FPuts(fp, "\033[4m");
                                             break;
                                        case 'U': // underline off
                                             FPuts(fp, "\033[24m");
                                             break;
                                        case 'n': // normal
                                             FPuts(fp, "\033[0w");
                                             break;
                                        case 'c': // condensed
                                             FPuts(fp, "\033[4w");
                                             break;
                                        case 'C': // condensed off
                                             FPuts(fp, "\033[3w");
                                             break;
                                        case 'e': // enlarged
                                             FPuts(fp, "\033[6w");
                                             break;
                                        case 'E': // enlarged off
                                             FPuts(fp, "\033[5w");
                                             break;
                                        case 'w': // shadow
                                             FPuts(fp, "\033[6\"z");
                                             break;
                                        case 'W': // shadow off
                                             FPuts(fp, "\033[5\"z");
                                             break;
                                        case 'p': // superscript
                                             FPuts(fp, "\033[2v");
                                             break;
                                        case 'P': // superscript off
                                             FPuts(fp, "\033[1v");
                                             break;
                                        case 's': // subscript
                                             FPuts(fp, "\033[4v");
                                             break;
                                        case 'S': // subscript off
                                             FPuts(fp, "\033[3v");
                                             break;
                                        case '&': // &
                                             FPuts(fp, "&");
                                             break;
                                   }
                                   break;
                              case '\\':
                                   switch(buf[++i])
                                   {
                                        case 'n': // newline
                                             FPuts(fp, "\n");
                                             break;
                                        case 'r': // cr
                                             FPuts(fp, "\r");
                                             break;
                                        case 't': // tab
                                             FPuts(fp, "\t");
                                             break;
                                        case '\\': // backslash
                                             FPuts(fp, "\\");
                                             break;
                                        default:
                                             FPrintf(fp, "%c", buf[i]);
                                             break;
                                   }
                                   break;
                              case '%':
                                   if(p = Strset(buf + i, "nfleotckaACszyhwxgmSFKH"))
                                   {
                                        UBYTE *astr = NULL, out_buf[101], namebuf[101];

                                        l = p - (buf + i);

                                        switch(*p)
                                        {
                                             case 'n': // name
                                                  strncpy(namebuf, ae->FirstName, sizeof(namebuf) - 1);
                                                  strncat(namebuf, " ", sizeof(namebuf) - 1);
                                                  strncat(namebuf, ae->LastName, sizeof(namebuf) - 1);
                                                  astr = namebuf;
                                                  break;
                                             case 'f': // first name
                                                  astr = ae->FirstName;
                                                  break;
                                             case 'l': // last name
                                                  astr = ae->LastName;
                                                  break;
                                             case 'e': // e-mail address
                                                  astr = ae->Address;
                                                  break;
                                             case 'o': // organization
                                                  astr = ae->Organization;
                                                  break;
                                             case 't': // title
                                                  astr = ae->Title;
                                                  break;
                                             case 'c': // comments
                                                  astr = ae->Comments;
                                                  break;
                                             case 'k': // nickname
                                                  astr = ae->Nickname;
                                                  break;
                                             case 'a': // address1
                                                  astr = ae->StreetAddress1;
                                                  break;
                                             case 'A': // address2
                                                  astr = ae->StreetAddress2;
                                                  break;
                                             case 'C': // city
                                                  astr = ae->City;
                                                  break;
                                             case 's': // state
                                                  astr = ae->State;
                                                  break;
                                             case 'z': // zip
                                                  astr = ae->ZIP;
                                                  break;
                                             case 'y': // country
                                                  astr = ae->Country;
                                                  break;
                                             case 'h': // home phone
                                                  astr = ae->HomePhone;
                                                  break;
                                             case 'w': // workphone
                                                  astr = ae->WorkPhone;
                                                  break;
                                             case 'x': // fax
                                                  astr = ae->Fax;
                                                  break;
                                             case 'g': // group?
                                                  astr = (ae->Members) ? "Group" : "Single";
                                                  break;
                                             case 'm': // mailing list?
                                                  astr = (ae->Flags & AEF_ISML) ? "ML" : "No ML";
                                                  break;
                                             case 'S': // signature
                                                  astr = ae->SigFile;
                                                  break;
                                             case 'F': // form
                                                  astr = ae->Form;
                                                  break;
                                             case 'K': // key ID
                                                  astr = ae->KeyID;
                                                  break;
                                             case 'H': // homepage
                                                  astr = ae->Homepage;
                                                  break;
                                        }

                                        if(astr)
                                        {
                                             *p = '\0';

                                             strncpy(out_buf, buf + i, sizeof(out_buf) - 1);
                                             strncat(out_buf, "s", sizeof(out_buf) - 1);

                                             FPrintf(fp, out_buf, strlen(astr) ? (STRPTR)astr : (STRPTR)"<Empty>");

                                             i += l;

                                             break;
                                        }
                                   }
                              default:
                                   FPrintf(fp, "%lc", (LONG)buf[i]);
                                   break;
                         }

                         i++;
                    }
               }

               Close(form_fp);
          }
          else
               error = err_UNABLETOOPENFILE;

          FreeVec(buf);
     }
     else
          error = err_INSUFFICIENTMEMORY;

     return(error);
}
#endif

LONG GetRecipientAddress(UBYTE *who, UBYTE *orig, UBYTE *addr, LONG addr_len)
{
     struct Window *window;
     Object *winobj = NULL;
     struct Gadget *layout, *listbrowser, *okay;
     struct List list;
     struct Node *node;
     LONG error = 0;
     UBYTE *p, *n, *c;
     enum { GAD_GRA_LISTBROWSER=1, GAD_GRA_OKAY, };

     NewList(&list);

     p = orig;
     for(n=orig; n; p=n)
     {
          if(c = strchr(p, ','))
          {
               *c = '\0';
               n = c + 1;
          }
          else
               n = NULL;

          if(node = AllocListBrowserNode(1,
                    LBNA_Column,        0,
                         LBNCA_CopyText,     TRUE,
                         LBNCA_Text,         p,
                    TAG_END))
          {
               AddTail(&list, node);
          }
          else
               goto thatsit;
     }

     if(winobj = NewObject(WINDOW_GetClass(), NULL,
               WINDOW_Position,              WPOS_CENTERSCREEN,
               WA_Width,                     350,
               WA_Height,                    200,

               WA_Activate,                  TRUE,
               WA_CloseGadget,               TRUE,
               WA_DragBar,                   TRUE,
               WA_DepthGadget,               TRUE,
               WA_SizeGadget,                TRUE,
               WA_SizeBBottom,               TRUE,
               WA_RMBTrap,                   TRUE,
               WINDOW_BackFillName,          sprefs->Backfill,

               WA_PubScreen,                 euc.Screen,
               WA_Title,                     LocaleString(ETXT_TITLE_SELECTADDRESS),
               WA_ScreenTitle,               eucsctitle,

               WINDOW_Layout,      layout = VLayoutObject,
                    LAYOUT_DeferLayout,      TRUE,
                    LAYOUT_SpaceOuter,       TRUE,
                    StartVGroup,
                         StartMember,   ButtonObject,
                              GA_Text,            LocaleString(ETXT_MULTIPLEADDRESSESFOR1),
                              GA_ReadOnly,        TRUE,
                              BUTTON_BevelStyle,  BVS_NONE,
                              BUTTON_Transparent, TRUE,
                         EndMember,
                         StartMember,   ButtonObject,
                              GA_Text,            who,
                              GA_ReadOnly,        TRUE,
                              BUTTON_BevelStyle,  BVS_NONE,
                              BUTTON_Transparent, TRUE,
                         EndMember,
                         StartMember,   ButtonObject,
                              GA_Text,            LocaleString(ETXT_MULTIPLEADDRESSESFOR2),
                              GA_ReadOnly,        TRUE,
                              BUTTON_BevelStyle,  BVS_NONE,
                              BUTTON_Transparent, TRUE,
                         EndMember,
                    EndGroup,
                    CHILD_WeightedHeight,    0,
                    StartMember,   listbrowser = ListBrowserObject,
                         LISTBROWSER_Labels,           &list,
                         LISTBROWSER_AutoFit,          TRUE,
                         LISTBROWSER_ShowSelected,     TRUE,
                         GA_ID,                        GAD_GRA_LISTBROWSER,
                         GA_RelVerify,                 TRUE,
                         GA_TabCycle,                  TRUE,
                    EndMember,
                    StartMember,   okay = ButtonObject,
                         GA_Text,                      LocaleString(ETXT_BUTTON_OKAY),
                         GA_ID,                        GAD_GRA_OKAY,
                         GA_RelVerify,                 TRUE,
                         GA_TabCycle,                  TRUE,
                    EndMember,
                    CHILD_WeightedHeight,    0,
               End,
               TAG_END))
     {
          if(window = CA_OpenWindow(winobj))
          {
               struct Node *node = NULL;
               BOOL done = FALSE;

               while(!done)
               {
                    LONG signals;

                    signals = Wait((1L << window->UserPort->mp_SigBit) |
                                   SIG_KILL);

                    if(signals & SIG_KILL)
                    {
                         Signal(FindTask(NULL), SIG_KILL);
                         done = TRUE;
                         break;
                    }

                    if(signals & (1L << window->UserPort->mp_SigBit))
                    {
                         LONG result;
                         WORD code = 0;

                         while(WMHI_LASTMSG != (result = CA_HandleInput(winobj, code)))
                         {
                              switch(result & WMHI_CLASSMASK)
                              {
                                   case WMHI_CLOSEWINDOW:
                                        error = err_CANCELLED;
                                        done = TRUE;
                                        break;
                                   case WMHI_GADGETUP:
                                        switch(result & WMHI_GADGETMASK)
                                        {
                                             LONG val, relevt;
                                             STRPTR text;

                                             case GAD_GRA_LISTBROWSER:
                                                  GetAttr(LISTBROWSER_RelEvent, listbrowser, (ULONG *)&relevt);
                                                  GetAttr(LISTBROWSER_Selected, listbrowser, (ULONG *)&val);
                                                  node = GetListNum(&list, val);
                                                  if(SetGadgetAttrs(okay, window, NULL,
                                                            GA_Disabled,        !node,
                                                            TAG_END))
                                                       RefreshGList(okay, window, NULL, 1);

                                                  if(!(relevt & LBRE_DOUBLECLICK))
                                                       break;;
                                             case GAD_GRA_OKAY:
                                                  if(!node)
                                                       break;

                                                  text = NULL;
                                                  GetListBrowserNodeAttrs(node,
                                                            LBNA_Column,        0,
                                                                 LBNCA_Text,         &text,
                                                            TAG_END);
                                                  if(text)
                                                  {
                                                       strncpy(addr, text, addr_len - 1);
                                                       done = TRUE;
                                                  }
                                                  break;
                                        }
                                        break;
                              }
                         }
                    }
               }
          }
     }

     thatsit:
     if(winobj)
          DisposeObject(winobj);

     while(node = RemHead(&list))
          FreeListBrowserNode(node);

     return(error);
}

#endif

void address_smart_get_first_last_name(AddressEntry *ae, gchar *first_buf, glong fb_len, gchar *last_buf, glong lb_len)
{
	gchar *f, *l, *n;
	
	if(n = address_get_field(ae, "Name"))
	{
		gchar *c, *nf;
		
		nf = g_strdup(n);
		
		if(c = strchr(nf, ' '))
		{
			*c = 0;
			
			strncpy(first_buf, nf, fb_len - 1);
			strncpy(last_buf, c + 1, lb_len - 1);
		}
		else
		{
			strncpy(first_buf, nf, fb_len - 1);
			strcpy(last_buf, "");
		}
		
		g_free(nf);
	
		return;
	}
	
	f = address_get_field(ae, "FirstName");
	l = address_get_field(ae, "LastName");
	if(!l)
		l = address_get_field(ae, "Surname");

	if(f || l)
	{
		strcpy(first_buf, "");
		if(f)
			strncpy(first_buf, f, fb_len - 1);
			
		strcpy(last_buf, "");
		if(l)
			strncpy(last_buf, l, lb_len - 1);
	}
}

