#ifndef EUCALYPTUS_COMPOSE_H
#define EUCALYPTUS_COMPOSE_H

/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Header: compose.h
 *         Header info for composition threads
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#define MRT_REPLY             (0)
#define MRT_FORWARD           (1)
#define MRT_BOUNCE            (2)

#define MRTT_ALL              (1 << 0)
#define MRTT_NOTEXT           (1 << 1)
#define MRTT_ALLHEADERS       (1 << 2)
#define MRTT_SOMEHEADERS      (1 << 3)

struct _Composition
{
	pid_t					 ID;
     GtkWidget                     *Window,
     						*Layout,
     						*Menu,
     						*Toolbar;
     struct
     {
     	GtkWidget                *Pager, *Page[4], *Tab[4],

							// page 1
							*AddrTo, *AddrToSel,
							*AddrCc, *AddrCcSel,
							*AddrBcc, *AddrBccSel,
                                   *AddressList,
                                   *NewAddr, *DelAddr, *ClearAddrs,
                                   *AddrTypes,
                                   *ManualAddr, *MatchAddr, *AddAddr,

							// page 2
                                   *AttachList,
                                   *NewAtch, *EditAtch, *DelAtch,
                                   
                                   // page 3
                                   *Sign, *Encrypt, 
                                   *EncryptTo, *EncryptToSelf,
                                   
                                   // page 4
                                   *ReplyTo,
                                   *UseTagline, 
                                   *Tagline, *NewTagline,
                                   *UseSigFile,
		                         *SignatureFile, *EditSigFile,
                                   *CustomHeaders,
                                   *Priority, *ReturnReceipt,
                                   
                                   *Subject,
                                   *Format,
                                   
                                   *Textfield;
	} w;
	
	struct {
		GtkWidget				*Window,
							*Name,
							*Filename,
							*Desc,
							*Type;
		gchar				*Title;
		CompAttachment			*catt;
	} att_info;
	
	struct {
		GtkWidget				*Window,
							*AddrList,
							*MatchAddr;
		AddressEntry			*Entry;
		gchar				 Type;
	} addr_sel;
	
	struct {
		GtkWidget				*Dialog;
		gchar				 Type;
		gchar				 Filename[FILENAME_MAX];
	} file_sel;
	
	struct {
		GtkWidget				*Window,
							*HdrList,
							*Add, *Edit, *Del,
							*Text;
		glong				 current;
	} cust_hdr;

	CompAttachment				*SelectedAttachment;
     Speller                		*Speller;
     struct List                    Attachments,
                                    CustomHeaders;
     glong                          Flags,
                                    State;

	struct {
	     gchar                     Subject[201],
	     					 Tagline[1025],
	     					 ReplyTo[201],
	     					 Reference[201],
	     					 EncryptTo[501],
	     					 SignatureFile[FILENAME_MAX],
	     					 Filename[FILENAME_MAX],
							 PostponedFile[FILENAME_MAX];
	} t;
	struct {
		GString				*To,
							*Cc,
							*Bcc;
	} str;

     gchar                          MsgFormat,
                                    MsgPriority,
                                    Boundary[31];
};

// Composition Flags
#define CF_SIGN                    (1L << 0)
#define CF_ENCRYPT                 (1L << 1)
#define CF_USESIGFILE              (1L << 2)
#define CF_USETAGLINE              (1L << 3)
#define CF_RETURNRECEIPT           (1L << 4)
#define CF_NOHEADER                (1L << 5)
#define CF_NOFOOTER                (1L << 6)
#define CF_ENCRYPTTOSELF           (1L << 7)
#define CF_QUEUE                   (1L << 8)
#define CF_QUIT                    (1L << 16)
#define CF_DONE				CF_QUIT
#define CF_SPELLCHECK              (1L << 20)
#define CF_WAITINGSPELLER          (1L << 21)
#define CF_RESUMEDMSG              (1L << 24)
#define CF_REPLIEDMSG              (1L << 25)
#define CF_EDITING                 (1L << 26)
#define CF_AUTO                    (1L << 27)
#define CF_FORWARDEDMSG            (1L << 28)

// Composition States
#define CSTATE_STATELESS           (0)
#define CSTATE_STARTME             (1)
#define CSTATE_INITIALIZING        (2)
#define CSTATE_RUNNING             (3)
#define CSTATE_TERMINATING         (4)
#define CSTATE_DEAD                (5)

struct _CompAddress
{
     gchar                          Type,
     						 Name[101],
                                    Address[101],
                                    Display[201];
	glong					 Flags;
};

// Types for CompAddress
#define CAT_TO                (0)
#define CAT_CC                (1)
#define CAT_BCC               (2)

// formats for AddCompAddress
#define ACAFT_NAME            (0)
#define ACAFT_ADDRESS         (1)
#define ACAFT_PARSE           (2)

// Flags for CompAddress
#define CAF_SELECTED		(1 << 0)

struct _CompAttachment
{
     glong                          Type;
     gchar                          Filename[FILENAME_MAX],
                                    Name[101],
                                    Description[201],
                                    ContentType[101],
     	                          Encoding;
};

// Encodings for CompAttachment
#define CAE_BASE64QP          (0)

// Types for CompAttachment
// same as content types/sub-types (see msgs.h)

struct _QueueInfo
{
     gboolean                       Suspend;
     gint                           Type;
     gchar                         *Date;
};

// QueueInfo Types
#define QUEUE_NONE       (0)
#define QUEUE_AT         (1)
#define QUEUE_EVERY      (2)

struct _Speller
{
     GtkWidget                     *Window,
                                   *Layout,
                                   *Word, *ChangeTo,
                                   *ListBrowser,
                                   *Change, *ChangeAll,
                                   *Ignore, *IgnoreAll,
                                   *Suggest,
                                   *Add, *Delete,
                                   *Status;
	pid_t					 ID;
     GList          	           Suggestions,
                                    IgnoreList;
     gpointer                       Module;
     glong                          Flags,
                                    Position, Length,
                                    RC, RC2;
     struct {
	     gchar                    *Word,
                                   *ChangeTo,
                                   *Status;
     } t;
     gchar                         *ModArgs,
                                   *Result;
};

// Flags
#define CSF_CLOSEWINDOW            (1L << 0)

struct _Word
{
     gchar                         *Word,
                                   *Replacement;
};

struct _Dictionary
{
	   glong					Port;
	   gchar 					Host[101],
						     Username[51],
							Password[51],
							PortText[10];
};

/* Types for compose_insert_file_*()
 */
enum
{
	COMPOSE_INSERTFILE_NORMAL,
	COMPOSE_INSERTFILE_BRACKETED,
	COMPOSE_INSERTFILE_INDENTED,
	COMPOSE_INSERTFILE_FORM,
	COMPOSE_INSERTFILE_UUENCODED,
	COMPOSE_SAVE_FILE,
};

#endif /* EUCALYPTUS_COMPOSE_H */
