#ifndef EUCALYPTUS_H
#define EUCALYPTUS_H

/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: eucalyptus.h
 *   	 General header file
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include "config.h"

#include <gnome.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <libgnorba/gnorba.h>
#include <gtk/gtk.h>
#include <glib.h>

#include <pthread.h>
#include <errno.h>

#include "typedefs.h"
#include "e-config.h"
#include "defines.h"
#include "externs.h"
#include "localize.h"
#include "errors.h"
#include "gadgets.h"
#include "prefs.h"
#include "folders.h"
#include "msgs.h"
#include "addressbook.h"
#include "compose.h"
#include "pop.h"
#include "smtp.h"
#include "structs.h"
#include "protos.h"

struct Eucalyptus
{
	GnomeClient		*GnomeClient;
	GtkWidget			*Window,	// GnomeApp
					*Layout,	// GtkVBox (main layout)
					*FolderList,	// GtkCTree
					*ScrolledWindow, // (for folder list)
					*Pane,	// folder list / folder
					*FolderLayout, // (inside pane 2)
					*MsgList,	// (inside FolderLayout)
					*StatusBar,
					*About,	// GnomeAbout
					*Menu;

	Folder  			*SelectedFolder;
	MailMsg			*CurrentMsg;

	struct { /* Address book control info */
	     GtkWidget           *Window,
	                         *Layout,
	                         *AddressList,
	                         *Top, *Up, *Down, *Bottom,
	                         *Edit, *New, *NewMember, *Delete, *Clear,
	                         *Sort, *SortGroup,
	                         *MoveTo, *CopyTo, *SendTo,
	                         *Print,
	                         *SelectAll, *SelectNone,
	                         *Form, *PrintForm;
	     AddressEntry 		*Current;
	     struct List	 	 Addresses,
	     				 Items;
	     glong                NumSelected,
	                          NumGroups,
	                          NumMembers;
	} addr;

	guint				 HouseKeeper,
						 CompTimer;

	struct { /* Message queues. */
		struct List		 Msgs;	/* for general msg handling commands from other threads */
		pthread_mutex_t	 Mutex;
	} q;
	
	struct { /* Folder monitor. */
		struct List		 Msgs;
		pthread_mutex_t	 Mutex;
		pthread_t			 Thread;
		pthread_attr_t		 Attr;
	} monitor;

	struct List    	 Folders,
					 Composes,
					 Servers,
					 Viewers,
					 Taglines,
					 Filters,
					 Macros,
					 Dictionaries,
					 Identities;
					 
	struct {
		pthread_mutex_t	Folders,
						Composes,
						Servers,
						Dictionaries,
						Prefs,
						Filters,
						Viewers;
	} mutex;

	CORBA_Environment	 CorbaEnv;
	CORBA_ORB			 Orb;

	glong 			 Flags;
};

// Flags
#define EGF_QUIT    			(1L << 0)
#define EGF_QUIET   			(1L << 1)
#define EGF_UPDATEFOLDERS		(1L << 2)
#define EGF_HELP    			(1L << 3)
/* 4 undefined */
#define EGF_COMPOSEONLY  		(1L << 5)
#define EGF_NEWPREFS			(1L << 6)
#define EGF_ICONIFY 			(1L << 7)
#define EGF_UPDATEADDRESSBOOK 	(1L << 8)
#define EGF_DISPATCHED			(1L << 9)
#define EGF_OPENWINDOW   		(1L << 10)
#define EGF_CLOSEABOUTWINDOW  	(1L << 11)
#define EGF_CLOSEERRORWINDOW  	(1L << 12)
#define EGF_ORGANIZEFOLDERS   	(1L << 13)
#define EGF_LOGOUT  			(1L << 14)
#define EGF_PRESTART			(1L << 15)
#define EGF_INITCONFIG			(1L << 16)

// ConfigFlags
#define CONFIGF_OPENGENERAL   (1L << 0)
#define CONFIGF_CLOSEGENERAL  (1L << 16)
#define CONFIGF_OPENNETWORK   (1L << 1)
#define CONFIGF_CLOSENETWORK  (1L << 17)
#define CONFIGF_OPENVIEWERS   (1L << 2)
#define CONFIGF_CLOSEVIEWERS  (1L << 18)
#define CONFIGF_OPENFILTERS   (1L << 3)
#define CONFIGF_CLOSEFILTERS  (1L << 19)
#define CONFIGF_OPENUSER 	(1L << 4)
#define CONFIGF_CLOSEUSER	(1L << 20)
#define CONFIGF_OPENMAIL 	(1L << 5)
#define CONFIGF_CLOSEMAIL	(1L << 21)
#define CONFIGF_OPENTICKLER   (1L << 6)
#define CONFIGF_CLOSETICKLER  (1L << 22)

// ConfigProcTypes

struct VersInfo
{
	gint32 			 VersID;
	gchar			 Vers[51];
};

/* Requester flags for euc_request()
 */
enum
{
	EUCRF_ASYNC = 1 << 0
};

#endif /* EUCALYPTUS_H */
