/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: folder_support.c
 *         Support functions for folders
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "eucalyptus.h"

#if 0
void FolderImport(struct Folder *folder)
{
     struct List *list;

     BusyPointer(folder->Window, TRUE);
     if(list = GetFiles(LocaleString(ETXT_TITLE_IMPORTMESSAGES), folder->Window,
               ASLFR_DoMultiSelect,          TRUE,
               TAG_END))
     {
          struct Node *node;

          SetFolderStatus(folder, LocaleString(ETXT_FOLDERSTATUS_IMPORTINGMSGS));
          for(node = list->lh_Head;
              node->ln_Succ;
              node = node->ln_Succ)
          {
               if(node->ln_Name)
                    ImportMsgs(node->ln_Name, folder);
          }
          SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
                    LISTBROWSER_Labels,      ~0,
                    TAG_END);
          UpdateFolderIndex(folder);
          SortMsgs(folder);
          if(SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
                    LISTBROWSER_Labels,      &folder->Msgs,
                    LISTBROWSER_AutoFit,     TRUE,
                    TAG_END))
               RefreshGList(folder->ListBrowser, folder->Window, NULL, 1);
          SetFolderStatus(folder, NULL);

          FreeFiles(list);
     }
     BusyPointer(folder->Window, FALSE);
}

void FolderApplyFilters(struct Folder *folder)
{
     BusyPointer(folder->Window, TRUE);
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      ~0,
               TAG_END);
     ApplyFilters(folder);
     SortMsgs(folder);
     if(SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      &folder->Msgs,
               LISTBROWSER_AutoFit,     TRUE,
               TAG_END))
          RefreshGList(folder->ListBrowser, folder->Window, NULL, 1);
     SetFolderStatus(folder, NULL);
     BusyPointer(folder->Window, FALSE);
}

void FolderMailMsgReply(struct Folder *folder, BYTE type, BYTE replyflags)
{
     struct MailMsg *mmsg;

     if(CheckBrowserSelections(folder->ListBrowser) > 1)
     {
          if(!IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_MULTIPLEMSGSSELECTED),
                    ISENREQUEST_Button,      LocaleString(ETXT_REQ_YES_NO),
                    TAG_END))
               return;
     }
     BusyPointer(folder->Window, TRUE);
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      ~0,
               TAG_END);
     for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
         mmsg->Node.ln_Succ;
         mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          LONG val;

          GetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      (ULONG *)&val,
                    TAG_END);
          if(val)
          {
               BusyPointer(folder->Window, TRUE);
               if(!CreateMsgReply(mmsg, type, replyflags, NULL))
               {
                    if(oprefs->IconifySuccessive)
                         folder->Data.Flags |= FFF_ICONIFY;
               }
               BusyPointer(folder->Window, FALSE);
          }
     }
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      &folder->Msgs,
               LISTBROWSER_AutoFit,     TRUE,
               TAG_END);
     BusyPointer(folder->Window, FALSE);
}

void FolderMailMsgDelete(struct Folder *folder)
{
     struct MailMsg *mmsg = NULL;
     LONG etext, btext = ETXT_REQ_YES_NO;
     UBYTE action = 1;
     BOOL setvalid = FALSE, refresh = FALSE, attached = TRUE;

     if(!folder)
          return;

     if(CheckBrowserSelections(folder->ListBrowser) > 1)
     {
          if(ETXT_TOGGLEDELETE == (etext = CheckDeletionState(folder)))
               btext = ETXT_REQ_YES_DELETE_UNDELETE_CANCEL;
          if(!(action = IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(etext),
                    ISENREQUEST_Button,      LocaleString(btext),
                    TAG_END)))
               return;

          attached = FALSE;
          SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
                    LISTBROWSER_Labels,           ~0,
                    TAG_END);
     }

     for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
         mmsg->Node.ln_Succ;
         mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          LONG val;

          GetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      (ULONG *)&val,
                    TAG_END);
          if(val)
          {
               switch(action)
               {
                    case 1:   // toggle
                         mmsg->Index.Flags ^= MIEF_DELETED;
//                       if(!(mmsg->Index.Flags & MIEF_DELETED))
//                            mmsg->Flags |= MMF_CLOSEWINDOW;
                         setvalid = TRUE;
                         break;
                    case 2:   // delete
                         mmsg->Index.Flags |= MIEF_DELETED;
//                       mmsg->Flags |= MMF_CLOSEWINDOW;
                         setvalid = TRUE;
                         break;
                    case 3:   // undelete
                         mmsg->Index.Flags &= ~MIEF_DELETED;
                         setvalid = TRUE;
                         break;
               }

               UpdateMsgStatus(mmsg);
               if(UpdateMsgInfo(mmsg, attached))
                    refresh = TRUE;

               folder->Data.Flags |= FFF_INDEXMODIFIED;
          }
     }

     if(!attached)
          if(SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
                    LISTBROWSER_Labels,           &folder->Msgs,
                    LISTBROWSER_AutoFit,          TRUE,
                    TAG_END))
               refresh = TRUE;

     if(folder->Window && refresh)
          RefreshGList(folder->ListBrowser, folder->Window, NULL, 1);

     if(setvalid)
          SetFolderIndexValid(folder, FALSE);
}

void FolderMailMsgPrint(struct Folder *folder)
{
     struct MailMsg *mmsg;

     if(CheckBrowserSelections(folder->ListBrowser) > 1)
     {
          if(!IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_MULTIPLEMSGSSELECTED),
                    ISENREQUEST_Button,      LocaleString(ETXT_REQ_YES_NO),
                    TAG_END))
               return;
     }
     BusyPointer(folder->Window, TRUE);
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      ~0,
               TAG_END);
     for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
         mmsg->Node.ln_Succ;
         mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          LONG val;

          GetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      (ULONG *)&val,
                    TAG_END);
          if(val)
          {
               struct PrintOptions *opts;

               BusyPointer(folder->Window, TRUE);
               if(opts = GetPrintOptions(mmsg))
               {
                    MailMsgPrint(mmsg, opts);
                    FreePrintOptions(opts);
               }
               BusyPointer(folder->Window, FALSE);
          }
     }
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      &folder->Msgs,
               LISTBROWSER_AutoFit,     TRUE,
               TAG_END);
     BusyPointer(folder->Window, FALSE);
}

void FolderSort(struct Folder *folder)
{
     if(!folder)
          return;

     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      &folder->Msgs,
               LISTBROWSER_AutoFit,     TRUE,
               TAG_END);
     SortMsgs(folder);
     if(SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      &folder->Msgs,
               LISTBROWSER_AutoFit,     TRUE,
               TAG_END))
          RefreshGList(folder->ListBrowser, folder->Window, NULL, 1);
}

void FolderMailMsgCopy(struct Folder *folder, struct Folder *dfolder, BOOL moveme)
{
     struct MailMsg *mmsg, *nmsg;
     BOOL yup = FALSE;

     if(!folder ||
        !dfolder)
          return;

     BusyPointer(folder->Window, TRUE);
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      ~0,
               TAG_END);
     SetFolderStatus(folder, LocaleString(moveme ? ETXT_FOLDERSTATUS_MOVINGMSGS : ETXT_FOLDERSTATUS_COPYINGMSGS));
     mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
     while(nmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          LONG val;

          GetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      (ULONG *)&val,
                    TAG_END);
          if(val)
          {
               if(!CopyMsg(mmsg->Index.Filename, dfolder, NULL, 0))
               {
                    if(moveme)
                    {
                         UBYTE name[31];

                         DeleteFile(mmsg->Index.Filename);
                         if(folder->Data.Type & FTF_OUTGOING)
                         {
                              strncpy(name, mmsg->Index.Filename, sizeof(name) - 1);
                              memset(name, '_', 3);
                              DeleteFile(name);
                              name[2] = 'a';
                              DeleteFile(name);
                              name[2] = 'i';
                              DeleteFile(name);
                         }
                         Remove((struct Node *)mmsg);
                         AdjustMsgCount(folder, 1, mmsg->Index.Size, mmsg->Index.Flags, -1);
                         FreeMailMsg(mmsg);
                         folder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_UPDATEMAIN | FFF_BUILDNEWINDEX);
                         SetFolderIndexValid(folder, FALSE);
                    }

                    AdjustMsgCount(dfolder, 1, mmsg->Index.Size, mmsg->Index.Flags, 1);
                    dfolder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_NEWMSGS | FFF_UPDATEMAIN);
               SetFolderIndexValid(dfolder, FALSE);
                    yup = TRUE;
               }
          }

          mmsg = nmsg;
     }
     SetFolderStatus(folder, NULL);
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      &folder->Msgs,
               LISTBROWSER_AutoFit,     TRUE,
               TAG_END);
     euc.Flags |= EGF_UPDATEFOLDERS;
     Signal(euc.ThisProcess, SIG_UPDATE);
     if(yup &&
        dfolder->Process)
          Signal(dfolder->Process, SIG_UPDATE);
     BusyPointer(folder->Window, FALSE);
}

void FolderMailMsgRead(struct Folder *folder)
{
     struct MailMsg *mmsg;
     BOOL refresh = FALSE;

     if(!folder)
          return;

     if(CheckBrowserSelections(folder->ListBrowser) > 1)
     {
          if(!IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_MULTIPLEMSGSSELECTED),
                    ISENREQUEST_Button,      LocaleString(ETXT_REQ_YES_NO),
                    TAG_END))
               return;
     }
     BusyPointer(folder->Window, TRUE);
     for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
         mmsg->Node.ln_Succ;
         mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          LONG val, error = 0;

          GetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      (ULONG *)&val,
                    TAG_END);
          if(val)
          {
               switch(error = OpenFolderMsg(folder, mmsg))
               {
                    case err_UPDATEDISPLAY:
                         refresh = TRUE;
                    case 0:
                         if(oprefs->IconifySuccessive)
                              folder->Data.Flags |= FFF_ICONIFY;
                         break;
                    default:
                         DisplayError(LocaleString(ETXT_UNABLETOOPENWINDOW), error);
                         break;
               }
          }
     }
     BusyPointer(folder->Window, FALSE);

     if(refresh)
          if(folder->ListBrowser && folder->Window)
               RefreshGList(folder->ListBrowser, folder->Window, NULL, 1);
}

void FolderMailMsgEdit(struct Folder *folder)
{
     struct MailMsg *mmsg;

     if(!folder)
          return;

     if(CheckBrowserSelections(folder->ListBrowser) > 1)
     {
          if(!IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_MULTIPLEMSGSSELECTED),
                    ISENREQUEST_Button,      LocaleString(ETXT_REQ_YES_NO),
                    TAG_END))
               return;
     }
     BusyPointer(folder->Window, TRUE);
     for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
         mmsg->Node.ln_Succ;
         mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          LONG val;

          GetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      (ULONG *)&val,
                    TAG_END);
          if(val)
          {
               if(mmsg->WindowO)
               {
                    CloseMsgWindow(mmsg);
               }

               EditQueuedMessage(mmsg);
          }
     }
     BusyPointer(folder->Window, FALSE);
}

void FolderGetAddress(struct Folder *folder)
{
     struct AddressEntry *aeg;
     struct MailMsg *mmsg;
     LONG l;
     BOOL autoadd = FALSE;

     if(!folder)
          return;

     if(!(l = CheckBrowserSelections(folder->ListBrowser)))
          return;
     else if(l > 20)
     {
          if(!IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_MORETHAN20ADDRS),
                    ISENREQUEST_Button,      LocaleString(ETXT_REQ_OKAY_CANCEL),
                    TAG_END))
               return;
          autoadd = TRUE;
     }
     else if(l > 1)
     {
          if(!IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_MULTIPLEMSGSSELECTED),
                    ISENREQUEST_Button,      LocaleString(ETXT_REQ_YES_NO),
                    TAG_END))
               return;
     }

     if(!(aeg = SelectAddressGroup()))
          return;

     BusyPointer(folder->Window, TRUE);
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      ~0,
               TAG_END);
     for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
         mmsg->Node.ln_Succ;
         mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          LONG val;

          GetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      (ULONG *)&val,
                    TAG_END);
          if(val)
               GetMsgAddress(mmsg, aeg, autoadd);
     }
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      &folder->Msgs,
               LISTBROWSER_AutoFit,     TRUE,
               TAG_END);
     euc.Flags |= EGF_UPDATEADDRESSBOOK;
     Signal(euc.ThisProcess, SIG_UPDATE);
     BusyPointer(folder->Window, FALSE);
}

void FolderSetStatus(struct Folder *folder)
{
     struct MailMsg *mmsg;
     struct FlagNode *flagnode = NULL;
     LONG l;

     if(!(l = CheckBrowserSelections(folder->ListBrowser)))
          return;
     BusyPointer(folder->Window, FALSE);
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      ~0,
               TAG_END);
     if(l > 1)
     {
          if(flagnode = GetMultiMsgFlags())
          {
               for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
                   mmsg->Node.ln_Succ;
                   mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
               {
                    LONG val;

                    GetListBrowserNodeAttrs((struct Node *)mmsg,
                              LBNA_Selected,      (LONG)&val,
                              TAG_END);
                    if(val)
                    {
                         switch(flagnode->New)
                         {
                              case FNSF_SET:
                                   mmsg->Index.Flags |= MIEF_NEW;
                                   folder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_UPDATEMAIN);
                                   break;
                              case FNSF_CLEAR:
                                   mmsg->Index.Flags &= ~MIEF_NEW;
                                   folder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_UPDATEMAIN);
                                   break;
                         }
                         switch(flagnode->Unread)
                         {
                              case FNSF_SET:
                                   mmsg->Index.Flags |= MIEF_UNREAD;
                                   folder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_UPDATEMAIN);
                                   break;
                              case FNSF_CLEAR:
                                   mmsg->Index.Flags &= ~MIEF_UNREAD;
                                   folder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_UPDATEMAIN);
                                   break;
                         }
                         switch(flagnode->Replied)
                         {
                              case FNSF_SET:
                                   mmsg->Index.Flags |= MIEF_REPLIED;
                                   folder->Data.Flags |= FFF_INDEXMODIFIED;
                                   break;
                              case FNSF_CLEAR:
                                   mmsg->Index.Flags &= ~MIEF_REPLIED;
                                   folder->Data.Flags |= FFF_INDEXMODIFIED;
                                   break;
                         }
                         switch(flagnode->Forwarded)
                         {
                              case FNSF_SET:
                                   mmsg->Index.Flags |= MIEF_FORWARDED;
                                   folder->Data.Flags |= FFF_INDEXMODIFIED;
                                   break;
                              case FNSF_CLEAR:
                                   mmsg->Index.Flags &= ~MIEF_FORWARDED;
                                   folder->Data.Flags |= FFF_INDEXMODIFIED;
                                   break;
                         }
                         switch(flagnode->Bounced)
                         {
                              case FNSF_SET:
                                   mmsg->Index.Flags |= MIEF_BOUNCED;
                                   folder->Data.Flags |= FFF_INDEXMODIFIED;
                                   break;
                              case FNSF_CLEAR:
                                   mmsg->Index.Flags &= ~MIEF_BOUNCED;
                                   folder->Data.Flags |= FFF_INDEXMODIFIED;
                                   break;
                         }
                         switch(flagnode->Deleted)
                         {
                              case FNSF_SET:
                                   mmsg->Index.Flags |= MIEF_DELETED;
                                   folder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_UPDATEMAIN);
                                   break;
                              case FNSF_CLEAR:
                                   mmsg->Index.Flags &= ~MIEF_DELETED;
                                   folder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_UPDATEMAIN);
                                   break;
                         }
                         switch(flagnode->DontDelete)
                         {
                              case FNSF_SET:
                                   mmsg->Index.Flags |= MIEF_DONTDELETE;
                                   folder->Data.Flags |= FFF_INDEXMODIFIED;
                                   break;
                              case FNSF_CLEAR:
                                   mmsg->Index.Flags &= ~MIEF_DONTDELETE;
                                   folder->Data.Flags |= FFF_INDEXMODIFIED;
                                   break;
                         }
                         switch(flagnode->Notify)
                         {
                              case FNSF_SET:
                                   mmsg->Index.Flags |= MIEF_NOTIFY;
                                   folder->Data.Flags |= FFF_INDEXMODIFIED;
                                   break;
                              case FNSF_CLEAR:
                                   mmsg->Index.Flags &= ~MIEF_NOTIFY;
                                   folder->Data.Flags |= FFF_INDEXMODIFIED;
                                   break;
                         }

                         UpdateMsgStatus(mmsg);
                         UpdateMsgInfo(mmsg, FALSE);
                    }
               }

               FreeVec(flagnode);
          }
     }
     else
     {
          for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
              mmsg->Node.ln_Succ;
              mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
          {
               LONG val;

               GetListBrowserNodeAttrs((struct Node *)mmsg,
                         LBNA_Selected,      (LONG)&val,
                         TAG_END);
               if(val)
               {
                    SetMsgStatus(mmsg);
                    UpdateMsgStatus(mmsg);
                    UpdateMsgInfo(mmsg, FALSE);
               }
          }
     }
     if(SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      &folder->Msgs,
               LISTBROWSER_AutoFit,     TRUE,
               TAG_END))
          RefreshGList(folder->ListBrowser, folder->Window, NULL, 1);
     if(folder->Data.Flags & FFF_INDEXMODIFIED)
          SetFolderIndexValid(folder, FALSE);
     euc.Flags |= EGF_UPDATEFOLDERS;
     Signal(euc.ThisProcess, SIG_UPDATE);
     BusyPointer(folder->Window, FALSE);
}

void FolderSendNow(struct Folder *folder)
{
     struct MailMsg *mmsg;

     if(CheckBrowserSelections(folder->ListBrowser) > 1)
     {
          if(!IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_DELETESELECTEDMSGS),
                    ISENREQUEST_Button,      LocaleString(ETXT_REQ_YES_NO),
                    TAG_END))
               return;
     }
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      ~0,
               TAG_END);
     for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
         mmsg->Node.ln_Succ;
         mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          LONG val;

          GetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      (ULONG *)&val,
                    TAG_END);
          if(val)
          {
               // set all marked messages to "first-avail" send status
          }
     }
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      &folder->Msgs,
               LISTBROWSER_AutoFit,     TRUE,
               TAG_END);
     euc.Flags |= EGF_UPDATEFOLDERS;
     folder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_UPDATEMAIN);
     SetFolderIndexValid(folder, FALSE);
     Signal(euc.ThisProcess, SIG_UPDATE);
}

void FolderResend(struct Folder *folder)
{
     struct MailMsg *mmsg, *nmsg;

     if(CheckBrowserSelections(folder->ListBrowser) > 1)
     {
          if(!IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_RESENDSELECTEDMSGS),
                    ISENREQUEST_Button,      LocaleString(ETXT_REQ_YES_NO),
                    TAG_END))
               return;
     }
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      ~0,
               TAG_END);
     mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
     while(nmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          LONG val;

          GetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      (ULONG *)&val,
                    TAG_END);
          if(val)
          {
               struct Folder *dfolder;

               // send all marked messages to outgoing folder
               ObtainSemaphore(&euc.Locks.Folders);
               if(dfolder = GetSpecialFolder(FTF_OUTGOING))
               {
                    if(!CopyMsg(mmsg->Index.Filename, dfolder, NULL, 0))
                    {
                         DeleteFile(mmsg->Index.Filename);
                         AdjustMsgCount(dfolder, 1, mmsg->Index.Size, mmsg->Index.Flags, 1);
                         AdjustMsgCount(folder, 1, mmsg->Index.Size, mmsg->Index.Flags, -1);
                         Remove((struct Node *)mmsg);
                         folder->Data.Flags |= FFF_BUILDNEWINDEX;
                         FreeMailMsg(mmsg);
                    }
               }
               ReleaseSemaphore(&euc.Locks.Folders);
          }

          mmsg = nmsg;
     }
     if(SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      &folder->Msgs,
               LISTBROWSER_AutoFit,     TRUE,
               TAG_END))
          if(folder->Window)
            RefreshGList(folder->ListBrowser, folder->Window, NULL, 1);
     euc.Flags |= EGF_UPDATEFOLDERS;
     folder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_UPDATEMAIN);
     SetFolderIndexValid(folder, FALSE);
     Signal(euc.ThisProcess, SIG_UPDATE);
}

void FolderSelectAllMsgs(struct Folder *folder)
{
     struct MailMsg *mmsg;

     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      ~0,
               TAG_END);
     for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
         mmsg->Node.ln_Succ;
         mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          SetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      TRUE,
                    TAG_END);
     }
     if(SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      &folder->Msgs,
               LISTBROWSER_AutoFit,     TRUE,
               TAG_END))
          if(folder->Window)
               RefreshGList(folder->ListBrowser, folder->Window, NULL, 1);

}

void FolderMailMsgResume(struct Folder *folder)
{
     struct MailMsg *mmsg;

     if(CheckBrowserSelections(folder->ListBrowser) > 1)
     {
          if(!IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_MULTIPLEMSGSSELECTED),
                    ISENREQUEST_Button,      LocaleString(ETXT_REQ_YES_NO),
                    TAG_END))
               return;
     }
     BusyPointer(folder->Window, TRUE);
     for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
         mmsg->Node.ln_Succ;
         mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          LONG val;

          GetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      (ULONG *)&val,
                    TAG_END);
          if(val)
          {
                if(ResumePostponedMsg(mmsg))
                {
                     IsenRequest(euc.Screen,
                         ISENREQUEST_Text,   LocaleString(ETXT_UNABLETORESUMEMSG),
                         TAG_END);
                }
                else
                {
                     if(oprefs->IconifySuccessive)
                       folder->Data.Flags |= FFF_ICONIFY;
                }
          }
     }
     BusyPointer(folder->Window, FALSE);
}

void FolderExpunge(struct Folder *folder)
{
     struct MailMsg *mmsg;
     BOOL update = FALSE;

     BusyPointer(folder->Window, TRUE);

/*
     SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      ~0,
               TAG_END);
*/
     for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
         mmsg->Node.ln_Succ;
         mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          if(mmsg->Index.Flags & MIEF_DELETED)
          {
//             LONG flags;

               mmsg->Index.Flags |= MIEF_PURGED;
               mmsg->Flags |= MMF_REMOVE;
/*
               GetListBrowserNodeAttrs((struct Node *)mmsg,
                         LBNA_Flags,         (ULONG *)&flags,
                         TAG_END);
               flags |= LBFLG_HIDDEN;
               LBEditNode(folder->ListBrowser, folder->Window, NULL, (struct Node *)mmsg,
                         LBNA_Flags,         flags,
                         TAG_END);
*/
               UpdateIndexEntry(folder, &mmsg->Index);
//             Remove((struct Node *)mmsg);
               DeleteFile(mmsg->Index.Filename);
               folder->Data.Flags |= FFF_BUILDNEWINDEX;
               update = TRUE;
//             FreeMailMsg(mmsg);
          }
     }
/*
     if(SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
               LISTBROWSER_Labels,      &folder->Msgs,
               LISTBROWSER_AutoFit,     TRUE,
               TAG_END))
          if(folder->Window)
               RefreshGList(folder->ListBrowser, folder->Window, NULL, 1);
*/

     if(update)
          Signal(folder->Process, SIG_CHECK);

     BusyPointer(folder->Window, FALSE);
}

void FolderMailMsgBounce(struct Folder *folder)
{
     struct MailMsg *mmsg;
     LONG mcount = 0;
     UBYTE addr[201];
     BOOL ask = TRUE, mult_ask = TRUE;

     if((mcount = CheckBrowserSelections(folder->ListBrowser)) > 1)
     {
          if(!IsenRequest(euc.Screen,
                    ISENREQUEST_Text,        LocaleString(ETXT_MULTIPLEMSGSSELECTED),
                    ISENREQUEST_Button,      LocaleString(ETXT_REQ_YES_NO),
                    TAG_END))
               return;
     }
     BusyPointer(folder->Window, TRUE);
     for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
         mmsg->Node.ln_Succ;
         mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          LONG val;

          GetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      (ULONG *)&val,
                    TAG_END);
          if(val)
          {
               LONG bwhich;

               BusyPointer(folder->Window, TRUE);

               // ask for address
               if(ask)
               {
                    if(!RequestString(LocaleString(ETXT_ENTERBOUNCEADDRESS), addr, sizeof(addr), 0))
                         break;
               }

               // ask to use same address for all
               if((mcount > 1) && mult_ask)
               {
                    if(!(bwhich = IsenRequest(euc.Screen,
                              ISENREQUEST_Text,        LocaleString(ETXT_USEFORALLBOUNCES),
                              ISENREQUEST_Button,      LocaleString(ETXT_REQ_ALL_ONE_ABORT),
                              ISENREQUEST_Default,     1,
                              TAG_END)))
                         break;

                    if(bwhich == 1)
                         ask = FALSE;

                    mult_ask = FALSE;
               }

               MailMsgBounce(mmsg, addr);

               BusyPointer(folder->Window, FALSE);
          }
     }
     BusyPointer(folder->Window, FALSE);
}

#ifdef EUCALYPTUS_PRO
void FolderMailMsgExport(struct Folder *folder)
{
     struct MailMsg *mmsg;
     Object *getfile;

     if(getfile = NewObject(GETFILE_GetClass(), NULL,
               GETFILE_TitleText,       LocaleString(ETXT_TITLE_EXPORT),
               GETFILE_Drawer,          oprefs->DefaultSavePath,
               GETFILE_DoSaveMode,      TRUE,
               TAG_END))
     {
          if(CA_RequestFile(getfile, folder->Window))
          {
               struct AsyncFile *out_fp;
               STRPTR text;

               BusyPointer(folder->Window, TRUE);

               GetAttr(GETFILE_FullFile, getfile, (ULONG *)&text);

               if(out_fp = OpenAsync(text, MODE_WRITE, ASYNCIO_BUF, SysBase, DOSBase))
               {
                    for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
                        mmsg->Node.ln_Succ;
                        mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
                    {
                         LONG val;

                         GetListBrowserNodeAttrs((struct Node *)mmsg,
                                   LBNA_Selected,      (ULONG *)&val,
                                   TAG_END);
                         if(val)
                         {
                              MailMsgExport(mmsg, out_fp);
                         }
                    }

                    CloseAsync(out_fp);
               }

               BusyPointer(folder->Window, FALSE);
          }

          DisposeObject(getfile);
     }
}
#endif

void FolderMarkMsgs(struct Folder *folder, BYTE type, BOOL all)
{
     struct MailMsg *mmsg;
     BOOL updated = FALSE;

     BusyPointer(folder->Window, TRUE);
     for(mmsg = (struct MailMsg *)folder->Msgs.lh_Head;
         mmsg->Node.ln_Succ;
         mmsg = (struct MailMsg *)mmsg->Node.ln_Succ)
     {
          LONG val;

          GetListBrowserNodeAttrs((struct Node *)mmsg,
                    LBNA_Selected,      (ULONG *)&val,
                    TAG_END);
          if(val || all)
          {
               switch(type)
               {
                    case FMMF_READ:
                         mmsg->Index.Flags &= ~(MIEF_NEW | MIEF_UNREAD);
                         break;
                    case FMMF_UNREAD:
                         mmsg->Index.Flags |= MIEF_UNREAD;
                         break;
                    case FMMF_IMPORTANT:
                         mmsg->Index.Flags |= MIEF_NOTIFY;
                         break;
               }

               UpdateMsgStatus(mmsg);
               UpdateMsgInfo(mmsg, TRUE);
               UpdateIndexEntry(folder, &mmsg->Index);

               updated = TRUE;
          }
     }
     BusyPointer(folder->Window, FALSE);
}
#endif
