/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: folder_select.c
 *   	 Select folder window functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "eucalyptus.h"

struct _SelectFolder
{
	GtkWidget *window,
			*folder_list,
			*okay,
			*cancel;
	Folder *source;
	Folder *selected;
	FolderInfo *finfo;
	glong flags;
	gchar why;
};

gboolean folder_select_open_window(gchar why, glong flags, gpointer data)
{
	SelectFolder *sf = NULL;
	GtkWidget *vbox;

	sf = g_new0(SelectFolder, 1);
	sf->flags = flags;
	switch(sf->why = why)
	{
		case SELECT_FOLDER_COPY:
		case SELECT_FOLDER_MOVE:
			sf->source = (Folder *)data;
			break;

		case SELECT_FOLDER_INFO:
			sf->finfo = (FolderInfo *)data;
			sf->finfo->sf = sf;
			break;
	}

	sf->window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_object_set_user_data(GTK_OBJECT(sf->window), sf);

	gtk_signal_connect(GTK_OBJECT(sf->window), "destroy",
			GTK_SIGNAL_FUNC(folder_select_destroyed), sf);

	vbox = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
	{
		GtkWidget *hbox, *scrw;
		
		scrw = gtk_scrolled_window_new(NULL, NULL);
		{
			sf->folder_list = gtk_ctree_new(1, 0);
			gtk_signal_connect(GTK_OBJECT(sf->folder_list), "tree-select-row",
					GTK_SIGNAL_FUNC(folder_select_select_row), sf);

			gtk_container_add(GTK_CONTAINER(scrw), sf->folder_list);

			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrw),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);
		}
		gtk_box_pack_start(GTK_BOX(vbox), scrw, TRUE, TRUE, 0);

		hbox = gtk_hbox_new(TRUE, GNOME_PAD_SMALL);
		{
			sf->okay = gtk_button_new_with_label(_("Select"));
			gtk_signal_connect(GTK_OBJECT(sf->okay), "clicked",
					GTK_SIGNAL_FUNC(folder_select_okay), sf);
			gtk_box_pack_start(GTK_BOX(hbox), sf->okay, FALSE, TRUE, 0);

			sf->cancel = gtk_button_new_with_label(_("Cancel"));
			gtk_signal_connect(GTK_OBJECT(sf->cancel), "clicked",
					GTK_SIGNAL_FUNC(folder_select_cancel), sf);
			gtk_box_pack_start(GTK_BOX(hbox), sf->cancel, FALSE, TRUE, 0);
		}
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
	}
	gtk_container_add(GTK_CONTAINER(sf->window), vbox);

	{
		GList *node;
		GtkCTreeNode *parent = NULL, *root = NULL;
		gint row = 0;
		gchar *items[2];

		if(flags & SFF_ROOT)
		{
			items[0] = _("<Root>");
			items[1] = NULL;
			
			/* Make a new one.
			 */
			root = gtk_ctree_insert_node(GTK_CTREE(sf->folder_list),
					NULL, NULL,
					items,
					0,
					NULL, NULL, NULL, NULL,
					FALSE, TRUE);
			gtk_ctree_node_set_row_data(GTK_CTREE(sf->folder_list), root, (gpointer)SFOLDER_ROOT);
		}

		for(node = g_list_first(euc.Folders.List); node; node = g_list_next(node))
		{
			Folder *f = (Folder *)node->data;
			GtkCTreeNode *tnode, *p;

			if((f->Data.Type & FTF_POSTPONED) &&
			   !(flags & SFF_INCLUDEPOSTPONED))
				continue;
		
			/* Need to determine hierarchy.
			 */
			parent = NULL;
			if(f->Parent)
			{
				parent = gtk_ctree_find_by_row_data(GTK_CTREE(sf->folder_list), root, f->Parent);
			}
			if(!parent)
				parent = root;
			 
			items[0] = f->Data.Name;
			items[1] = NULL;
			
			/* Make a new one.
			 */
			tnode = gtk_ctree_insert_node(GTK_CTREE(sf->folder_list),
					parent, NULL,
					items,
					0,
					NULL, NULL, NULL, NULL,
					(gboolean)parent, TRUE);
			gtk_ctree_node_set_row_data(GTK_CTREE(sf->folder_list), tnode, f);
		}
	}

	/* Show it.
	 */
	gtk_widget_show_all(sf->window);

	return TRUE;
}

/* Callbacks.
 */
void folder_select_destroyed(GtkWidget *widget, gpointer data)
{
	SelectFolder *sf = (SelectFolder *)data;

	g_free(sf);
}

void folder_select_select_row(GtkCTree *tree, GList *node, gint column, gpointer data)
{
	SelectFolder *sf = (SelectFolder *)data;

	sf->selected = gtk_ctree_node_get_row_data(GTK_CTREE(tree), (GtkCTreeNode *)node);
}

void folder_select_okay(GtkButton *button, gpointer data)
{
	SelectFolder *sf = (SelectFolder *)data;
	Folder *folder = NULL, *sourcef;
	MailMsg *msg;
	GList *node, *next;
	gint row;
	gboolean move = FALSE, update = FALSE;

	folder = sf->selected;

	gtk_widget_unref(sf->window);

	switch(sf->why)
	{
		case SELECT_FOLDER_INFO:
			if(folder == SFOLDER_ROOT)
				memset(sf->finfo->info.BelongsTo, 0, sizeof(sf->finfo->info.BelongsTo));
			else
				strncpy(sf->finfo->info.BelongsTo, folder->Data.Name, sizeof(sf->finfo->info.BelongsTo) - 1);

			euc_update_listview();

			sf->finfo->sf = NULL;
			break;

		case SELECT_FOLDER_MOVE:
			move = TRUE;
			/* drop through */

		case SELECT_FOLDER_COPY:
			sourcef = sf->source;

			node = g_list_first(sourcef->Msgs.List);
			while(node)
			{
				next = g_list_next(node);
				msg = (MailMsg *)node->data;

				if(msg->Flags & MMF_SELECTED)
				{
					gchar *fname, *dir;

					dir = folder_get_full_path(sourcef);
					fname = path_build(dir, msg->Index.Filename, NULL);
					if(mailmsg_copy(fname, folder, NULL, 0))
					{
						update = TRUE;
						if(move)
						{
							mailmsg_remove(msg, NULL);
						}
					}

					g_free(fname);
				     g_free(dir);
				}

				euc_main_iteration();

				node = next;
			}

			if(update)
			{
				if(move)
				{
					folder_update_index(sourcef);
					folder_update_visual(sourcef);
				}

				folder_update_index(folder);
				folder_update_visual(folder);

				euc_update_listview();
			}
			break;
	}
}

void folder_select_cancel(GtkButton *button, gpointer data)
{
	SelectFolder *sf = (SelectFolder *)data;

	gtk_widget_unref(sf->window);

	switch(sf->why)
	{
		case SELECT_FOLDER_INFO:
			sf->finfo->sf = NULL;
			break;
	}
}

