/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: folder_support.c
 *   	 Support functions for folders
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "eucalyptus.h"

gboolean folders_init_required(void)
{
	Folder *folder;
	struct FolderData *fd = NULL;
	gboolean success = TRUE;

	if(fd = (struct FolderData *)g_malloc0(sizeof(struct FolderData)))
	{
		fd->Type = FTF_INCOMING | FTF_SPECIAL;
		fd->SortBy |= (FSF_BYDATE | FSF_THREAD);
		fd->Flags |= FFF_UPDATEMAIN;
		strncpy(fd->Name, "Incoming", sizeof(fd->Name) - 1);
		strncpy(fd->Directory, "Incoming", sizeof(fd->Directory) - 1);

		if(folder = folder_new(fd))
		{
			folder_gather_info(folder, TRUE);
//			folder_update_info(folder);
			list_append(&euc.Folders, folder);
		}
		else
		{
			success = FALSE;
			goto thatsit;
		}

		fd->Type = FTF_POSTPONED | FTF_SPECIAL;
		fd->SortBy |= (FSF_BYDATE | FSF_THREAD);
		fd->Flags |= FFF_UPDATEMAIN;
		strncpy(fd->Name, "Postponed", sizeof(fd->Name) - 1);
		strncpy(fd->Directory, "Postponed", sizeof(fd->Directory) - 1);

		if(folder = folder_new(fd))
		{
			folder_gather_info(folder, TRUE);
//			folder_update_info(folder);
			list_append(&euc.Folders, folder);
		}
		else
		{
			success = FALSE;
			goto thatsit;
		}

		fd->Type = FTF_OUTGOING | FTF_SPECIAL;
		fd->SortBy |= (FSF_BYDATE | FSF_THREAD);
		fd->Flags |= FFF_UPDATEMAIN;
		strncpy(fd->Name, "Outgoing", sizeof(fd->Name) - 1);
		strncpy(fd->Directory, "Outgoing", sizeof(fd->Directory) - 1);

		if(folder = folder_new(fd))
		{
			folder_gather_info(folder, TRUE);
//			folder_update_info(folder);
			list_append(&euc.Folders, folder);
		}
		else
		{
			success = FALSE;
			goto thatsit;
		}

		fd->Type = FTF_SENT | FTF_SPECIAL;
		fd->SortBy |= (FSF_BYDATE | FSF_THREAD);
		fd->Flags |= FFF_UPDATEMAIN;
		strncpy(fd->Name, "Sent", sizeof(fd->Name) - 1);
		strncpy(fd->Directory, "Sent", sizeof(fd->Directory) - 1);

		if(folder = folder_new(fd))
		{
			folder_gather_info(folder, TRUE);
//			folder_update_info(folder);
			list_append(&euc.Folders, folder);
		}
		else
		{
			success = FALSE;
			goto thatsit;
		}
	}
	else
		success = FALSE;

	thatsit:
	if(fd)
		g_free(fd);

	return success;
}

/* Access to the folder list is NOT arbitrated by this function.
 * Calling code must handle this!
 */
Folder *folder_get_special(glong type)
{
	Folder *folder;
	GList *node;

	node = g_list_first(euc.Folders.List);
	while(node)
	{
		folder = (Folder *)node->data;
		
		if(folder->Data.Type & FTF_SPECIAL)
		{
			if(folder->Data.Type & type)
				return(folder);
		}

		node = g_list_next(node);
	}

	return(NULL);
}

Folder *folder_find_by_name(gchar *name)
{
	Folder *folder;
	GList *node;
	
	for(node = g_list_first(euc.Folders.List); node; node = g_list_next(node))
	{
		folder = (Folder *)node->data;

		if(!strcmp(folder->Data.Name, name))
			return folder;
	}
	
	return NULL;
}

glong folder_import_msgs(Folder *folder, const gchar *sf, gboolean truncate)
{
	struct stat st;
	glong count = 0;
	gchar *spool_file = NULL;
	
	g_return_val_if_fail(folder != NULL, 0);

	folder_set_status(folder, _("Importing messages..."));

	spool_file = g_strdup(sf ? sf : folder->Data.SpoolFile);

	if(spool_file && *spool_file)
	{
		FILE *fp, *msg_fp;

		if(!stat(spool_file, &st))
		{
			if(st.st_size <= 0)	// paranoia
			{
				g_free(spool_file);
				return 0;
			}
		}
		else
		{
			g_free(spool_file);
			return 0;
		}

		if(fp = fopen(spool_file, "r"))
		{
			gchar text[1024], fname[FILENAME_MAX];
			
			count++;

			if(folder_get_next_msg_num(folder, NULL, fname, sizeof(fname)))
			{
				msg_fp = fopen(fname, "w");

				if(fgets(text, sizeof(text) - 1, fp))
					if(msg_fp)
						fputs(text, msg_fp);

				while(fgets(text, sizeof(text) - 1, fp))
				{
					if((!strncasecmp(text, "From ", 5)))
					{
						if(msg_fp)
						{    // close last message file
							fclose(msg_fp);
							msg_fp = NULL;
						}
	
						if(folder_get_next_msg_num(folder, NULL, fname, sizeof(fname)))
						{
							msg_fp = fopen(fname, "w");
					
							count++;
						}
					}

					if(msg_fp)
						fputs(text, msg_fp);
				}
			}

			if(msg_fp)
				fclose(msg_fp);

			fclose(fp);
			
			if(truncate)
			{
				if(fp = fopen(spool_file, "w"))
					fclose(fp);
			}
		}

		if(folder->Data.ApplyFiltersOnImport)
		{
			folder_update_index(folder);
			folder_apply_filters(folder);

			euc_update_listview();
		}
	}

	g_free(spool_file);

	return count;
}

void folder_apply_filters(Folder *folder)
{
}

gboolean folder_get_next_msg_num(Folder *folder, gchar *prefix, gchar *buf, glong buflen)
{
	gchar fname[FILENAME_MAX], *dir;
	gboolean success = TRUE;
	FILE *fp;

	g_return_if_fail(folder != NULL);
	g_return_if_fail(buf != NULL);

	if(!prefix)
		prefix = "msg";
		
	dir = folder_get_full_path(folder);

	for(;;)
	{
		sprintf(fname, "%s" G_DIR_SEPARATOR_S "%s%8.8lX", dir, prefix, folder->Data.NextMsgNum++);
		if(fp = fopen(fname, "r"))
			fclose(fp);
		else
		{
			if(errno = ENOENT)
			{
				strncpy(buf, fname, buflen - 1);
				break;
			}

			continue;
		}
	}
	
	g_free(dir);
	
	return success;
}

glong folder_build_index(Folder *folder)
{
	FILE *fp;
	glong error = 0;
	gchar *fname, *dir;
	
	g_return_val_if_fail(folder != NULL, -1);

	/* Set status message.
	 */
	folder_set_status(folder, _("Creating index..."));
	busy_pointer(euc.Window, TRUE);

	/* Get directory and index filename.
	 */
	dir = folder_get_full_path(folder);
	fname = path_build(dir, INDEX_FILE, NULL);

	/* Open the file for writing.
	 */
	if(fp = fopen(fname, "w"))
	{
		struct MatchState *ms;
		MailMsg *msg;
		struct VersInfo vers;
		size_t len = 0;
		
		/* Write version info.
		 */
		vers.VersID = FOLDERINDEXVERSION;
		strncpy(vers.Vers, FOLDERINDEXSTRING, sizeof(vers.Vers) - 1);

		len = fwrite(&vers, sizeof(struct VersInfo), 1, fp);
		if(len < 1)
			goto out;

		if(ms = g_malloc0(sizeof(struct MatchState)))
		{
			errno = match_first(ms, dir, select_new_msg_file);
			while(!errno)
			{
				/* Get the msg struct for this file.
				 */
				if(msg = folder_process_new_msg(folder, ms->entry->d_name, ms->st->st_size, ms->meta))
				{
					/* Write the data.
					 */
					len = fwrite(&msg->Index, sizeof(MailIndexEntry), 1, fp);
					if(len < 1)
						goto out;
				}
			
				errno = match_next(ms);
			}
			match_end(ms);

			g_free(ms);
		}

		out:
		fclose(fp);
	}
	else
	{
		error = errno;
	}
	
	g_free(fname);
	g_free(dir);

	/* Clear status.
	 */
	folder_set_status(folder, NULL);
	busy_pointer(euc.Window, FALSE);

	return error;
}

gboolean folder_update_index(Folder *folder)
{
	gchar *dir, *fname;
	struct MatchState *ms;
	GList *node;
	gboolean updated = FALSE;
	FILE *fp;

	g_return_val_if_fail(folder != NULL, FALSE);

	/* Set status message.
	 */
	folder_set_status(folder, _("Updating index..."));
	busy_pointer(euc.Window, TRUE);
	
	/* Traverse the list, clearing the VERIFIED
	 * flag.
	 */
	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		MailMsg *msg = (MailMsg *)node->data;
		
		msg->Index.Flags &= ~MIEF_VERIFIED;
	}

	/* Get folder's directory.
	 */
	dir = folder_get_full_path(folder);
	
	/* Gather the folder's files, and
	 * check them against what's in the list.
	 */
	if(ms = g_malloc0(sizeof(struct MatchState)))
	{
		errno = match_first(ms, dir, select_new_msg_file);
		while(!errno)
		{
			MailMsg *msg;
			
			/* Check if this entry is in the list.
			 */
			if(msg = folder_find_msg_by_filename(folder, ms->entry->d_name))
			{
				/* Set the VERIFIED flag.
				 */
				msg->Index.Flags |= MIEF_VERIFIED;
			}
			else
			{
				/* This file was not in the list, must be
				 * a new message.
				 */
				if(msg = folder_process_new_msg(folder, ms->entry->d_name, ms->st->st_size, ms->meta))
				{
					updated = TRUE;
					msg->Index.Flags |= MIEF_VERIFIED;
				}
			}
			 
			errno = match_next(ms);
		}
		match_end(ms);

		g_free(ms);
	}
	
	/* Get index filename.
	 */
	fname = path_build(dir, INDEX_FILE, NULL);

	/* Open the file for writing.
	 */
	if(fp = fopen(fname, "w"))
	{
		struct MatchState *ms;
		MailMsg *msg;
		GList *node, *next;
		struct VersInfo vers;
		glong len;

		/* Write version info.
		 */	
		vers.VersID = FOLDERINDEXVERSION;
		strncpy(vers.Vers, FOLDERINDEXSTRING, sizeof(vers.Vers) - 1);

		len = fwrite(&vers, sizeof(struct VersInfo), 1, fp);
		if(len < 1)
			goto out;
			
		/* Walk the list, writing only the folders
		 * the have the VERIFIED flag set, and removing
		 * the ones that don't.
		 */
		node = g_list_first(folder->Msgs.List);
		while(node)
		{
			msg = (MailMsg *)node->data;
			next = g_list_next(node);
			
			if(msg->Index.Flags & MIEF_VERIFIED)
			{
				len = fwrite(&msg->Index, sizeof(msg->Index), 1, fp);
				if(len < 1)
					break;
					
				msg->Index.Flags &= ~MIEF_VERIFIED;
			}
			else
			{
				updated = TRUE;
				folder->Msgs.List = list_remove(&folder->Msgs, node);
				mailmsg_free(msg);
			}
			
			node = next;
		}

		out:
		fclose(fp);
	}

	g_free(fname);
	g_free(dir);

	/* Update the counts.
	 */
	folder_gather_info(folder, FALSE);

	/* Clear folder status bar.
	 */
	folder_set_status(folder, NULL);
	busy_pointer(euc.Window, FALSE);

	return updated;
}

gboolean folder_update_index_entry(Folder *folder, MailMsg *msg)
{
	gboolean success = TRUE;

	/* FIXME */

	return success;
}

MailMsg *folder_process_new_msg(Folder *folder, gchar *msgname, glong size, gchar *meta)
{
	MailMsg *new_msg = NULL;
	struct MatchState *ms;
	gboolean success = FALSE;

	if(!folder_find_msg_by_filename(folder, path_file_part(msgname)))
	{
		MailMsg *mptr, *msg;

		if(mptr = msg = mailmsg_new(folder))
		{
			struct List *list;
			gchar *fname;
			FILE *msg_fp;

			strncpy(msg->Index.Filename, path_file_part(msgname), sizeof(msg->Index.Filename) - 1);
			msg->Index.Size = size;
			
			fname = mailmsg_get_full_path(msg);

//   		if(strstr(meta, "Del;"))
//   			goto msg_out;

			if(msg_fp = fopen(fname, "r"))
			{
				gchar badmsg[51] = {0,};

				if(list = mailmsg_get_header(GMHT_FILE, msg_fp, NULL))
				{
					HdrNode *hnode;

					msg->Index.Flags = MIEF_NEW;

					if(folder->Data.Type & (FTF_SENT | FTF_OUTGOING | FTF_POSTPONED))
					{
						if(hnode = header_find_tag("To", list, 0))
						{
							parse_address(hnode->Data, msg->Index.From, sizeof(msg->Index.From), NULL, 0, TRUE);
							if(!*msg->Index.From)
								parse_address(hnode->Data, NULL, 0, msg->Index.From, sizeof(msg->Index.From), TRUE);
						}
					}
					else
					{
						if(hnode = header_find_tag("From", list, 0))
						{
							parse_address(hnode->Data, msg->Index.From, sizeof(msg->Index.From), NULL, 0, FALSE);
							if(!*msg->Index.From)
								parse_address(hnode->Data, NULL, 0, msg->Index.From, sizeof(msg->Index.From), FALSE);
						}
					}

					if(hnode = header_find_tag("Date", list, 0))
					{
						parse_msg_date(hnode->Data, &msg->Index.Date);
						header_remove_whitespace(hnode->Data, 0);
					}

					if(hnode = header_find_tag("Subject", list, 0))
					{
						strncpy(msg->Index.Subject, hnode->Data, sizeof(msg->Index.Subject) - 1);
						header_remove_whitespace(hnode->Data, 0);
						header_decode(msg->Index.Subject);
					}

					if(hnode = header_find_tag("Content-Type", list, 0))
					{
						if(CONTENTTYPE_MULTIPART == (content_type_verify(hnode->Data) & CONTENTTYPE_MASK))
							msg->Index.Flags |= MIEF_ATTACHMENTS;
					}

					if(!(hnode = header_find_tag("UIDL", list, 0)))
						hnode = header_find_tag("X-UIDL", list, 0);
					if(hnode)
					{
						strncpy(msg->Index.UIDL, hnode->Data, sizeof(msg->Index.UIDL) - 1);
						header_remove_whitespace(hnode->Data, 0);
					}

					if(hnode = header_find_tag("Priority", list, 0))
					{
						g_strdown(hnode->Data);
						if(strstr(hnode->Data, "high") ||
						   strstr(hnode->Data, "urgent"))
							msg->Index.Flags |= MIEF_PRIORITY;
					}

					if(hnode = header_find_tag("To", list, 0))
					{
						if(stristr(hnode->Data, uprefs.Address))
							msg->Index.Flags |= MIEF_PERSONAL;
					}

					if(hnode = header_find_tag("Return-Receipt-To", list, 0))
						msg->Index.Flags |= MIEF_RETURNRECEIPTREQ;

					mailmsg_free_header(list);

					if(meta)
					{
						if(strstr(meta, "Opn;"))
						{
							msg->Index.Flags &= ~(MIEF_NEW | MIEF_UNREAD);
						}
						else if(strstr(meta, "Unr;"))
						{
							msg->Index.Flags &= ~MIEF_NEW;
							msg->Index.Flags |= MIEF_UNREAD;
						}
						if(strstr(meta, "Rep;"))
						{
							msg->Index.Flags |= MIEF_REPLIED;
						}
						if(strstr(meta, "Fwd;"))
						{
							msg->Index.Flags |= MIEF_FORWARDED;
						}
						if(strstr(meta, "Bnc;"))
						{
							msg->Index.Flags |= MIEF_BOUNCED;
						}
						if(strstr(meta, "Del;"))
						{
							msg->Index.Flags |= MIEF_DELETED;
						}
						if(strstr(meta, "NoD;"))
						{
							msg->Index.Flags |= MIEF_DONTDELETE;
						}
						if(strstr(meta, "Not;"))
						{
							msg->Index.Flags |= MIEF_NOTIFY;
						}
						if(strstr(meta, "Per;"))
						{
							msg->Index.Flags |= MIEF_PERSONAL;
						}
						if(strstr(meta, "Pri;"))
						{
							msg->Index.Flags |= MIEF_PRIORITY;
						}
						if(strstr(meta, "Rct;"))
						{
							msg->Index.Flags |= MIEF_RECEIPTED;
						}
						if(strstr(meta, "Edt;"))
						{
							msg->Index.Flags |= MIEF_EDITING;
						}
					}

					mailmsg_update_info(msg, FALSE);
					list_append(&folder->Msgs, msg);
					folder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_UPDATEMAIN);

					if(!strncmp(msg->Index.Filename, "bad", 3) ||
					   !strncmp(msg->Index.Filename, "new", 3))
					{
						gchar *dir, *newname, *oldname;

						dir = folder_get_full_path(folder);
						oldname = path_build(dir, msg->Index.Filename, NULL);
						memcpy(msg->Index.Filename, "msg", 3);
						newname = path_build(dir, msg->Index.Filename, NULL);

						g_free(dir);

						rename(oldname, newname);

						g_free(oldname);
						g_free(newname);
					}

					new_msg = msg;
					msg = NULL;
					success = TRUE;
				}
				else
				{

					gchar *dir, *newname, *oldname;

					dir = folder_get_full_path(folder);
					oldname = path_build(dir, msg->Index.Filename, NULL);
					memcpy(msg->Index.Filename, "bad", 3);
					newname = path_build(dir, msg->Index.Filename, NULL);

					g_free(dir);

					rename(oldname, newname);

					g_free(oldname);
					g_free(newname);

#if 0
					gchar *oldname, *newname, *dir, tmpname[51];

					dir = folder_get_full_path(folder);
					oldname = path_build(dir, msg->Index.Filename, NULL);

					strncpy(tmpname, msg->Index.Filename, sizeof(tmpname) - 1);
					tmpname[0] = 'b';
					tmpname[1] = 'a';
					tmpname[2] = 'd';
					rename(msg->Index.Filename, tmpname);
#endif
				}

				fclose(msg_fp);
			}

			msg_out:
			if(msg)
				mailmsg_free(msg);
				msg = NULL;
		}
	}

	return new_msg;
}

gboolean folder_verify_directory(Folder *folder)
{
	gchar *dir;
	gboolean okay = FALSE;
	
	g_return_val_if_fail(folder != NULL, FALSE);

	dir = folder_get_full_path(folder);
	okay = path_is_dir(dir);
	g_free(dir);
	
	return okay;
}

gboolean folder_verify_uidl(Folder *folder, gchar *UIDL)
{
     MailMsg *msg;
     GList *node;

	g_return_val_if_fail(folder != NULL, TRUE);
	g_return_val_if_fail(UIDL != NULL, TRUE);

	if(!(*UIDL))
		return TRUE;

	node = g_list_first(folder->Msgs.List);
	while(node)
     {
          if(msg->Index.UIDL[0])
          {
	          if(!strcasecmp(msg->Index.UIDL, UIDL))
     	     {
          	     return FALSE;
	          }
	     }
          
          node = g_list_next(node);
     }

     return TRUE;
}

/* Return value must be freed by caller!
 */
gchar *folder_get_full_path(Folder *folder)
{
	gchar *path = NULL;
	
	g_return_val_if_fail(folder != NULL, NULL);

	/* Resolve parentage.
	 */
	if(*folder->Data.BelongsTo && !folder->Parent)
	{
		folder->Parent = folder_find_by_name(folder->Data.BelongsTo);
	}

	if(folder->Parent)
	{
		gchar *ppath;

		ppath = folder_get_full_path(folder->Parent);
		path = path_build(ppath, folder->Data.Directory, NULL);
		g_free(ppath);
	}
	else
	{
		if(g_path_is_absolute(uprefs.MailDir))
			path = path_build(uprefs.MailDir, folder->Data.Directory, NULL);
		else
			path = path_build(g_get_home_dir(), uprefs.MailDir, folder->Data.Directory, NULL);
	}

	return path;
}

gchar *folder_get_file_path(Folder *folder, gchar *file)
{
	gchar *dir, *fname;

	dir = folder_get_full_path(folder);
	fname = path_build(dir, file, NULL);
	g_free(dir);

	return fname;
}

Folder *folder_find_by_dir(const gchar *dir)
{
	GList *node;
	
	g_return_val_if_fail(dir != NULL, NULL);
	
	/* Traverse the list of folders.
	 */
	for(node = g_list_first(euc.Folders.List);
	    node;
	    node = g_list_next(node))
	{
		Folder *folder;
		
		folder = (Folder *)node->data;
		if(!strcmp(folder->Data.Directory, dir))
			return folder;
	}
	
	return NULL;
}

MailMsg *folder_find_msg_by_filename(Folder *folder, const gchar *fname)
{
	MailMsg *msg = NULL;
	GList *node;

	for(node = g_list_first(folder->Msgs.List);
	    node;
	    node = g_list_next(node))
	{
		MailMsg *msg;
		
		msg = (MailMsg *)node->data;
		if(!strcmp(msg->Index.Filename, fname))
			return msg;
	}

	return NULL;
}

glong folder_check_selections(Folder *folder, glong status)
{
	GList *node;
	glong result = FCSR_NONE, yea = 0, nay = 0;

     if(!folder)
		   return FCSR_NONE;

	for(node = g_list_first(folder->Msgs.List); node; node = g_list_next(node))
	{
		MailMsg *msg = (MailMsg *)node->data;

		if(!(msg->Flags & MMF_SELECTED))
			continue;

		if(msg->Index.Flags & status)
			yea++;
		else
			nay++;
	}

	if(yea && nay)
	{
		result = FCSR_MIXED;
	}
	else if(yea && !nay)
	{
		result = FCSR_ALL;
	}

	return result;
}

glong folder_expunge(Folder *folder)
{
	   GList *node, *next;
	   glong count = 0;

	   g_return_val_if_fail(folder != NULL, 0);

	   node = g_list_first(folder->Msgs.List);
	   while(node)
	   {
			 MailMsg *msg;
			 
			 next = g_list_next(node);
			 msg = (MailMsg *)node->data;

			 /* Check for DELETED flag.
			  */
			 if(msg->Index.Flags & MIEF_DELETED)
			 {
				    gchar *fname, *dir;

				    /* Get folder's and msg file's path.
				     */
				    dir = folder_get_full_path(folder);
				    fname = path_build(dir, msg->Index.Filename, NULL);
				    unlink(fname);

				    /* Increment removal count.
				     */
				    count++;

				    /* Remove the node from the list.
				     */
				    folder->Msgs.List = list_remove(&folder->Msgs, msg);
				    mailmsg_free(msg);
			 }
			 
			 /* Get next node.
			  */
			 node = next;
	   }

	   return count;
}

/* Callbacks.
 */
void folder_mark_deleted(gpointer mdata, gpointer user_data)
{
	MailMsg *msg = (MailMsg *)mdata;
	glong disp = GPOINTER_TO_INT(user_data);
	
	if(!(msg->Flags & MMF_SELECTED))
		return;
	
	switch(disp)
	{
		case DELETE_DISPOSITION_DELETE:
			msg->Index.Flags |= MIEF_DELETED;
			break;
			
		case DELETE_DISPOSITION_TOGGLE:
			msg->Index.Flags ^= MIEF_DELETED;
			break;
			
		case DELETE_DISPOSITION_UNDELETE:
			msg->Index.Flags &= ~MIEF_DELETED;
			break;
	}
}
