#ifndef EUCALYPTUS_FOLDERS_H
#define EUCALYPTUS_FOLDERS_H

/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Header: folders.h
 *   	 Folder structure information
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <sys/types.h>
#include <sys/dir.h>

#include "list.h"

/* Definitions.
 */
#define FOLDERDATAVERSION	(0x0106)
#define FOLDERDATAVERS   	".folders 1.2"
#define FOLDERDATASTRING 	"$VER: " FOLDERDATAVERS " " __DATE__

#define FOLDERINDEXVERSION    (0x0107)
#define FOLDERINDEXVERS  	".index 1.4"
#define FOLDERINDEXSTRING	"$VER: " FOLDERINDEXVERS " " __DATE__

/* special folder value */
#define SFOLDER_ROOT		((Folder *)2)

struct FolderData
{
	gulong			 SortBy,
					 Flags,
					 Type,
					 NextMsgNum;
	glong			 MsgCount,
					 NewMsgs,
					 UnreadMsgs,
					 NotifyCount,
					 Size,
					 IMAPPort,
					 MaxMsgs;
	gboolean			 ApplyFiltersOnImport,
					 ProtectMsgs,
					 OpenAtStartup,
					 IMAP;
	gchar			 Name[101],
					 BelongsTo[101],
					 Directory[FILENAME_MAX],
					 SpoolFile[FILENAME_MAX],
					 IMAPServer[51],
					 DefaultTo[51];	/* default send-to address when hitting compose in this folder */
};

/* folder/data SortBy flags */
#define FSF_NONE    	(0)
#define FSF_BYDATE  	(1)
#define FSF_BYNAME  	(2)
#define FSF_BYSUBJECT    (3)
#define FSF_BYSIZE  	(4)
#define FSF_BYSTATUS	(5)
#define FSF_MAXSORT 	(6)

#define FSF_THREAD  	(1L << 30)
#define FSF_REVERSE 	(1L << 31)

#define FSF_SORTMASK	(0x3FFFFFFF)

/* folder/data Types */
#define FTF_TRASH   	(1L << 2) /* not used */
#define FTF_INCOMING	(1L << 3)
#define FTF_OUTGOING	(1L << 4)
#define FTF_POSTPONED    (1L << 5)
#define FTF_SENT    	(1L << 6)
#define FTF_SPECIAL 	(1L << 7)

#define FTF_MASK    	(0xFC)
#define FTF_TYPEMASK	(0x04)
#define FTF_SPECIALMASK  (0x7C)

/* folder/data Flags */
#define FFF_INDEXMODIFIED		(1L << 0) 	/* index was modified */
#define FFF_SUPPRESSUPDATE    	(1L << 1) 	/* don't update when initially opened */
#define FFF_NEWMSGS 			(1L << 2) 	/* folder has new msgs */
#define FFF_ICONIFY 			(1L << 3) 	/* folder should be iconified */
#define FFF_EXPUNGE 			(1L << 4) 	/* ??? */
#define FFF_UPDATEMAIN   		(1L << 5) 	/* should update status in main window */
#define FFF_INITABORTED  		(1L << 6) 	/* abort button pressed during folder init */
#define FFF_ALREADYORGANIZED  	(1L << 7) 	/* folder contents already organized */
#define FFF_HIDDEN  			(1L << 8) 	/* ??? */
#define FFF_REFRESH 			(1L << 9) 	/* folder should be refreshed */
#define FFF_ALREADYCLEANED    	(1L << 10)
#define FFF_BUILDNEWINDEX		(1L << 11)	/* need to save a fresh index to disk */
#define FFF_UPDATE				(1L << 12)	/* update this folder's data on disk (rewrite) */

struct _Folder
{
#if 0
	struct {
		GtkWidget	  	*Layout,
					*MsgList,
					*ScrolledWindow;
	} w;
#endif
	GtkCTreeNode		*TreeNode;
	Folder			*Parent;		/* for hierarchical display, etc. */
	MailMsg		 	*SelectedMsg;
	SelectMsgs		*SelectMsgs;
	pthread_mutex_t	 Mutex;
	struct FolderData    Data;
	struct List		 Msgs,    	/* struct MailMsg * */
					 MsgQueue;
	struct {
		gchar		*WindowTitle,
					*Status,   	/* for folder's status display */
					 Size[21],	/* size string for folder list */
					 Counts[21];	/* <unread>/<total> */
	} t;
	gchar			*work;
	gchar			 ReadingDirection;  /* direction user was going using next/prev (0=none, 1=down, 2=up) */
};

struct _SelectMsgs
{
	GtkWidget		  		*Window,
				  		*Layout,
						*Status,
						*New,
						*Unread,
						*Replied,
						*Forwarded,
						*Bounced,
						*Deleted,
						*Protected,
						*Notified,
						*From,
						*Subject,
						*Size,
						*SizeType,
						*SizeMeas,
						*SubjectThread;
	GList 		   		*NewList,
						*UnreadList,
						*RepliedList,
						*ForwardedList,
						*BouncedList,
						*DeletedList,
						*ProtectedList,
						*NotifiedList,
						*SizeList,
						*SizeMeasList,
						*DateList;
/*	struct FlagNode		 FlagNode; */
	glong 				 Activate,
						 Bytes,
						 SubjThreadVal,
						 SizeTypeVal,
						 SizeMeasVal;
	gboolean				 Done;
	struct {
		gchar			*Status;
	} t;
};

/* folder_check_selections() 
 * result definitions
 */
enum
{
	FCSR_NONE,
	FCSR_MIXED,
	FCSR_ALL
};

/* folder_mark_deleted()
 * disposition definitions
 */
enum 
{
	DELETE_DISPOSITION_DELETE,
	DELETE_DISPOSITION_TOGGLE,
	DELETE_DISPOSITION_UNDELETE
};

struct _FolderInfo
{
	   GtkWidget *window;
	   struct {
		   GtkWidget *name,
		   		   *parent,
				   *spool,
				   *defaultto,
				   *sort,
				   *startup,
				   *reverse,
				   *thread,
				   *applyfilters,
				   *protect;
	   } w;
	   GtkWidget *filesel;
	   Folder *folder;
	   SelectFolder *sf;
	   struct FolderData info;
};

#endif /* EUCALYPTUS_FOLDERS_H */
