/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: gui.c
 *   	 Main window and requester functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>

#include <libgnomeui/gnome-app.h>
#include <libgnomeui/gnome-app-helper.h>

#include "eucalyptus.h"
#include "gui_menus.h"
#include "gui_toolbars.h"

gboolean euc_open_window(void)
{
	GnomeUIInfo *menu, *tb;
	GtkWidget *hbox;
	gchar *titles[] = { N_("Folder"), N_("Count"), N_("Size"), NULL, };
	gboolean success = TRUE;

	if(euc.Window)
		return TRUE;

	/* Create the main window.
	 */		
	euc.Window = gnome_app_new(APPNAME, "Eucalyptus");
	
	/* Connect the signals.
	 */
	gtk_signal_connect(GTK_OBJECT(euc.Window), "delete_event", 
			GTK_SIGNAL_FUNC(euc_quit_requested_cb), NULL);
	
	/* Add menus.
	 */
	gnome_app_create_menus(GNOME_APP(euc.Window), main_menu);
	 
	/* Add the toolbar.
	 */
	gnome_app_create_toolbar(GNOME_APP(euc.Window), main_toolbar);

	/* Add the status bar.
	 */
	gnome_app_set_statusbar(GNOME_APP(euc.Window), 
			euc.StatusBar = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_USER));
	
	/* Add other stuff to the window.
	 */
	euc.Layout = gtk_vbox_new(FALSE, 5);
	gnome_app_set_contents(GNOME_APP(euc.Window), euc.Layout);

	euc.Pane = gtk_hpaned_new();
	gtk_paned_set_position(GTK_PANED(euc.Pane), 150 /* FIXME: user-definable position */ );
	gtk_box_pack_start(GTK_BOX(euc.Layout), euc.Pane, TRUE, TRUE, 0);

	/* Create folder list tree.
	 */
	euc.ScrolledWindow = gtk_scrolled_window_new(NULL, NULL);
	{
		titles[0] = _(titles[0]); 
		titles[1] = _(titles[1]);
		titles[2] = _(titles[2]);
		euc.FolderList = gtk_ctree_new_with_titles(3, 0, titles);
		gtk_clist_set_column_auto_resize(GTK_CLIST(euc.FolderList), 0, TRUE);

		gtk_signal_connect(GTK_OBJECT(euc.FolderList), "tree-select-row",
				GTK_SIGNAL_FUNC(euc_select_folder), NULL);
		gtk_signal_connect(GTK_OBJECT(euc.FolderList), "drag-end",
				GTK_SIGNAL_FUNC(euc_folders_drag_end_cb), NULL);

		/* Add to scrolled window.
		 */
		gtk_container_add(GTK_CONTAINER(euc.ScrolledWindow), euc.FolderList);

		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(euc.ScrolledWindow),
				GTK_POLICY_AUTOMATIC,	/* horizontal */
				GTK_POLICY_AUTOMATIC);	/* vertical */
	}
	gtk_paned_add1(GTK_PANED(euc.Pane), euc.ScrolledWindow);

	/* Populate it.
	 */
	euc_update_listview(); 
	
	/* Set window position.
	 */
	if(prefs.MainWindowLeft)
		gtk_widget_set_uposition(euc.Window, 
				prefs.MainWindowLeft, 
				prefs.MainWindowTop);
	if(prefs.MainWindowWidth)
		gtk_window_set_default_size(GTK_WINDOW(euc.Window),
				prefs.MainWindowWidth,
				prefs.MainWindowHeight);
	else
		gtk_window_set_default_size(GTK_WINDOW(euc.Window), 
				DEFAULT_WINDOW_WIDTH, 
				DEFAULT_WINDOW_HEIGHT);

	/* Set window class.
	 */
	gtk_window_set_wmclass(GTK_WINDOW(euc.Window), "Eucalyptus", "Main window");
	 
	/* Show it.
	 */
	gtk_widget_show_all(euc.Window);

	return success;
}

void euc_close_window(void)
{
}

void populate_main_listview(gpointer f_data, gpointer data)
{
	GtkCTreeNode *parent = NULL;
	Folder *folder = (Folder *)f_data;
	gchar *items[4];

	/* Need to determine hierarchy.
	 */
	if(folder->Parent)
	{
fprintf(stderr, "Folder (%s) has parent (%s)\n", folder->Data.Name, folder->Parent->Data.Name);
		parent = gtk_ctree_find_by_row_data(GTK_CTREE(euc.FolderList), NULL, folder->Parent);
fprintf(stderr, "parent = %x\n", parent);
	}	 
	 
	/* Set up the column items.
	 */
	folder_update_info(folder);
	
	items[0] = folder->Data.Name;
	items[1] = folder->t.Counts;
	items[2] = folder->t.Size;
	items[3] = NULL;

	/* Make a new one.
	 */
	if(parent)
		gtk_ctree_set_node_info(GTK_CTREE(euc.FolderList), parent,
				folder->Parent->Data.Name, 0,
				NULL, NULL, NULL, NULL,
				FALSE,
				TRUE);
	if(folder->TreeNode = gtk_ctree_insert_node(GTK_CTREE(euc.FolderList),
			parent, NULL,
			items,
			0,
			NULL, NULL, NULL, NULL,
			(gboolean)parent, TRUE))
	{
		gtk_ctree_node_set_row_data(GTK_CTREE(euc.FolderList), folder->TreeNode, folder);
		if(folder->Data.NotifyCount + 
		   folder->Data.NewMsgs + 
		   folder->Data.UnreadMsgs)
			gtk_ctree_node_set_foreground(GTK_CTREE(euc.FolderList), 
					folder->TreeNode, &prefs.ImportantPen);
	}
}

void euc_update_listview(void)
{
	if(!euc.FolderList)
		return;

	/* Prevent updates.
	 */
	gtk_clist_freeze(GTK_CLIST(euc.FolderList));
	
	/* Clear the list.
	 */
	gtk_clist_clear(GTK_CLIST(euc.FolderList));

	/* Sort the folder list.
	 */
	euc.Folders.List = g_list_sort(euc.Folders.List, euc_compare_folders);

	/* Repopulate it.
	 */
	g_list_foreach(euc.Folders.List, populate_main_listview, NULL);

	/* Resume updates.
	 */
	gtk_clist_thaw(GTK_CLIST(euc.FolderList));
}

gint euc_compare_folders(gconstpointer f1, gconstpointer f2)
{
	return strcasecmp(((Folder *)f1)->Data.Name, ((Folder *)f2)->Data.Name);
}

void euc_select_folder(GtkCTree *tree, GList *node, gint column, gpointer data)
{
	Folder *folder;

	/* Set selected folder.
	 */
	folder = (Folder *)gtk_ctree_node_get_row_data(tree, (GtkCTreeNode *)node);

	/* Check for right-click first.
	 */
	/* FIXME */
	
	
	/* Check for a change in the selection.
	 */
	if(folder == euc.SelectedFolder)
		return;
	
	folder_open(folder);
}

void busy_pointer(GtkWidget *window, gboolean busy)
{
#if 0
	GdkCursor *cursor;
	
	if(!window)
		return;

	cursor = gdk_cursor_new(busy ? GDK_CLOCK : GDK_LAST_CURSOR);

	if(cursor)
		gdk_window_set_cursor(window->window, cursor);
#endif
}

/* Callbacks.
 */
void euc_about_requested_cb(GtkWidget *widget, gpointer data)
{
	GtkWidget *hbox, *href;
	const gchar *authors[] = {"Paul A. Schifferer <gandalf@isengard-dev.org>", NULL, };
	
	if(euc.About)
	{
		gtk_widget_show_all(euc.About);
		return;
	}
	
	if(euc.About = gnome_about_new(
			"Eucalyptus",
			VERSION,
			"Copyright (c) 1997-2000 Isengard Developments",
			authors,
			_("Advanced MIME e-mail application"),
			"eucalyptus-logo.png"))
	{
		gtk_signal_connect(GTK_OBJECT(euc.About), "destroy",
				GTK_SIGNAL_FUNC(gtk_widget_destroyed), &euc.About);

		hbox = gtk_hbox_new(TRUE, 0);
		href = gnome_href_new("http://www.isengard-dev.org/eucalyptus/",
					 _("Eucalyptus home page"));
		gtk_box_pack_start(GTK_BOX(hbox), href, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(euc.About)->vbox), hbox, FALSE, TRUE, 0);
				
		gtk_widget_show_all(euc.About);
	}
}

void euc_open_folder_cb(GtkWidget *widget, gpointer data)
{
	if(!euc.SelectedFolder)
		return;

	errno = 0;

	if(!folder_open(euc.SelectedFolder) && errno)
	{
		gchar *str;
		
		str = g_strdup_printf(_("Unable to open folder window.\n%s"), strerror(errno));
		euc_request(euc.Window,
				_("Eucalyptus error"),
				str,
				euc_ok_button, 0,
				EUCRF_ASYNC);
		g_free(str);
	}
}

void euc_new_folder_cb(GtkWidget *widget, gpointer data)
{
	folder_info(NULL);

	euc_update_listview();
}

void euc_edit_folder_cb(GtkWidget *widget, gpointer data)
{
	folder_info(euc.SelectedFolder);

	euc_update_listview();
}

void euc_close_folder_cb(GtkWidget *widget, gpointer data)
{
	if(!euc.SelectedFolder)
		return;

	folder_close(euc.SelectedFolder);
}

void euc_delete_folder_cb(GtkWidget *widget, gpointer data)
{
	   const gchar *buttons[] = { N_("Folder & contents"), N_("Folder only"), N_("Cancel"), NULL, };
	   gboolean delete_dir = FALSE;

	   if(euc.SelectedFolder->Data.Type & FTF_SPECIAL)
	   {
		   euc_request(euc.Window,
				   _("Eucalyptus warning"),
				   _("You cannot delete this folder!"),
				   euc_ok_button, 0,
				   EUCRF_ASYNC);
		   return;
	   }
	   
	   switch(!euc_request(euc.Window,
						  _("Eucalyptus request"),
						  _("Do you want to delete this folder's contents,\n"
								"or just the folder entry?"),
				    buttons, 2,
				    0))
	   {
			 case 0:	/* both */
				    delete_dir = TRUE;
				    /* drop through */

			 case 1:	/* folder only */
				    folder_free(euc.SelectedFolder, FALSE, delete_dir);
				    break;
	   }
}

void euc_refresh_folders_cb(GtkWidget *widget, gpointer data)
{
	Folder *folder;
	   
	folder_update_index(euc.SelectedFolder);
	folder_update_visual(euc.SelectedFolder);
	euc_update_listview();
}

void euc_refresh_folders_all_cb(GtkWidget *widget, gpointer data)
{
	GList *node;
	gboolean updated = FALSE;

	for(node = g_list_first(euc.Folders.List); node; node = g_list_next(node))
	{
		Folder *folder;

		folder = (Folder *)node->data;
		if(folder_update_index(folder))
		{
			folder_update_visual(folder);
			updated = TRUE;
		}
	}

	if(updated)
		   euc_update_listview();
}

void euc_save_folders_cb(GtkWidget *widget, gpointer data)
{
}

void pop_open_retriever_cb(GtkWidget *widget, gpointer data)
{
}

void smtp_open_sender_cb(GtkWidget *widget, gpointer data)
{
	smtp_open_window();
}

void euc_execute_macro_cb(GtkWidget *widget, gpointer data)
{
}

void euc_open_compose_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp;

	if(comp = compose_new())
	{
		list_append(&euc.Composes, comp);

		compose_open_window(comp);
	}
}

void euc_open_compose_form_cb(GtkWidget *widget, gpointer data)
{
	FileReq *req;
	gchar *fname;

	fname = path_build(g_get_home_dir(), ".eucalyptus", EUCALYPTUS_FORMS_DIR, NULL);	
	req = euc_file_request_new(FILEREQ_COMPOSE_FORM, _("Select form for compose"), fname);
	g_free(fname);
}

void euc_open_compose_addr_cb(GtkWidget *widget, gpointer data)
{	
	Composition *comp;

	if(comp = compose_new())
	{
		list_append(&euc.Composes, comp);

		compose_open_window(comp);
	}
}

void euc_bug_report_cb(GtkWidget *widget, gpointer data)
{	
	Composition *comp;

	if(comp = compose_new())
	{
		/* Add bug-report address.
		 * FIXME
		 */


		list_append(&euc.Composes, comp);

		compose_open_window(comp);
	}
}

void euc_folders_drag_end_cb(GtkWidget *widget, GdkDragContext *context, gpointer data)
{
	GtkWidget *source;

	source = gtk_drag_get_source_widget(context);
}

