/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: gui_compose.c
 *   	 GUI-related functions for composing
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <sys/stat.h>

#include "eucalyptus.h"
#include "gui_menus.h"
#include "gui_toolbars.h"

gboolean compose_open_window(Composition *comp)
{
	GtkWidget *label, *hbox, *table, *scrw;
	glong i;
	gchar *tabs[] = { N_("Addresses"), N_("Attachments"), N_("Security"), N_("Misc."), NULL, };
	gboolean success = TRUE;

	g_return_val_if_fail(comp != NULL, FALSE);
	
	if(comp->Window)
	{
		gtk_widget_show_all(comp->Window);
		return TRUE;
	}
	
	/* Create the main window.
	 */
	comp->Window = gnome_app_new(APPNAME, _("Compose message"));
	
	/* Connect the signals.
	 */
	gtk_signal_connect(GTK_OBJECT(comp->Window), "delete_event", 
			GTK_SIGNAL_FUNC(compose_close_requested_cb), comp);
	gtk_signal_connect(GTK_OBJECT(comp->Window), "destroy", 
			GTK_SIGNAL_FUNC(compose_destroy_cb), comp);
	
	/* Add menus.
	 */
	gnome_app_create_menus_with_data(GNOME_APP(comp->Window), compose_menu, comp);
	 
	/* Add the toolbar.
	 */
	gnome_app_create_toolbar_with_data(GNOME_APP(comp->Window), compose_toolbar, comp);
	
	/* Add other stuff to the window.
	 */
	comp->Layout = gtk_vbox_new(FALSE, 5);
	gnome_app_set_contents(GNOME_APP(comp->Window), comp->Layout);

	/* Create notebook pages and contents.
	 */
	comp->w.Pager = gtk_notebook_new();

	for(i=0; tabs[i]; i++)
	{
		GtkWidget *page, *tab;
	
		tab = comp->w.Tab[i] = gtk_label_new(_(tabs[i]));

		switch(i)
		{
			GtkWidget *label;
			
			case 0:	// addresses
				page = comp->w.Page[i] = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
				{
					GtkWidget *table;
					
					table = gtk_table_new(3, 3, FALSE);
					{
						GtkWidget *label;
						
						/* To:
						 */
						label = gtk_label_new("To:");
						gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
						gtk_table_attach(GTK_TABLE(table), label,
								0, 1, 0, 1,
								GTK_FILL, GTK_FILL,
								2, 2);
								
						comp->w.AddrTo = gtk_entry_new();
						gtk_entry_set_text(GTK_ENTRY(comp->w.AddrTo), comp->str.To->str);
						gtk_table_attach(GTK_TABLE(table), comp->w.AddrTo,
								1, 2, 0, 1,
								GTK_FILL | GTK_EXPAND, GTK_FILL,
								2, 2);
								
						comp->w.AddrToSel = gtk_button_new_with_label("...");
						gtk_signal_connect(GTK_OBJECT(comp->w.AddrToSel), "clicked",
								GTK_SIGNAL_FUNC(compose_select_to_cb), comp);
						gtk_table_attach(GTK_TABLE(table), comp->w.AddrToSel,
								2, 3, 0, 1,
								GTK_FILL, GTK_FILL,
								2, 2);

						/* Cc:
						 */
						label = gtk_label_new("Cc:");
						gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
						gtk_table_attach(GTK_TABLE(table), label,
								0, 1, 1, 2,
								GTK_FILL, GTK_FILL,
								2, 2);
								
						comp->w.AddrCc = gtk_entry_new();
						gtk_entry_set_text(GTK_ENTRY(comp->w.AddrCc), comp->str.Cc->str);
						gtk_table_attach(GTK_TABLE(table), comp->w.AddrCc,
								1, 2, 1, 2,
								GTK_FILL | GTK_EXPAND, GTK_FILL,
								2, 2);
						
						comp->w.AddrCcSel = gtk_button_new_with_label("...");
						gtk_signal_connect(GTK_OBJECT(comp->w.AddrCcSel), "clicked",
								GTK_SIGNAL_FUNC(compose_select_cc_cb), comp);
						gtk_table_attach(GTK_TABLE(table), comp->w.AddrCcSel,
								2, 3, 1, 2,
								GTK_FILL, GTK_FILL,
								2, 2);

						/* Bcc:
						 */
						label = gtk_label_new("Bcc:");
						gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
						gtk_table_attach(GTK_TABLE(table), label,
								0, 1, 2, 3,
								GTK_FILL, GTK_FILL,
								2, 2);
								
						comp->w.AddrBcc = gtk_entry_new();
						gtk_entry_set_text(GTK_ENTRY(comp->w.AddrBcc), comp->str.Bcc->str);
						gtk_table_attach(GTK_TABLE(table), comp->w.AddrBcc,
								1, 2, 2, 3,
								GTK_FILL | GTK_EXPAND, GTK_FILL,
								2, 2);
						
						comp->w.AddrBccSel = gtk_button_new_with_label("...");
						gtk_signal_connect(GTK_OBJECT(comp->w.AddrBccSel), "clicked",
								GTK_SIGNAL_FUNC(compose_select_bcc_cb), comp);
						gtk_table_attach(GTK_TABLE(table), comp->w.AddrBccSel,
								2, 3, 2, 3,
								GTK_FILL, GTK_FILL,
								2, 2);
					}
					gtk_box_pack_start(GTK_BOX(page), table, TRUE, TRUE, GNOME_PAD_SMALL);

				}
				
//				compose_populate_address_list(comp);
				break;
				
			case 1:	// attachments
				page = comp->w.Page[i] = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
				{
					GtkWidget *vbox, *scrw;
					gchar *titles[] = { N_("Name"), N_("Type"), N_("Encoding"), NULL, };
				
					scrw = gtk_scrolled_window_new(NULL, NULL);
					{
						   glong i;

						comp->w.AttachList = gtk_clist_new_with_titles(3, titles);
						for(i=0; i<3; i++)
							   gtk_clist_set_column_auto_resize(GTK_CLIST(comp->w.AttachList), i, TRUE);
						gtk_signal_connect(GTK_OBJECT(comp->w.AttachList), "select-row",
								GTK_SIGNAL_FUNC(compose_select_att_cb), comp);
								
						/* Attach to scrolled window.
						 */
						gtk_container_add(GTK_CONTAINER(scrw), comp->w.AttachList);
							
						gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrw),
								GTK_POLICY_NEVER,		/* horizontal */
								GTK_POLICY_AUTOMATIC);	/* vertical */
					}
					gtk_box_pack_start(GTK_BOX(page), scrw, TRUE, TRUE, 0);
					
					vbox = gtk_vbox_new(TRUE, GNOME_PAD_SMALL);
					{	// buttons
						comp->w.NewAtch = gtk_button_new_with_label(_("New"));
						gtk_signal_connect(GTK_OBJECT(comp->w.NewAtch), "clicked",
								GTK_SIGNAL_FUNC(compose_add_att_cb), comp);
						gtk_box_pack_start(GTK_BOX(vbox), comp->w.NewAtch, FALSE, FALSE, 0);
						
						comp->w.EditAtch = gtk_button_new_with_label(_("Edit"));
						gtk_box_pack_start(GTK_BOX(vbox), comp->w.EditAtch, FALSE, FALSE, 0);
						gtk_widget_set_sensitive(comp->w.EditAtch, FALSE);
						
						comp->w.DelAtch = gtk_button_new_with_label(_("Del"));
						gtk_box_pack_start(GTK_BOX(vbox), comp->w.DelAtch, FALSE, FALSE, 0);
						gtk_widget_set_sensitive(comp->w.DelAtch, FALSE);
						
						label = gtk_label_new("");
						gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
					}
					gtk_box_pack_start(GTK_BOX(page), vbox, FALSE, FALSE, 0);
				}
				
				compose_populate_attachment_list(comp);
				break;
				
			case 2:	// security
				page = comp->w.Page[i] = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
				{
					GtkWidget *vbox;
					
					comp->w.Sign = gtk_check_button_new_with_label(_("Sign?"));
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(comp->w.Sign), comp->Flags & CF_SIGN);
					gtk_signal_connect(GTK_OBJECT(comp->w.Sign), "toggled",
								 GTK_SIGNAL_FUNC(compose_sign_toggled_cb), comp);
					gtk_box_pack_start(GTK_BOX(page), comp->w.Sign, FALSE, TRUE, 0);
					
					comp->w.Encrypt = gtk_check_button_new_with_label(_("Encrypt?"));
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(comp->w.Encrypt), comp->Flags & CF_ENCRYPT);
					gtk_signal_connect(GTK_OBJECT(comp->w.Encrypt), "toggled",
								 GTK_SIGNAL_FUNC(compose_encrypt_toggled_cb), comp);
					gtk_box_pack_start(GTK_BOX(page), comp->w.Encrypt, FALSE, TRUE, 0);

					comp->w.EncryptTo = gtk_entry_new();
					gtk_entry_set_text(GTK_ENTRY(comp->w.EncryptTo), comp->t.EncryptTo);
					gtk_box_pack_start(GTK_BOX(page), comp->w.EncryptTo, FALSE, TRUE, 0);
					
					comp->w.EncryptToSelf = gtk_check_button_new_with_label(_("Encrypt to self?"));
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(comp->w.EncryptToSelf), comp->Flags & CF_ENCRYPTTOSELF);
					gtk_signal_connect(GTK_OBJECT(comp->w.EncryptToSelf), "toggled",
								 GTK_SIGNAL_FUNC(compose_encrypttoself_toggled_cb), comp);
					gtk_box_pack_start(GTK_BOX(page), comp->w.EncryptToSelf, FALSE, TRUE, 0);
				}
				break;
				
			case 3:	// misc.
				page = comp->w.Page[i] = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
				{
					GtkWidget *table, *hbox;
					guint top = 0;

					table = gtk_table_new(3, 3, FALSE);
					{
						comp->w.ReplyTo = gtk_entry_new();
						gtk_entry_set_text(GTK_ENTRY(comp->w.ReplyTo), comp->t.ReplyTo);
						prefs_layout_table_item(table, comp->w.ReplyTo, _("Reply to"), &top, 0);

						comp->w.UseTagline = gtk_check_button_new_with_label(_("Use tagline?"));
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(comp->w.UseTagline), comp->Flags & CF_USETAGLINE);
						gtk_signal_connect(GTK_OBJECT(comp->w.UseTagline), "clicked",
								GTK_SIGNAL_FUNC(compose_use_tagline_cb), comp);
						gtk_table_attach(GTK_TABLE(table), comp->w.UseTagline,
								0, 1, 1, 2,
								GTK_FILL, GTK_FILL,
								GNOME_PAD_SMALL, GNOME_PAD_SMALL);
						
						hbox = gtk_hbox_new(FALSE, 0);
						{
							comp->w.Tagline = gtk_entry_new();
							gtk_entry_set_text(GTK_ENTRY(comp->w.Tagline), comp->t.Tagline);
							gtk_widget_set_sensitive(comp->w.Tagline, comp->Flags & CF_USETAGLINE);
							gtk_box_pack_start(GTK_BOX(hbox), comp->w.Tagline, TRUE, TRUE, 0);
							
							comp->w.NewTagline = gtk_button_new_with_label(_("New"));
							gtk_widget_set_sensitive(comp->w.NewTagline, comp->Flags & CF_USETAGLINE);
							gtk_signal_connect(GTK_OBJECT(comp->w.NewTagline), "clicked",
									GTK_SIGNAL_FUNC(compose_get_new_tagline_cb), comp);
							gtk_box_pack_start(GTK_BOX(hbox), comp->w.NewTagline, FALSE, TRUE, 0);
						}
						gtk_table_attach(GTK_TABLE(table), hbox,
								1, 2, 1, 2,
								GTK_FILL | GTK_EXPAND, 
								GTK_FILL,
								GNOME_PAD_SMALL, GNOME_PAD_SMALL);

						comp->w.UseSigFile = gtk_check_button_new_with_label(_("Use signature?"));
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(comp->w.UseSigFile), comp->Flags & CF_USESIGFILE);
						gtk_signal_connect(GTK_OBJECT(comp->w.UseSigFile), "clicked",
								GTK_SIGNAL_FUNC(compose_use_sigfile_cb), comp);
						gtk_table_attach(GTK_TABLE(table), comp->w.UseSigFile,
								0, 1, 2, 3,
								GTK_FILL, GTK_FILL,
								GNOME_PAD_SMALL, GNOME_PAD_SMALL);

						comp->w.SignatureFile = gnome_file_entry_new("signature", _("Select signature file"));
						gtk_widget_set_sensitive(comp->w.SignatureFile, comp->Flags & CF_USESIGFILE);
						gnome_file_entry_set_default_path(GNOME_FILE_ENTRY(comp->w.SignatureFile), comp->t.SignatureFile);
						gtk_table_attach(GTK_TABLE(table), comp->w.SignatureFile,
								1, 2, 2, 3,
								GTK_FILL | GTK_EXPAND, 
								GTK_FILL,
								GNOME_PAD_SMALL, GNOME_PAD_SMALL);

						/* Table, column 3
						 */
						comp->w.CustomHeaders = gtk_button_new_with_label(_("Custom headers"));
						gtk_signal_connect(GTK_OBJECT(comp->w.CustomHeaders), "clicked",
								GTK_SIGNAL_FUNC(compose_custom_headers_cb), comp);
						gtk_table_attach(GTK_TABLE(table), comp->w.CustomHeaders,
								2, 3, 0, 1,
								GTK_FILL, GTK_FILL,
								GNOME_PAD_SMALL, GNOME_PAD_SMALL);

						comp->w.ReturnReceipt = gtk_check_button_new_with_label(_("Request return receipt?"));
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(comp->w.ReturnReceipt), comp->Flags & CF_RETURNRECEIPT);
						   gtk_signal_connect(GTK_OBJECT(comp->w.ReturnReceipt), "toggled",
									    GTK_SIGNAL_FUNC(compose_receipt_toggled_cb), comp);
						gtk_table_attach(GTK_TABLE(table), comp->w.ReturnReceipt,
								2, 3, 1, 2,
								GTK_FILL, GTK_FILL,
								GNOME_PAD_SMALL, GNOME_PAD_SMALL);

					}
					gtk_box_pack_start(GTK_BOX(page), table, TRUE, TRUE, 0);
				}
				break;
		}

		gtk_notebook_append_page(GTK_NOTEBOOK(comp->w.Pager), page, tab);
	}

	gtk_box_pack_start(GTK_BOX(comp->Layout), comp->w.Pager, FALSE, TRUE, 0);
	 
	/* Create reply-to and subject widgets.
	 */
	table = gtk_table_new(1, 2, FALSE);
	{
		guint top = 0;

		comp->w.Subject = gtk_entry_new();
		gtk_entry_set_text(GTK_ENTRY(comp->w.Subject), comp->t.Subject);
		prefs_layout_table_item(table, comp->w.Subject, _("Subject"), &top, 0);
	}
	gtk_box_pack_start(GTK_BOX(comp->Layout), table, FALSE, TRUE, 0);
	 
	/* Create text entry widget.
	 */
	scrw = gtk_scrolled_window_new(NULL, NULL);
	{
		comp->w.Textfield = gtk_text_new(NULL, NULL);
		gtk_text_set_editable(GTK_TEXT(comp->w.Textfield), TRUE);
		
		/* Attach to scrolled window.
		 */
		gtk_container_add(GTK_CONTAINER(scrw), comp->w.Textfield);
		
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrw),
				GTK_POLICY_AUTOMATIC,	/* horizontal */
				GTK_POLICY_AUTOMATIC);	/* vertical */
	}
	gtk_box_pack_start(GTK_BOX(comp->Layout), scrw, TRUE, TRUE, 0);
	
	/* Load the appropriate text.
	 */
	if(*comp->t.Filename)
	{
		struct stat *st;
		gchar *buf;
		FILE *fp;

		if(st = path_stat(comp->t.Filename))
		{
			buf = g_malloc0(st->st_size + 1);
			if(fp = fopen(comp->t.Filename, "r"))
			{
				fread(buf, 1, st->st_size, fp);
				fclose(fp);

				gtk_text_set_point(GTK_TEXT(comp->w.Textfield), gtk_text_get_length(GTK_TEXT(comp->w.Textfield)));
				gtk_text_insert(GTK_TEXT(comp->w.Textfield), 
						NULL, // font
						NULL, NULL, // fore,back color
						buf, -1);
			}
		}
	}

 	if(prefs.ComposeWindowLeft)
		gtk_widget_set_uposition(comp->Window,
			prefs.ComposeWindowLeft,
			prefs.ComposeWindowTop);
	if(prefs.ComposeWindowWidth)
		gtk_widget_set_usize(comp->Window,
			prefs.ComposeWindowWidth,
			prefs.ComposeWindowHeight);
	else
		gtk_widget_set_usize(comp->Window,
			DEFAULT_COMPOSE_WINDOW_WIDTH,
			DEFAULT_COMPOSE_WINDOW_HEIGHT);

	/* Show it.
	 */
	gtk_widget_show_all(comp->Window);
	
	return success;
}

#if 0
static void compose_build_addr_string(gpointer node_data, gpointer data)
{
	Composition *comp;
	CompAddress *caddr;
	glong i;
	gchar *cols[2];
	gchar *caddr_type[] = { "To", "Cc", "Bcc", };

	comp = (Composition *)data;
	caddr = (CompAddress *)node_data;

	/* Clear out the columns.
	 */		
	memset(cols, 0, sizeof(cols));

	/* Build the display string.
	 */
	if(*caddr->Address)
		g_snprintf(caddr->Display, sizeof(caddr->Display) - 1, "%s: %s <%s>", caddr_type[caddr->Type], caddr->Name, caddr->Address);
	else
		g_snprintf(caddr->Display, sizeof(caddr->Display) - 1, "%s: %s", caddr_type[caddr->Type], caddr->Name);

	cols[0] = caddr->Display;
		
	/* Add it to the list.
	 */
	gtk_clist_append(GTK_CLIST(comp->w.AddressList), cols);
	
	/* Add some data.
	 */
	i = g_list_index(comp->Recipients.List, caddr);
	gtk_clist_set_row_data(GTK_CLIST(comp->w.AddressList), i, caddr);

	if(caddr->Flags & CAF_SELECTED)
		gtk_clist_select_row(GTK_CLIST(comp->w.AddressList), i, 0);
}
#endif

void compose_populate_address_list(Composition *comp)
{
	GList *node;
	gchar *str;
	glong i = 0;

	if(!comp)
		return;

	str = g_strdup("blah");
	g_free(str);
#if 0
	/* Freeze and clear the widget.
	 */		
	gtk_clist_freeze(GTK_CLIST(comp->w.AddressList));
	gtk_clist_clear(GTK_CLIST(comp->w.AddressList));
		
	/* Walk the list.
	 */
	g_list_foreach(comp->Recipients.List, compose_build_addr_string, comp);

	/* Update some other widgets.
	 */
	gtk_widget_set_sensitive(comp->w.ClearAddrs, (gboolean)g_list_length(comp->Recipients.List));
	
	/* Restore updates.
	 */
	gtk_clist_thaw(GTK_CLIST(comp->w.AddressList));
#endif

}

void compose_populate_attachment_list(Composition *comp)
{
	GList *node;
	
	if(!comp)
		return;

	/* Freeze and clear the widget.
	 */		
	gtk_clist_freeze(GTK_CLIST(comp->w.AttachList));
	gtk_clist_clear(GTK_CLIST(comp->w.AttachList));
		
	/* Walk the list.
	 */
	for(node = g_list_first(comp->Attachments.List); node; node = g_list_next(node))
	{
		CompAttachment *catt = (CompAttachment *)node->data;
		gchar *cols[3];
		gchar *encodings[] = { "Base64/QP", NULL, };

		/* Clear out the columns.
		 */		
		memset(cols, 0, sizeof(cols));
		
		/* Build the display strings.
		 */
		cols[0] = catt->Name;
		cols[1] = catt->ContentType;
		cols[2] = encodings[catt->Encoding];
					
		/* Add it to the list.
		 */
		gtk_clist_append(GTK_CLIST(comp->w.AttachList), cols);
	}
	
	/* Restore updates.
	 */
	gtk_clist_thaw(GTK_CLIST(comp->w.AttachList));
}

void compose_populate_custom_headers_list(Composition *comp)
{
}

gboolean compose_file_req(Composition *comp, gchar type)
{
	gchar *title;

	if(comp->file_sel.Dialog)
	{
		euc_request(comp->Window,
				_("Eucalyptus information"),
				_("This compose window's file requester is\n"
				  "already open for another file request.\n"
				  "Please close it first and try again."),
				euc_ok_button, 0,
				EUCRF_ASYNC);
		return FALSE;
	}
		
	switch(type)
	{
		case COMPOSE_SAVE_FILE:
			title = _("Save compose text");
			break;
			
		case COMPOSE_INSERTFILE_FORM:
			title = _("Select form to insert");
			break;
			
		default:
			title = _("Select file to insert");
			break;
	}
	
	comp->file_sel.Dialog = gtk_file_selection_new(title);
	comp->file_sel.Type = type;
	
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(comp->file_sel.Dialog)->ok_button), "clicked",
			GTK_SIGNAL_FUNC(compose_filesel_okay_cb), comp);
			
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(comp->file_sel.Dialog)->ok_button), "clicked",
			GTK_SIGNAL_FUNC(gtk_widget_destroy), comp->file_sel.Dialog);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(comp->file_sel.Dialog)->cancel_button), "clicked",
			GTK_SIGNAL_FUNC(gtk_widget_destroy), comp->file_sel.Dialog);

	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(comp->file_sel.Dialog)->ok_button), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed), &comp->file_sel.Dialog);
	
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(comp->file_sel.Dialog), comp->t.Filename);

	gtk_widget_show(comp->file_sel.Dialog);
}

/* Callbacks.
 */
void compose_destroy_cb(GtkWidget *window, gpointer data)
{
	Composition *comp = (Composition *)data;

	comp->Window = NULL;
	comp->Flags |= CF_DONE;

	if(comp->att_info.Window)
		   gtk_widget_unref(comp->att_info.Window);
}

void compose_filesel_okay_cb(GtkButton *button, gpointer data)
{
	Composition *comp = (Composition *)data;
	gchar *filename;

	filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(comp->file_sel.Dialog));
	strncpy(comp->file_sel.Filename, filename, sizeof(comp->file_sel.Filename) - 1);
	
	switch(comp->file_sel.Type)
	{
		case COMPOSE_INSERTFILE_NORMAL:
		case COMPOSE_INSERTFILE_BRACKETED:
		case COMPOSE_INSERTFILE_INDENTED:
			compose_insert_file(comp, comp->file_sel.Type, filename);
			break;

		case COMPOSE_INSERTFILE_FORM:
			compose_insert_form(comp, filename);
			break;

		case COMPOSE_INSERTFILE_UUENCODED:
			compose_insert_file_uuencoded(comp, filename);
			break;
			
		case COMPOSE_SAVE_FILE:
			compose_save_file(comp, filename);
			break;
	}
}

gint compose_close_requested_cb(GtkWidget *window, GdkEventAny *e, gpointer data)
{
	/* Ask.
	 */
	if(euc_request(window, 
			_("Eucalyptus: Close composition"),
			_("This message has not yet been sent;\nclose anyway?"),
			euc_ok_cancel_buttons, 1,
			0))
		return TRUE;

	/* Return FALSE, to destroy the window.
	 */
	return FALSE;
}
 
void compose_send_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	/* Attempt to write the message.
	 */
	if(compose_write_msg(comp, NULL))
	{
		/* Destroy the window.
		 */
		gtk_widget_destroy(comp->Window);

		/* Update the Outgoing folder.
		 */
		folder_update_index(folder_get_special(FTF_OUTGOING));
		euc_update_listview();
	}
	else if(errno)
	{
		gchar *errmsg;
		
		errmsg = g_strdup_printf(_("Unable to create message.\n%s"), strerror(errno));
		euc_request(comp->Window,
				_("Eucalyptus error"),
				errmsg, 
				euc_ok_button, 0,
				0);
				
		g_free(errmsg);
	}
}

void compose_queue_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;
	QueueInfo *qi;
	
	if(qi = g_malloc0(sizeof(QueueInfo)))
	{
		/* Get queue info.
		 * FIXME: NOT IMPLEMENTED!
		 */
		 
		 
		/* Write the message.
		 */
		if(compose_write_msg(comp, qi))
		{
			gtk_widget_destroy(comp->Window);
			
			/* Update the folder.
			 */
			folder_update_index(folder_get_special(FTF_OUTGOING));
		}
		else if(errno)
		{
			gchar *errmsg;
			
			errmsg = g_strdup_printf(_("Unable to create message.\n%s"), strerror(errno));
			euc_request(comp->Window,
					_("Eucalyptus error"),
					errmsg, 
					euc_ok_button, 0,
					0);
					
			g_free(errmsg);
		}
		
		g_free(qi);
	}
}

void compose_postpone_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	/* Write the draft.
	 */
	if(compose_postpone_msg(comp))
	{
		gtk_widget_destroy(comp->Window);
		
		/* Update the folder.
		 */
		folder_update_index(folder_get_special(FTF_POSTPONED));
	}
	else if(errno)
	{
		gchar *errmsg;
		
		errmsg = g_strdup_printf(_("Unable to create message.\n%s"), strerror(errno));
		euc_request(comp->Window,
				_("Eucalyptus error"),
				errmsg, 
				euc_ok_button, 0,
				0);
				
		g_free(errmsg);
	}
}

void compose_cancel_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	/* Ask.
	 */
	if(!euc_request(comp->Window, 
			_("Eucalyptus: Close composition"),
			_("This message has not yet been sent;\nclose anyway?"),
			euc_ok_cancel_buttons, 1,
			0))
	{
		gtk_widget_destroy(comp->Window);
	}
}

void compose_insert_file_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	compose_file_req(comp, COMPOSE_INSERTFILE_NORMAL);
}

void compose_insert_file_bracketed_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	compose_file_req(comp, COMPOSE_INSERTFILE_BRACKETED);
}

void compose_insert_file_indented_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	compose_file_req(comp, COMPOSE_INSERTFILE_INDENTED);
}

void compose_insert_form_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	compose_file_req(comp, COMPOSE_INSERTFILE_FORM);
}

void compose_insert_file_uuencoded_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	compose_file_req(comp, COMPOSE_INSERTFILE_UUENCODED);
}

void compose_undo_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

}

void compose_redo_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

}

void compose_cut_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	gtk_editable_cut_clipboard(GTK_EDITABLE(comp->w.Textfield));
}

void compose_copy_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	gtk_editable_copy_clipboard(GTK_EDITABLE(comp->w.Textfield));
}

void compose_paste_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	gtk_editable_paste_clipboard(GTK_EDITABLE(comp->w.Textfield));
}

void compose_clear_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	if(!euc_request(comp->Window,
			_("Eucalyptus query"),
			_("This operation cannot be undone!\nContinue?"),
			euc_yes_no_buttons, 1,
			0))
	{
		gtk_editable_delete_text(GTK_EDITABLE(comp->w.Textfield), 0, -1);
	}
}

void compose_erase_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	gtk_editable_delete_selection(GTK_EDITABLE(comp->w.Textfield));
}

void compose_save_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;
	
	compose_file_req(comp, COMPOSE_SAVE_FILE);
}

void compose_plain_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

}

void compose_bold_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

}

void compose_italic_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

}

void compose_underline_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

}

void compose_select_color_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

}

void compose_left_align_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

}

void compose_center_align_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

}

void compose_right_align_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

}

void compose_spellcheck_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

}

void compose_defcheck_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

}

void compose_get_new_tagline_cb(GtkWidget *widget, gpointer data)
{
	Composition *comp = (Composition *)data;

	compose_get_tagline(comp);
}

void compose_use_tagline_cb(GtkToggleButton *toggle, gpointer data)
{
	Composition *comp = (Composition *)data;
	gboolean active;
	
	active = gtk_toggle_button_get_active(toggle);
	
	if(active)
		comp->Flags |= CF_USETAGLINE;
	else
		comp->Flags &= ~CF_USETAGLINE;
	
	gtk_widget_set_sensitive(comp->w.Tagline, active);
	gtk_widget_set_sensitive(comp->w.NewTagline, active);
}

void compose_use_sigfile_cb(GtkToggleButton *toggle, gpointer data)
{
	Composition *comp = (Composition *)data;
	gboolean active;
	
	active = gtk_toggle_button_get_active(toggle);
	
	if(active)
		comp->Flags |= CF_USESIGFILE;
	else
		comp->Flags &= ~CF_USESIGFILE;
	
	gtk_widget_set_sensitive(comp->w.SignatureFile, active);
}

void compose_sign_toggled_cb(GtkToggleButton *toggle, gpointer data)
{
	Composition *comp = (Composition *)data;
	gboolean active;
	
	active = gtk_toggle_button_get_active(toggle);
	
	if(active)
		comp->Flags |= CF_SIGN;
	else
		comp->Flags &= ~CF_SIGN;
}

void compose_encrypt_toggled_cb(GtkToggleButton *toggle, gpointer data)
{
	Composition *comp = (Composition *)data;
	gboolean active;
	
	active = gtk_toggle_button_get_active(toggle);
	
	if(active)
		comp->Flags |= CF_ENCRYPT;
	else
		comp->Flags &= ~CF_ENCRYPT;
}

void compose_encrypttoself_toggled_cb(GtkToggleButton *toggle, gpointer data)
{
	Composition *comp = (Composition *)data;
	gboolean active;
	
	active = gtk_toggle_button_get_active(toggle);
	
	if(active)
		comp->Flags |= CF_ENCRYPTTOSELF;
	else
		comp->Flags &= ~CF_ENCRYPTTOSELF;
}

void compose_receipt_toggled_cb(GtkToggleButton *toggle, gpointer data)
{
	Composition *comp = (Composition *)data;
	gboolean active;
	
	active = gtk_toggle_button_get_active(toggle);
	
	if(active)
		comp->Flags |= CF_RETURNRECEIPT;
	else
		comp->Flags &= ~CF_RETURNRECEIPT;
}

/* Addresses.
 */
#if 0
void compose_select_addr_cb(GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data)
{
	Composition *comp = (Composition *)data;
	CompAddress *caddr = gtk_clist_get_row_data(list, row);
	GList *node;
	gboolean sel = FALSE;
	
	caddr->Flags |= CAF_SELECTED;

	node = g_list_first(comp->Recipients.List);
	while(node)
	{
		caddr = (CompAddress *)node->data;
		
		if(caddr->Flags & CAF_SELECTED)
			sel = TRUE;
	
		node = g_list_next(node);
	}
	
	gtk_widget_set_sensitive(comp->w.DelAddr, sel);
}

void compose_unselect_addr_cb(GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data)
{
	Composition *comp = (Composition *)data;
	CompAddress *caddr = gtk_clist_get_row_data(list, row);
	GList *node;
	gboolean sel = FALSE;
	
	caddr->Flags &= ~CAF_SELECTED;

	node = g_list_first(comp->Recipients.List);
	while(node)
	{
		caddr = (CompAddress *)node->data;
		
		if(caddr->Flags & CAF_SELECTED)
			sel = TRUE;
	
		node = g_list_next(node);
	}
	
	gtk_widget_set_sensitive(comp->w.DelAddr, sel);
}
#endif

void compose_select_to_cb(GtkButton *button, gpointer data)
{
	Composition *comp = (Composition *)data;

	address_select_open_window(comp, CAT_TO, TRUE);
}

void compose_select_cc_cb(GtkButton *button, gpointer data)
{
	Composition *comp = (Composition *)data;

	address_select_open_window(comp, CAT_CC, TRUE);
}

void compose_select_bcc_cb(GtkButton *button, gpointer data)
{
	Composition *comp = (Composition *)data;

	address_select_open_window(comp, CAT_BCC, TRUE);
}

/* Attachments.
 */
void compose_add_att_cb(GtkButton *button, gpointer data)
{
	Composition *comp = (Composition *)data;

	if(comp->att_info.Window)
	{
		/* Bring the window to the front.
		 */
		gtk_widget_show_all(comp->att_info.Window);

		return;
	}
	
	/* Create an attachment info window.
	 */
	compose_create_attachment_window(comp, NULL);
}

void compose_edit_att_cb(GtkButton *button, gpointer data)
{
	Composition *comp = (Composition *)data;

	if(comp->att_info.Window)
	{
		/* Bring the window to the front.
		 */
		gtk_widget_show_all(comp->att_info.Window);

		return;
	}
	
	/* Create an attachment info window.
	 */
	compose_create_attachment_window(comp, comp->SelectedAttachment);
}

void compose_select_att_cb(GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data)
{
	Composition *comp = (Composition *)data;
	
	comp->SelectedAttachment = g_list_nth_data(comp->Attachments.List, row);

	gtk_widget_set_sensitive(comp->w.EditAtch, !!comp->SelectedAttachment);
	gtk_widget_set_sensitive(comp->w.DelAtch, !!comp->SelectedAttachment);
}

void compose_unselect_att_cb(GtkCList *list, gint row, gint column, GdkEventButton *event, gpointer data)
{
	Composition *comp = (Composition *)data;
	
	comp->SelectedAttachment = NULL;

	gtk_widget_set_sensitive(comp->w.EditAtch, FALSE);
	gtk_widget_set_sensitive(comp->w.DelAtch, FALSE);
}

/* Custom headers.
 */
void compose_custom_headers_cb(GtkButton *button, gpointer data)
{
	Composition *comp = (Composition *)data;

}



#if 0
						comp->w.CustomHeadersList = gtk_clist_new(1);
						gtk_clist_set_selection_mode(GTK_CLIST(comp->w.CustomHeadersList), GTK_SELECTION_MULTIPLE);
						gtk_box_pack_start(GTK_BOX(vbox), comp->w.CustomHeadersList, TRUE, TRUE, 0);
					
						hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
						{
							GtkWidget *label;
							
							label = gtk_label_new(_("Custom headers"));
							gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
							
							comp->w.AddCustHdr = gtk_button_new_with_label(_("Add"));
							gtk_box_pack_start(GTK_BOX(hbox), comp->w.AddCustHdr, FALSE, TRUE, 0);

							comp->w.DelCustHdr = gtk_button_new_with_label(_("Del"));
							gtk_widget_set_sensitive(comp->w.DelCustHdr, FALSE);
							gtk_box_pack_start(GTK_BOX(hbox), comp->w.DelCustHdr, FALSE, TRUE, 0);
						}
						gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
#endif

