/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: gui_pop.c
 *   	 POP visuals
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <netinet/in.h>
#include <netdb.h>
#include <gtk/gtk.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "eucalyptus.h"

gboolean pop_open_window(Server *server)
{
	gboolean success = TRUE, dotry = FALSE;

	if(!server->w.Window)
	{
		GtkWidget *hbox, *scrw, *table;

		server->w.Window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		g_snprintf(server->t.Title, sizeof(server->t.Title), _("Eucalyptus: %s"), server->Host);
		gtk_window_set_title(GTK_WINDOW(server->w.Window), server->t.Title);

		server->w.Layout = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
		gtk_container_add(GTK_CONTAINER(server->w.Window), server->w.Layout);

		  hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
		  {
			  server->w.Status = gtk_label_new(server->t.Status);
			  gtk_label_set_justify(GTK_LABEL(server->w.Status), GTK_JUSTIFY_LEFT);
			  gtk_misc_set_alignment(GTK_MISC(server->w.Status), 0.0, 0.5);
			  gtk_box_pack_start(GTK_BOX(hbox), server->w.Status, TRUE, TRUE, GNOME_PAD_SMALL);

			  server->w.Stop = gtk_button_new_with_label(_("Stop"));
			  gtk_signal_connect(GTK_OBJECT(server->w.Stop), "clicked",
						   GTK_SIGNAL_FUNC(pop_stop_cb), server);
			  gtk_widget_set_sensitive(server->w.Stop, FALSE);
			  gtk_box_pack_start(GTK_BOX(hbox), server->w.Stop, FALSE, TRUE, GNOME_PAD_SMALL);
		  }
		  gtk_box_pack_start(GTK_BOX(server->w.Layout), hbox, TRUE, TRUE, GNOME_PAD_SMALL);
		  
		  server->w.Progress = gtk_progress_bar_new();
		  gtk_progress_bar_set_bar_style(GTK_PROGRESS_BAR(server->w.Progress), GTK_PROGRESS_DISCRETE);
		  gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(server->w.Progress), GTK_PROGRESS_LEFT_TO_RIGHT);
		  gtk_box_pack_start(GTK_BOX(server->w.Layout), server->w.Progress, TRUE, TRUE, GNOME_PAD_SMALL);
		  
		  gtk_widget_set_usize(server->w.Window, 300, -1);
		  
		  pop_control.Flags |= SCF_USINGSUMMARY;

		gtk_signal_connect(GTK_OBJECT(server->w.Window), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed), &server->w.Window);
	}

	gtk_widget_show_all(server->w.Window);

 	pop_populate_progress_window(server);

	return success;
}

void pop_close_window(Server *server)
{
fprintf(stderr, "pop_close_window()\n");
	if(server->w.Window)
		gtk_widget_destroy(server->w.Window);
fprintf(stderr, "server->w.Window = %x\n", server->w.Window);
		server->w.Window = NULL;
fprintf(stderr, "done closing pop window\n");
}

void pop_populate_progress_window(Server *server)
{
	GList *node;
	
	if(!server->w.Window ||
	   !server->w.MsgList)
		return;
		
	gtk_clist_freeze(GTK_CLIST(server->w.MsgList));
	gtk_clist_clear(GTK_CLIST(server->w.MsgList));

	for(node = g_list_first(server->Messages.List); node; node = g_list_next(node))
	{
		ServerEntry *entry = (ServerEntry *)node->data;
		gchar *text[5];

#if 0
		QueuedMail *qm = (QueuedMail *)node->data;
		gchar *text[5];
		
		text[0] = qm->to;
		text[1] = qm->date;
		text[2] = qm->subject;
		
		if(qm->size_str)
			g_free(qm->size_str);
		qm->size_str = g_strdup_printf("%d", qm->size);
		text[3] = qm->size_str;

		text[4] = qm->status;
		
		gtk_clist_append(GTK_CLIST(server->w.MsgList), text);
#endif
	}

	gtk_clist_thaw(GTK_CLIST(server->w.MsgList));
}

void pop_update_progress(Server *server, ProgressMsg *pmsg)
{
	gfloat percent = 0;
fprintf(stderr, "pop_update_progress()\n");
	if(!server ||
	   !server->w.Window ||
	   !pmsg)
		return;

fprintf(stderr, "current = %d, total = %d\n", pmsg->current, pmsg->total);
	if(pmsg->total)
	{
		   percent = (gfloat)pmsg->current / (gfloat)pmsg->total;
fprintf(stderr, "percent = %f\n", percent);

		   gtk_progress_bar_update(GTK_PROGRESS_BAR(server->w.Progress), percent);
	}
fprintf(stderr, "text = '%s'\n", pmsg->text);
	if(*pmsg->text)
		gtk_label_set_text(GTK_LABEL(server->w.Status), pmsg->text);
}

/* Callbacks.
 */
void pop_stop_cb(GtkWidget *widget, gpointer data)
{
	Server *server = (Server *)data;

	server->Flags |= PCF_STOP;
}

