/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: gui_smtp.c
 *   	 SMTP visuals
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <netinet/in.h>
#include <netdb.h>
#include <gtk/gtk.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "eucalyptus.h"

gboolean smtp_open_window(void)
{
	gboolean success = TRUE, dotry = FALSE;

	if(!smtp_control.w.Window)
	{
		GtkWidget *hbox, *scrw, *table;

		smtp_control.w.Window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW(smtp_control.w.Window), _("Eucalyptus: Sending messages"));

		smtp_control.w.Layout = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
		gtk_container_add(GTK_CONTAINER(smtp_control.w.Window), smtp_control.w.Layout);

		  hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
		  {
			  smtp_control.w.Status = gtk_label_new(smtp_control.t.Status);
			  gtk_label_set_justify(GTK_LABEL(smtp_control.w.Status), GTK_JUSTIFY_LEFT);
			  gtk_misc_set_alignment(GTK_MISC(smtp_control.w.Status), 0.0, 0.5);
			  gtk_box_pack_start(GTK_BOX(hbox), smtp_control.w.Status, TRUE, TRUE, GNOME_PAD_SMALL);

			  smtp_control.w.Stop = gtk_button_new_with_label(_("Stop"));
			  gtk_signal_connect(GTK_OBJECT(smtp_control.w.Stop), "clicked",
						   GTK_SIGNAL_FUNC(smtp_stop_cb), NULL);
			  gtk_widget_set_sensitive(smtp_control.w.Stop, FALSE);
			  gtk_box_pack_start(GTK_BOX(hbox), smtp_control.w.Stop, FALSE, TRUE, GNOME_PAD_SMALL);
		  }
		  gtk_box_pack_start(GTK_BOX(smtp_control.w.Layout), hbox, TRUE, TRUE, GNOME_PAD_SMALL);
		  
		  smtp_control.w.Progress = gtk_progress_bar_new();
		  gtk_progress_bar_set_bar_style(GTK_PROGRESS_BAR(smtp_control.w.Progress), GTK_PROGRESS_DISCRETE);
		  gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(smtp_control.w.Progress), GTK_PROGRESS_LEFT_TO_RIGHT);
		  gtk_box_pack_start(GTK_BOX(smtp_control.w.Layout), smtp_control.w.Progress, TRUE, TRUE, GNOME_PAD_SMALL);
		  
		  gtk_widget_set_usize(smtp_control.w.Window, 300, -1);
		  
		  smtp_control.Flags |= SCF_USINGSUMMARY;

		gtk_signal_connect(GTK_OBJECT(smtp_control.w.Window), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed), &smtp_control.w.Window);
	}

	gtk_widget_show_all(smtp_control.w.Window);

// 	smtp_populate_progress_window();

	return success;
}

void smtp_close_window(void)
{
	if(smtp_control.w.Window)
		gtk_widget_destroy(smtp_control.w.Window);
		smtp_control.w.Window = NULL;
}

void smtp_populate_progress_window(void)
{
	GList *node;
	
	if(!smtp_control.w.Window ||
	   !smtp_control.w.MsgList)
		return;
		
	gtk_clist_freeze(GTK_CLIST(smtp_control.w.MsgList));
	gtk_clist_clear(GTK_CLIST(smtp_control.w.MsgList));

	for(node = g_list_first(smtp_control.MsgList.List); node; node = g_list_next(node))
	{
		QueuedMail *qm = (QueuedMail *)node->data;
		gchar *text[5];
		
		text[0] = qm->to;
		text[1] = qm->date;
		text[2] = qm->subject;
		
		if(qm->size_str)
			g_free(qm->size_str);
		qm->size_str = g_strdup_printf("%d", qm->size);
		text[3] = qm->size_str;

		text[4] = qm->status;
		
		gtk_clist_append(GTK_CLIST(smtp_control.w.MsgList), text);
	}

	gtk_clist_thaw(GTK_CLIST(smtp_control.w.MsgList));
}

void smtp_update_progress(ProgressMsg *pmsg)
{
	gfloat percent = 0;

	if(!smtp_control.w.Window ||
	   !pmsg)
		return;

	if(pmsg->total)
	{
		   percent = pmsg->current / pmsg->total;

		   gtk_progress_bar_update(GTK_PROGRESS_BAR(smtp_control.w.Progress), percent);
	}
	if(*pmsg->text)
		gtk_label_set_text(GTK_LABEL(smtp_control.w.Status), pmsg->text);
}

/* Callbacks.
 */
void smtp_stop_cb(GtkWidget *widget, gpointer data)
{
	   smtp_control.Flags |= SCF_STOP;
}

