/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: headers.c
 *         Msg header misc. functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "eucalyptus.h"

void header_remove_comments(gchar *text)
{
     glong x = 0, start = -1, count = 0;
     gboolean quoting = FALSE;

     while(text[x])
     {
          switch(text[x])
          {
               case '(':
                    if(quoting)
                         break;
                    count++;
                    if(start == -1)
                         start = x;
                    break;
               case ')':
                    if(quoting)
                         break;
                    count--;
                    if(!count)
                    {
                         gchar *n;

                         text[start] = '\0';
                         n = &text[x + 1];

                         strcat(text, n);
                    }
                    break;
               case '\\':
                    x++;
                    break;
               case '\"':
                    quoting = !quoting;
                    break;
          }

          x++;
     }
}

void header_remove_whitespace(gchar *text, gint16 flags)
{
     glong x = 0;
     gboolean gotaspace = FALSE, quoting = FALSE;

     while(text[x])
     {
          switch(text[x])
          {
               glong i;

               case '\"':
                    quoting = !quoting;
                    break;
               case '\n':
               case '\r':
                    if(quoting)
                         break;
                    gotaspace = TRUE;
                    for(i=x; i<strlen(text); i++)
                         text[i] = text[i + 1];
                    break;
               case '\t':
               case ' ':
                    if(quoting)
                         break;
                    if(flags & RWSF_ONLYCRLF)
                         break;

                    // condense spaces
                    if(gotaspace ||
                       !x)    // leading spaces
                    {
                         for(i=x; i<strlen(text); i++)
                              text[i] = text[i + 1];
                    }
                    gotaspace = TRUE;

                    // eliminate spaces between atoms
                    if(!(flags & RWSF_NOATOMS))
                    {
                         if(x &&
                            (text[x - 1] == '.') ||
                            (text[x - 1] == '@'))
                         {
                              for(i=x; i<strlen(text); i++)
                                   text[i] = text[i + 1];
                         }
                    }
                    break;
               default:
                    gotaspace = FALSE;
                    break;
          }

          x++;
     }

     // get rid of trailing whitespace
     if(x = strlen(text))
     {
          x--;
          while((text[x] == ' ') ||
                (text[x] == '\r') ||
                (text[x] == '\n') ||
                (text[x] == '\t'))
               text[x--] = '\0';
     }
}

void header_remove_quotes(gchar *text)
{
     glong x = 0;
     gboolean escaped = FALSE;

     while(text[x])
     {
          switch(text[x])
          {
               glong i;

               case '\\':
                    escaped = TRUE;
                    for(i=x; text[i]; i++)
                         text[i] = text[i + 1];
                    x--;
                    break;

               case '\"':
                    if(!escaped)
                    {
                         for(i=x; text[i]; i++)
                              text[i] = text[i + 1];
                         x--;
                    }

               default:
                    escaped = FALSE;
                    break;
          }

          x++;
     }
}

HdrNode *header_find_tag(gchar *tag, struct List *list, gint16 flags)
{
	GList *node;
     HdrNode *hdr = NULL;

     if(!list ||
	   !list->List)
          return NULL;

	for(node = g_list_first(list->List); node; node = g_list_next(node))
	{
		hdr = (HdrNode *)node->data;

          if(!strcasecmp(hdr->Tag, tag))
          {
               return hdr;
          }
     }

     return NULL;
}

void header_decode(gchar *text)
{
     glong l;
     gchar work[2048], storage[2048], *p, ch[5];

     if(!text)
          return;
     if(!(l = strlen(text)))
          return;

	memset(work, 0, sizeof(work));
	strncpy(storage, text, sizeof(storage) - 1);

	   p = storage;

	   while(*p)
	   {
		  if(*p == '=')
		  {
			  if(*(p + 1) == '?')
			  {
				  glong Qs = 0;
				  gchar *ec_end = NULL, *np;
				  gboolean qlast = FALSE, out = FALSE;

				  np = p + 2;
				  while(*np)
				  {
					  switch(*np)
					  {
						  case '?':
							  Qs++;
							  qlast = TRUE;
							  break;
						  case '=':
							  if(qlast &&
								(Qs == 3))
							  {
								  out = TRUE;
								  break;
							  }
						  default:
							  qlast = FALSE;
							  break;
					  }

					  if(out)
						  break;

					  np++;
				  }

				  if(!out)
					  goto a_char;

				  if(ec_end = (np - 1))
				  {
					  gchar *charset = NULL, *encoding = NULL, *data = NULL, *c;

					  *ec_end = '\0';
					  charset = p + 2;

					  if(c = strchr(charset, '?'))
					  {
						  *c = '\0';
						  encoding = c + 1;

						  if(c = strchr(encoding, '?'))
						  {
							  *c = '\0';
							  data = c + 1;
						  }
					  }

					  if(encoding &&
						data)
					  {
						  if(!strcasecmp(charset, "iso-8859-1") ||
							!strcasecmp(charset, "us-ascii"))
						  {
							  glong x = 0;

							  if(!strcasecmp(encoding, "Q"))
							  {
								  while(data[x])
								  {
									  gchar hex[3] = {0,};
									  char ct[5];

									  switch(data[x])
									  {
										  glong val;

										  case '=':
											  if(!data[x + 1])
												  break;
											  hex[0] = data[x + 1];
											  if(!data[x + 2])
												  break;
											  hex[1] = data[x + 2];
											  hex[2] = 0;
											  if(isxdigit(hex[0]) &&
												isxdigit(hex[1]))
											  {
												  val = strtol(hex, NULL, 16);
												  if(val)
												  {
													  sprintf(ct, "%c", (gchar)val);
													  strncat(work, ct, sizeof(work) - 1);
												  }

												  x += 2;
											  }
											  break;
										  default:
											  sprintf(ct, "%c", data[x]);
											  strncat(work, ct, sizeof(work) - 1);
											  break;
									  }

									  x++;
								  }
							  }
							  else if(!strcasecmp(encoding, "B"))
							  {
								  strncat(work, "?", sizeof(work) - 1);
	   /*
								  while(data[x])
								  {
									  x++;
								  }
	   */
							  }
						  }
					  }

					  p = ec_end + 2;
				  }
			  }
			  else
			  {
			  }
		  }

		  a_char:
		  sprintf(ch, "%c", *p);
		  strncat(work, ch, sizeof(work) - 1);

		  p++;
	   }

	   strcpy(text, work);
}

gboolean header_encode(FILE *fp, gchar *hdr)
{
	glong x, l, count = 0;
	gboolean encode = FALSE;

	if(!fp ||
	   !hdr)
		return FALSE;

	l = strlen(hdr);
	for(x=0; x<l; x++)
	{
		if(((guchar)hdr[x]) > 127)
		{
			encode = TRUE;
			break;
		}
	}

	if(encode)
	{
		fprintf(fp, "=?ISO-8859-1?Q?");
		count += 15;
		for(x=0; x<l; x++)
		{
			if(((guchar)hdr[x]) > 127)
			{
				fprintf(fp, "=%02.2X", hdr[x]);
				count += 3;
			}
			else
			{
				if(hdr[x] == ' ')
				{
					fprintf(fp, "=20");
					count += 3;
				}
				else
				{
					fputc(hdr[x], fp);
					count++;
				}
			}
		}
		fprintf(fp, "?=");
		count += 2;
	}
	else
	{
		fprintf(fp, hdr);
		count += strlen(hdr);
	}
	
	fflush(fp);

	return(count);
}

gboolean header_write_quoted_printable(FILE *fp, gchar *text, glong size, glong begin)
{
	glong x, lpos = begin, opos = 0, error = 0;
	gchar from[5], work[50];
	gchar *eoltypes[] = {"\r\n", "\r", "\n", };
	gboolean quoting = FALSE;

	fflush(fp);

	memset(from, 0, sizeof(from));

	for(x=0; x<size; x++)
	{
		if((lpos > 70) &&
		   prefs.UseQuotedPrintable)
		{
			fprintf(fp, "=%s", eoltypes[prefs.EOLType]);
			lpos = 0;
		}

		switch(text[x])
		{
			case '\r':
			case '\n':
				fprintf(fp, eoltypes[prefs.EOLType]);
				lpos = 0;
				opos = 0;
				memset(from, 0, sizeof(from));
				quoting = FALSE;
				break;
			case 'f':
			case 'F':
				if(!strncasecmp(text + x, "From ", 5) &&
				   !lpos)
				{
					if(prefs.UseQuotedPrintable)
					{
						fprintf(fp, "%crom=20", text[x]);
						lpos++;
					}
					else
					{
						fprintf(fp, ">%crom ", text[x]);
						opos++;
					}
					lpos += 6;
					opos += 5;
					x += 5;
				}
				else
				{
					fputc(text[x], fp);
					lpos++;
					opos++;
				}
				break;
			case ' ':
				if((text[MIN(x,size-2) + 1] == '\r'))
				{
					if(prefs.UseQuotedPrintable)
					{
						fprintf(fp, "=20");
						lpos += 3;
						opos++;
						break;
					}
				}
				else
				{
					if((opos >= prefs.MaxLineLength) &&
					   !quoting)
					{
						lpos = 0;
						opos = 0;
						fprintf(fp, eoltypes[prefs.EOLType]);
						quoting = FALSE;
						break;
					}
				}
			default:
				if((text[x] == '>') &&
				   (lpos < 5))
					quoting = TRUE;
				memset(from, 0, sizeof(from));
				if(prefs.UseQuotedPrintable)
				{
					if(text[x] > 126)
					{
						fprintf(fp, "=%02.2X", text[x]);
						lpos += 3;
						opos++;
						break;
					}
					if(text[x] == '=')
					{
						fprintf(fp, "=3D");
						lpos += 3;
						opos++;
						break;
					}
				}

				fputc(text[x], fp);
				lpos++;
				opos++;
				break;
		}
	}

	return TRUE;
}
