/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: msg_common.c
 *         Common message functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "eucalyptus.h"

#if 0
void MailMsgReply(struct MailMsg *msg, BYTE type, BYTE replyflags)
{
     BusyPointer(msg->Window, TRUE);
     if(!CreateMsgReply(msg, type, replyflags, NULL))
     {
          if(gprefs->CloseMsgOnReply)
               msg->Flags |= MMF_CLOSEWINDOW;
          if(oprefs->IconifySuccessive)
               msg->Flags |= MMF_ICONIFY;
     }
     BusyPointer(msg->Window, FALSE);
}

BOOL MailMsgDelete(struct MailMsg *msg)
{
     BOOL next = FALSE;

     if(msg->Index.Flags & MIEF_DELETED)
     {
          msg->Index.Flags &= ~MIEF_DELETED;
     }
     else
     {
          msg->Index.Flags |= MIEF_DELETED;
          msg->Flags |= MMF_CLOSEWINDOW;
          next = TRUE;
          SetFolderIndexValid(msg->Folder, FALSE);
     }
     msg->Folder->Data.Flags |= FFF_INDEXMODIFIED;
     UpdateMsgStatus(msg);
     if(UpdateMsgInfo(msg, TRUE))
          if(msg->Folder->Window)
               RefreshGList(msg->Folder->ListBrowser, msg->Folder->Window, NULL, 1);

     if(next)
          MailMsgNext(msg, msg->Folder->ReadingDirection);

     euc.Flags |= EGF_UPDATEFOLDERS;
     Signal(euc.ThisProcess, SIG_UPDATE);

     return(TRUE);
}

BOOL MailMsgCopy(struct MailMsg *msg, struct Folder *dfolder, BOOL moveme)
{
     BOOL okay = TRUE;

     BusyPointer(msg->Window, TRUE);
     SetFolderStatus(msg->Folder, LocaleString(moveme ? ETXT_FOLDERSTATUS_MOVINGMSGS : ETXT_FOLDERSTATUS_COPYINGMSGS));
     if(!CopyMsg(msg->Index.Filename, dfolder, NULL, 0))
     {
          if(moveme)
          {
               msg->Flags |= MMF_REMOVE;
               msg->Folder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_UPDATEMAIN);
               SetFolderIndexValid(msg->Folder, FALSE);
          }

          dfolder->Data.Flags |= (FFF_INDEXMODIFIED | FFF_NEWMSGS | FFF_UPDATEMAIN);
          SetFolderIndexValid(dfolder, FALSE);
          if(dfolder->Process)
               Signal(dfolder->Process, SIG_UPDATE);

          euc.Flags |= EGF_UPDATEFOLDERS;
          Signal(euc.ThisProcess, SIG_UPDATE);
     }
     else
          okay = FALSE;

     SetFolderStatus(msg->Folder, NULL);
     BusyPointer(msg->Window, FALSE);

     return(okay);
}

BOOL MailMsgNext(struct MailMsg *msg, UBYTE dir)
{
     struct MailMsg *nmsg;
     LONG error = 0;

     if(nmsg = GetNextMailMsg(msg, dir))
     {
          BusyPointer(msg->Window, TRUE);
          if(error = OpenMsgInWindow(msg, nmsg))
          {
               DisplayError(LocaleString(ETXT_UNABLETOOPENWINDOW), error);
          }
          else
          {
               LBEditNode(msg->Folder->ListBrowser, msg->Folder->Window, NULL, msg,
                         LBNA_Selected,      FALSE,
                         TAG_END);
               LBEditNode(msg->Folder->ListBrowser, msg->Folder->Window, NULL, nmsg,
                         LBNA_Selected,      TRUE,
                         TAG_END);
          }
          BusyPointer(msg->Window, FALSE);
     }
     else
          return(FALSE);

     return(TRUE);
}

BOOL MailContentsSave(struct MailMsg *msg, BOOL print, struct PrintOptions *opts)
{
     struct MsgPart *mp;
     Object *getfile = NULL;
     STRPTR text = NULL;
     BOOL okay = TRUE;

     if(mp = (struct MsgPart *)msg->Parts.lh_Head)
     {
          if(print)
          {
               if(opts)
                    text = opts->Device;
               else
                    text = "PRT:";
          }
          else
          {
               if(getfile = NewObject(GETFILE_GetClass(), NULL,
                         GETFILE_TitleText,       LocaleString(ETXT_TITLE_SELECTTEXTFILE),
                         GETFILE_Drawer,          oprefs->DefaultSavePath,
                         GETFILE_DoSaveMode,      TRUE,
                         TAG_END))
               {
                    if(CA_RequestFile(getfile, msg->Window))
                    {
                         GetAttr(GETFILE_Drawer, getfile, (ULONG *)&text);
                         strncpy(msg->TempDir, text, sizeof(msg->TempDir) - 1);

                         GetAttr(GETFILE_FullFile, getfile, (ULONG *)&text);
                    }
               }
          }

          if(text)
          {
               LONG flags = 0;

               if(print && opts)
               {
                    switch(opts->HeadersType)
                    {
                         case POHT_ALL:
                              flags |= SMPF_SAVEHEADERS;
                         case POHT_BRIEF:
                              flags |= SMPF_BRIEFHEADERS;
                              break;
                    }
               }

               if(SaveMsgPart(text, msg, mp, flags))
               {
                    IsenRequest(euc.Screen,
                              ISENREQUEST_Text,        LocaleString(print ? ETXT_UNABLETOPRINTMSGTEXT : ETXT_UNABLETOSAVEMSGTEXT),
                              TAG_END);
               }
               else
                    okay = FALSE;
          }
          else
               okay = FALSE;
     }

     if(getfile)
          DisposeObject(getfile);

     return(okay);
}

LONG MailAttachSave(struct MailMsg *msg, struct MsgPart *mp, BOOL print, struct PrintOptions *opts)
{
     Object *getfile = NULL;
     LONG error = 0;
     STRPTR text = NULL;

     if(!msg ||
        !mp)
          return(err_UNKNOWNMSGPART);

     if(print)
     {
          if(opts)
               text = opts->Device;
          else
               text = "PRT:";
     }
     else
     {
          if(getfile = NewObject(GETFILE_GetClass(), NULL,
                         GETFILE_TitleText,       LocaleString(ETXT_TITLE_SELECTATTACHMENT),
                         GETFILE_Drawer,          strlen(msg->TempDir) ? msg->TempDir : oprefs->DefaultSavePath,
                         GETFILE_File,            mp->Name,
                         GETFILE_DoSaveMode,      TRUE,
                         TAG_END))
          {
               getit:
               if(CA_RequestFile(getfile, msg->Window))
               {
                    GetAttr(GETFILE_Drawer, getfile, (ULONG *)&text);
                    strncpy(msg->TempDir, text, sizeof(msg->TempDir) - 1);

                    GetAttr(GETFILE_FullFile, getfile, (ULONG *)&text);
               }
          }
     }

     if(text)
     {
          LONG flags = 0;
          BPTR fp;

          if(!print)
          {
               if(fp = Open(text, MODE_OLDFILE))
               {    // file exists
                    Close(fp);

                    switch(IsenRequest(euc.Screen,
                              ISENREQUEST_Text,        LocaleString(ETXT_ATTACHMENTEXISTS),
                              ISENREQUEST_Button,      LocaleString(ETXT_REQ_OVER_NEWNAME_SKIP_ABORT),
                              TAG_END))
                    {
                         case 1:   // overwrite
                              break;
                         case 2:   // new name
                              goto getit;
                         case 3:   // skip
                              goto disposeit;
                         default:
                              error = err_CANCELLED;
                              goto disposeit;
                    }
               }
          }
          else
          {
               if(print && opts)
               {
                    switch(opts->HeadersType)
                    {
                         case POHT_ALL:
                              flags |= SMPF_SAVEHEADERS;
                         case POHT_BRIEF:
                              flags |= SMPF_BRIEFHEADERS;
                              break;
                    }
               }
          }

          if(SaveMsgPart(text, msg, mp, flags))
          {
               IsenRequest(euc.Screen,
                         ISENREQUEST_Text,        LocaleString(print ? ETXT_UNABLETOPRINTMSGATTACHMENT : ETXT_UNABLETOSAVEMSGATTACHMENT),
                         TAG_END);
               error = err_UNABLETOSAVEFILE;
          }
     }
     else
          error = err_UNKNOWNMSGPART;

     disposeit:
     if(getfile)
          DisposeObject(getfile);

     return(error);
}

BOOL MailRawSave(struct MailMsg *msg, BOOL print, struct PrintOptions *opts)
{
     Object *getfile = NULL;
     UBYTE *buf;
     BPTR fp, newfp;
     BOOL okay = TRUE;

     if(buf = (UBYTE *)AllocVec(WORKLEN, MEMF_CLEAR))
     {
          STRPTR text = NULL;

          if(print)
          {
               if(opts)
                    text = opts->Device;
               else
                    text = "PRT:";
          }
          else
          {
               if(getfile = NewObject(GETFILE_GetClass(), NULL,
                         GETFILE_TitleText,       LocaleString(ETXT_TITLE_SELECTTEXTFILE),
                         GETFILE_Drawer,          oprefs->DefaultSavePath,
                         GETFILE_DoSaveMode,      TRUE,
                         TAG_END))
               {
                    if(CA_RequestFile(getfile, msg->Window))
                    {

                         GetAttr(GETFILE_FullFile, getfile, (ULONG *)&text);
                    }
               }
          }

          if(text)
          {
               if(fp = Open(msg->Index.Filename, MODE_OLDFILE))
               {
                    if(newfp = Open(text, MODE_NEWFILE))
                    {
                         while(FGets(fp, buf, WORKLEN - 1))
                              FPuts(newfp, buf);

                         Close(newfp);
                    }
                    else
                         okay = FALSE;

                    Close(fp);
               }
               else
                    okay = FALSE;
          }

          FreeVec(buf);
     }
     else
          okay = FALSE;

     if(getfile)
          DisposeObject(getfile);

     return(okay);
}

BOOL MailMsgPrint(struct MailMsg *msg, struct PrintOptions *opts)
{
     BOOL okay = FALSE;

     if(!msg ||
        !opts)
          return(FALSE);

     if(opts->Raw)
     {
          okay = MailRawSave(msg, TRUE, opts);

          return(okay);
     }

     if(opts->Text)
     {
          okay = MailContentsSave(msg, TRUE, opts);
     }

     if(opts->Attachments)
     {
          struct MsgPart *mp;
          struct Node *node;

          for(node = opts->WhichAtts.lh_Head;
              node->ln_Succ;
              node = node->ln_Succ)
          {
               LONG val;

               GetListBrowserNodeAttrs(node,
                         LBNA_UserData,      &val,
                         TAG_END);
               if(mp = (struct MsgPart *)GetListNum(&msg->Parts, val))
               {
                    if(MailAttachSave(msg, mp, TRUE, opts))
                    {
                         okay = FALSE;
                         break;
                    }
               }
          }
     }

     return(okay);
}

void MailMsgBounce(struct MailMsg *msg, UBYTE *addr)
{
     struct Folder *dfolder;

     if(dfolder = GetSpecialFolder(FTF_OUTGOING))
     {
          UBYTE buf[51];

          if(!CopyMsg(msg->Index.Filename, dfolder, buf, sizeof(buf)))
          {
               BPTR fp, dir;

               if(dir = CurrentDir(dfolder->DirLock))
               {
                    buf[0] = '_';
                    buf[1] = '_';
                    buf[2] = 'a';

                    if(fp = Open(buf, MODE_NEWFILE))
                    {
                         FPrintf(fp, "To: <%s>\n", addr);

                         Close(fp);
                    }
                    else
                    {
                         buf[0] = 'm';
                         buf[1] = 's';
                         buf[2] = 'g';

                         DeleteFile(buf);

                         IsenRequest(euc.Screen,
                                   ISENREQUEST_Text,        LocaleString(ETXT_UNABLETOBOUNCEMSG),
                                   TAG_END);
                    }

                    CurrentDir(dir);
               }
          }
     }
}

#ifdef EUCALYPTUS_PRO
void MailMsgExport(struct MailMsg *msg, struct AsyncFile *out_fp)
{
     struct AsyncFile *fp;
     UBYTE buf[1025];
     BOOL firstline = TRUE;

     if(!msg ||
        !out_fp)
          return;

     if(fp = OpenAsync(msg->Index.Filename, MODE_READ, ASYNCIO_BUF, SysBase, DOSBase))
     {
          while(FGetsAsync(fp, buf, sizeof(buf) - 1))
          {
               if(firstline)
               {
                    if(strnicmp(buf, "From ", 5))
                         WriteLineAsync(out_fp, "From eucalyptus@isengard-dev.com\r\n");

                    firstline = FALSE;
               }

               WriteLineAsync(out_fp, buf);
          }

          CloseAsync(fp);
     }
}
#endif

#endif