/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: msg_reply.c
 *         Support functions for message replies
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "eucalyptus.h"

gboolean mailmsg_create_reply(MailMsg *msg, gchar type, gchar replyflags, Composition *comp)
{
     gchar text[1024];
     gboolean leavecomp = FALSE, success = TRUE;

     if(!comp)
          comp = compose_new();
     else
          leavecomp = TRUE;

     if(comp)
     {
          if(!(replyflags & MRTT_NOTEXT))
          {
			gchar *fname = NULL, *dir = NULL;
               FILE *fp_in, *fp_out;

			fname = path_build(dir = folder_get_full_path(msg->Folder), msg->Index.Filename, NULL);
			g_free(dir);
               if(fp_in = fopen(fname, "r"))
               {
               	euc_temp_filename("/tmp", comp->t.Filename, sizeof(comp->t.Filename));

                    if(fp_out = fopen(comp->t.Filename, "w"))
                    {
                         struct List *list = NULL;
                         HdrNode *hnode = NULL, *content = NULL, *cte = NULL;
                         gchar *htext;

                         /* write out any user defined header */
                         switch(type)
                         {
                              case MRT_FORWARD:
                                   htext = prefs.ForwardHeader;
                                   break;
                              default:
                                   htext = prefs.ReplyHeader;
                                   break;
                         }

                         if(htext &&
                            *htext)
                         {
                              glong x = 0;

                              while(htext[x])
                              {
                                   gchar weekdays[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat",};
                                   gchar months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",};

                                   switch(htext[x])
                                   {
                                        case '\\':
                                             switch(htext[++x])
                                             {
                                                  case 'n':
                                                       fprintf(fp_out, "\r\n");
                                                       break;
                                                  case 't':
                                                       fprintf(fp_out, "\t");
                                                       break;
                                                  default:
                                                       fputc(htext[x], fp_out);
                                                       break;
                                             }
                                             break;
                                        case '%':
                                             switch(htext[++x])
                                             {
                                             	time_t t;
                                             	struct tm *tm;
                                                  gchar buf[256], work[101];

                                                  case '%': /* percent sign */
                                                       fprintf(fp_out, "%");
                                                       break;
                                                  case 'f': /* from */
                                                       /* parse_address(msg->Index.From, tempstr, sizeof(tempstr), NULL, NULL, 0); */
                                                       /* TranslateHeaderEncoding(tempstr); */
                                                       fprintf(fp_out, msg->Index.From /* tempstr */);
                                                       break;
                                                  case 'n': /* name of person to whom you're replying */
                                                       parse_address(msg->Index.From, buf, sizeof(buf), NULL, 0, FALSE);
                                                       fprintf(fp_out, buf);
                                                       break;
                                                  case 'a': /* address of person to whom you're replying */
                                                       parse_address(msg->Index.From, NULL, 0, buf, sizeof(buf), FALSE);
                                                       fprintf(fp_out, buf);
                                                       break;
                                                  case 'N': /* you */
                                                       fprintf(fp_out, uprefs.RealName);
                                                       break;
                                                  case 'A': /* your address */
                                                       fprintf(fp_out, uprefs.Address);
                                                       break;
                                                  case 'O': /* your organization */
                                                       fprintf(fp_out, uprefs.Organization);
                                                       break;
                                                  case 'd': /* date */
											t = time(NULL);
											tm = localtime(&t);

                                                       sprintf(work, "%s, %d %s %d",
                                                                      weekdays[tm->tm_wday],
                                                                      tm->tm_mday, months[tm->tm_mon - 1], tm->tm_year);
                                                       fprintf(fp_out, work);

                                                       break;
                                                  case 's': /* subject */
                                                       fprintf(fp_out, msg->Index.Subject);
                                                       break;
                                             }
                                             break;
                                        default:
                                             fputc(htext[x], fp_out);
                                             break;
                                   }

                                   x++;
                              }

                              fprintf(fp_out, "\r\n\r\n");
                         }

                         switch(type)
                         {
                              case MRT_FORWARD:
                                   fprintf(fp_out, "*** BEGIN FORWARDED MESSAGE ***\n");
                                   break;
                         }

                         if(list = mailmsg_get_header(GMHT_FILE, fp_in, (replyflags & MRTT_ALLHEADERS) ? fp_out : NULL))
                         {
                              glong content_type = CONTENTTYPE_TEXT, encoding = 0;
                              gchar *boundary = NULL;

                              if(replyflags & MRTT_SOMEHEADERS)
                              {
                                   if(hnode = header_find_tag("From", list, 0))
                                   {
                                        fprintf(fp_out, "From: %s\n",
										hnode->Data);
                                   }
                                   if(hnode = header_find_tag("To", list, 0))
                                   {
                                        fprintf(fp_out, "To: %s\n",
										hnode->Data);
                                   }
                                   if(hnode = header_find_tag("Date", list, 0))
                                   {
                                        fprintf(fp_out, "Date: %s\n",
										hnode->Data);
                                   }
                                   if(hnode = header_find_tag("Subject", list, 0))
                                   {
                                        fprintf(fp_out, "Subject: %s\n",
										hnode->Data);
                                   }
                              }

                              /* extract from address */
                              if(type != MRT_FORWARD)
                              {
                                   HdrNode *fnode, *rnode;
                                   gchar fname[128], rname[128];

                                   fnode = header_find_tag("From", list, 0);
                                   if(!(rnode = hnode = header_find_tag("Reply-To", list, 0)))
                                        hnode = fnode;
                                   else if(fnode && rnode)
                                   {
                                        parse_address(fnode->Data, NULL, 0, fname, sizeof(fname), 0);
                                        parse_address(rnode->Data, NULL, 0, rname, sizeof(rname), 0);

                                        if(!strcasecmp(fname, rname))
                                        {
                                        	glong which;
                                        	gchar *query;
                                        	const gchar *which_buttons[] = { "From:", "Reply-To:", NULL, };

                                             query = g_strdup_printf( 
                                             		_("The From and Reply-To addresses appear to be the same.\n"
                                             		  "Please choose one to use:\n"
                                             		  "%s\n%s"), 
                                             		fnode->Data, rnode->Data);
                                             which = euc_request(msg->Window,
                                             		_("Eucalyptus query"),
                                             		query,
                                             		which_buttons, 1,
                                             		0);
                                             g_free(query);
                                             
                                             switch(which)
                                             {
                                                  case 0:   /* from */
                                                       hnode = fnode;
                                                       break;
                                                  default:  /* replyto */
                                                       hnode = rnode;
                                                       break;
                                             }
                                        }
                                   }

                                   if(hnode)
                                   {
                                        compose_address_add(comp, hnode->Data, ACAFT_PARSE, CAT_TO);
                                   }

                                   if(replyflags & MRTT_ALL)
                                   {
                                        struct List *alist;
                                        GList *node;

                                        if(hnode = header_find_tag("To", list, 0))
                                        {
                                             if(alist = smtp_parse_addressees(hnode->Data, PA_FULL))
                                             {
                                                  for(node = g_list_first(alist->List);
                                                      node;
                                                      node = g_list_next(node))
                                                  {
                                                       compose_address_add(comp, (gchar *)node->data, ACAFT_PARSE, CAT_TO);
                                                  }
                                             }
                                        }

                                        if(hnode = header_find_tag("Cc", list, 0))
                                        {
                                             if(alist = smtp_parse_addressees(hnode->Data, PA_FULL))
                                             {
                                                  for(node = g_list_first(alist->List);
                                                      node;
                                                      node = g_list_next(node))
                                                  {
                                                       compose_address_add(comp, (gchar *)node->data, ACAFT_PARSE, CAT_TO);
                                                  }
                                             }
                                        }
                                   }
                              }

                              /* extract msg reference and other semi-pertinent info */
                              if(hnode = header_find_tag("Message-ID", list, 0))
                              {
                                   strncpy(comp->t.Reference, hnode->Data, sizeof(comp->t.Reference) - 1);
                              }

                              if(content = header_find_tag("Content-Type", list, 0))
                              {
                                   if(-1 == (content_type = content_type_verify(hnode->Data)))
                                        content_type = CONTENTTYPE_APPLICATION;
                              }
                              else
                                   content_type = CONTENTTYPE_TEXT;
                              if(cte = header_find_tag("Content-Transfer-Encoding", list, CHTF_CASECONV))
                                   encoding = content_transfer_encoding_verify(cte->Data);

                              switch(content_type & CONTENTTYPE_MASK)
                              {
                                   struct List *part_hdr;
                                   HdrNode *p_cte /* , *p_cnt */ ;
                                   gchar *str;

                                   case CONTENTTYPE_MULTIPART:
                                        if(!(str = content_type_find_param(content->Data, "boundary=", TRUE)))
                                             break;

                                        boundary = str + 9;
                                        header_remove_whitespace(boundary, 0);
                                        header_remove_quotes(boundary);

                                        /* get opening boundary string */
                                        while(fgets(text, sizeof(text) - 1, fp_in))
                                        {
                                             if(!strncmp(text, "--", 2) &&
                                                !strncmp(text + 2, boundary, strlen(boundary)))
                                                  break;
                                        }

                                        /* get header out of the way */
                                        if(part_hdr = mailmsg_get_header(GMHT_FILE, fp_in, NULL))
                                        {
/*
                                             if(p_cnt = header_find_tag("Content-Type", part_hdr, 0))
                                             {
                                                  struct List *ct;

                                                  ct = GetContentTypeParms(content->Data);
                                                  if(-1 == (content_type = VerifyContentType(ct)))
                                                       content_type = CONTENTTYPE_APPLICATION;

                                                  if(ct)
                                                       FreeNodeNamesList(ct, TRUE);
                                             }
                                             else
                                                  content_type = CONTENTTYPE_TEXT;
*/

                                             if(p_cte = header_find_tag("Content-Transfer-Encoding", part_hdr, CHTF_CASECONV))
                                                  encoding = content_transfer_encoding_verify(p_cte->Data);

                                             mailmsg_free_header(part_hdr);
                                        }
								/* drop through */

                                   default:
                                        /* quote text */
                                        while(fgets(text, sizeof(text) - 1, fp_in))
                                        {
                                             glong x;
                                             gboolean lastequals;

                                             if(boundary)
                                                  if(!strncmp(text, "--", 2) &&
                                                     !strncmp(text + 2, boundary, strlen(boundary)))
                                                       break;
                                             if(type != MRT_FORWARD)
                                             {
                                                  fprintf(fp_out, prefs.QuotePrefix);
                                                  fprintf(fp_out, " ");
                                             }

                                             if(encoding == ENCODING_QUOTEDPRINTABLE)
                                             {
                                                  x = 0;
                                                  lastequals = FALSE;
                                                  while(text[x])
                                                  {
                                                       switch(text[x])
                                                       {
                                                            case '=':
                                                                 lastequals = TRUE;
                                                                 x++;
                                                                 if((text[x] == '\r') ||
                                                                    (text[x] == '\n'))
                                                                 {
                                                                      if(text[x + 1] == '\n')
                                                                      {
                                                                           x++;
                                                                      }
                                                                      lastequals = FALSE;
                                                                      break;
                                                                 }
                                                            default:
                                                                 if(!text[x])
                                                                      break;
                                                                 if(lastequals)
                                                                 {
                                                                      glong val;
                                                                      gchar hex[3] = {0,};

                                                                      hex[0] = text[x++];
                                                                      if(text[x])
                                                                           hex[1] = text[x];
                                                                      hex[2] = 0;
                                                                      if(isxdigit(hex[0]) &&
                                                                         isxdigit(hex[1]))
                                                                      {
                                                                      	val = strtol(hex, NULL, 16);
                                                                           fputc(val, fp_out);
                                                                      }
                                                                 }
                                                                 else
                                                                      fputc(text[x], fp_out);
                                                                 lastequals = FALSE;
                                                                 break;
                                                       }

                                                       x++;
                                                  }
                                             }
                                             else
                                             {
                                                  fprintf(fp_out, text);
                                             }
                                        }
                                        break;
                              }

                              mailmsg_free_header(list);
                         }

                         fclose(fp_out);
                    }
               }

               fclose(fp_in);
          }

          switch(type)
          {
               gchar *c;

               case MRT_FORWARD:
               	g_snprintf(comp->t.Subject, sizeof(comp->t.Subject), 
						"Fwd: %s", msg->Index.Subject);
                    comp->Flags |= CF_FORWARDEDMSG;
                    msg->Index.Flags |= MIEF_FORWARDED;
                    msg->Folder->Data.Flags |= (FFF_INDEXMODIFIED);
                    break;
               default:
                    c = NULL;
                    comp->Flags |= CF_REPLIEDMSG;
                    msg->Index.Flags |= MIEF_REPLIED;
                    msg->Folder->Data.Flags |= FFF_INDEXMODIFIED;
                    if(strncasecmp(msg->Index.Subject, "Re:", 3))
                    {    /* no previous "Re:" */
                    	g_snprintf(comp->t.Subject, sizeof(comp->t.Subject), 
							"Re: %s", msg->Index.Subject);
                    }
                    else
                    {    
                         strncpy(comp->t.Subject, msg->Index.Subject, sizeof(comp->t.Subject) - 1);
                    }
                    break;
          }

          if(msg->Index.Flags & MIEF_NOTIFY)
          {
               glong answer = 1;
               gchar work[201];

               if(!leavecomp)
               {
                    g_snprintf(work, sizeof(work), 
                    		_("The notify flag is set on this message:\n"
                    		  "%s\n"
                    		  "Would you like to clear it?"),
                    		  msg->Index.Subject);
                    answer = euc_request(msg->Window,
                    		_("Eucalyptus query"),
                    		work,
                    		euc_yes_no_buttons, 1,
                    		0);
               }

               if(!answer)
               {
                    msg->Index.Flags &= ~MIEF_NOTIFY;

				folder_update_index(msg->Folder);
				folder_update_visual(msg->Folder);

				euc_update_listview();
               }
          }

          mailmsg_update_info(msg, FALSE);
          mailmsg_update_status(msg);

          if(!leavecomp)
          {
			list_append(&euc.Composes, comp);

			if(!compose_open_window(comp))
				success = FALSE;
		}
     }
	else
		success = FALSE;

     return success;
}

