/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: msgs.c
 *         Message handling functions
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <sys/socket.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

#include "eucalyptus.h"

MailMsg *mailmsg_new(Folder *folder)
{
     MailMsg *msg = NULL;

     if(msg = (MailMsg *)g_malloc0(sizeof(MailMsg)))
     {
          msg->Folder = folder;
          
          list_init(&msg->Parts);
     }

     return msg;
}

void mailmsg_free(MailMsg *msg)
{
     if(!msg)
          return;

     mailmsg_close_window(msg);

     mailmsg_free_header(msg->Headers);
     msg->Headers = NULL;

     g_free(msg);
}

struct List *mailmsg_get_header(glong type, gpointer fp, FILE *fp_out)
{
     struct List *list = NULL;


     switch(type)
     {
          case GMHT_SOCKET:
               if((int)fp < 0)
                    return(NULL);

               break;
          case GMHT_FILE:
               if(!fp)
                    return(NULL);
               break;
     }

     if(list = g_malloc0(sizeof(struct List)))
     {
          gchar text[WORKLEN];
          gboolean enough = FALSE, nomore = FALSE, abort = FALSE;

          list_init(list);

          for(;;)
          {
               GList *node;
               HdrNode *hdr;

               switch(type)
               {
                    case GMHT_FILE:
                         if(!fgets(text, sizeof(text), (FILE *)fp))
                              enough = TRUE;
                         if((*text == '\n') ||
                            (*text == '\r') ||
                            !*text)
                              enough = TRUE;
                         break;

                    case GMHT_SOCKET:
                         switch(pop_read_line((int)fp, text, sizeof(text) - 1))
                         {
                              case -1:  /* error, abort */
                                   abort = TRUE;
                              case 0:
                                   enough = TRUE;
                                   break;
                              default:
                                   if(!strncasecmp(text, "-ERR", 4))
                                   {
                                        abort = TRUE;
                                        enough = TRUE;
                                   }
                                   break;
                         }

                         if(abort)
                         {
                              mailmsg_free_header(list);
                              goto thatsit;
                         }

                         if((*text == '\n') ||
                            (*text == '\r') ||
                            !*text)
                              nomore = TRUE;
                         if(*text == '.')
                         {
                              if((text[1] == '\r') &&
                                 (text[2] == '\n'))
                                   enough = TRUE;
                              else
                                   strcpy(text, text + 1);
                         }
                         break;
               }

               if(fp_out)
                    fputs(text, fp_out);

               if(enough)
                    break;
               if(nomore)
                    continue;
               if(isspace(*text))
               {    /* attach to last node */
                    if(g_list_length(list->List))
                    {
                         gchar *t;

                         node = g_list_last(list->List);
                         hdr = (HdrNode *)node->data;
                         t = g_strdup_printf("%s %s", hdr->Data, strip_leading_blanks(text));
                         g_free(hdr->Data);
                         hdr->Data = t;
                         header_remove_whitespace(hdr->Data, RWSF_ONLYCRLF);
                    }

                    continue;
               }
               
               if(!strncasecmp(text, "From ", 5))
               	continue;

               if(hdr = g_malloc0(sizeof(HdrNode)))
               {
                    gchar *c;

                    if(c = strchr(text, ':'))
                    {
                         *c = '\0';

                    	hdr->Tag = g_strdup(strip_whitespace(text));
                    	hdr->Data = g_strdup(strip_leading_blanks(c + 1));

                         header_remove_whitespace(hdr->Data, RWSF_ONLYCRLF);

                         list_append(list, hdr);
                         continue;
                    }

                    g_free(hdr);
               }
          }
          
          thatsit:
     }

     if(!g_list_length(list->List))
     {
          g_free(list);
          list = NULL;
     }

     return list;
}

void mailmsg_free_header(struct List *list)
{
	GList *node;
     HdrNode *hdr;

	if(!list)
		return;

	while(node = g_list_first(list->List))
	{
		hdr = (HdrNode *)node->data;

		list->List = list_remove(list, hdr);

		g_free(hdr->Tag);
		g_free(hdr->Data);

		g_free(hdr);
     }

     g_free(list);
}

#if 0
LONG mailmsg_update_status(MailMsg *msg)
{
     LONG error = 0;
     STRPTR comment;

     if(!msg ||
        !msg->Folder)
          return(0);

     if(comment = (STRPTR)AllocVec(WORKLEN, MEMF_CLEAR))
     {
          if(!(msg->Index.Flags & (MIEF_NEW | MIEF_UNREAD)))
               strncpy(comment, "Opn;", WORKLEN - 1);
          if(msg->Index.Flags & MIEF_UNREAD)
               strncpy(comment, "Unr;", WORKLEN - 1);

          if(msg->Index.Flags & MIEF_REPLIED)
               strncat(comment, "Rep;", WORKLEN - 1);
          if(msg->Index.Flags & MIEF_FORWARDED)
               strncat(comment, "Fwd;", WORKLEN - 1);
          if(msg->Index.Flags & MIEF_BOUNCED)
               strncat(comment, "Bnc;", WORKLEN - 1);
          if(msg->Index.Flags & MIEF_DELETED)
               strncat(comment, "Del;", WORKLEN - 1);
          if(msg->Index.Flags & MIEF_DONTDELETE)
               strncat(comment, "NoD;", WORKLEN - 1);
          if(msg->Index.Flags & MIEF_NOTIFY)
               strncat(comment, "Not;", WORKLEN - 1);
          if(msg->Index.Flags & MIEF_PERSONAL)
               strncat(comment, "Per;", WORKLEN - 1);
          if(msg->Index.Flags & MIEF_PRIORITY)
               strncat(comment, "Pri;", WORKLEN - 1);
          if(msg->Index.Flags & MIEF_RECEIPTED)
               strncat(comment, "Rct;", WORKLEN - 1);
          if(msg->Index.Flags & MIEF_SUSPEND)
               strncat(comment, "Sus;", WORKLEN - 1);
          if(msg->Index.Flags & MIEF_EDITING)
               strncat(comment, "Edt;", WORKLEN - 1);

          SetComment(msg->Index.Filename, comment);

          FreeVec(comment);
     }

     UpdateIndexEntry(msg->Folder, &msg->Index);

     return(error);
}
#endif

gboolean mailmsg_update_info(MailMsg *msg, gboolean something)
{
#if 0
     UBYTE tempstr[21] = {0,}, *meas = NULL, oldflags[21];
     BOOL needsrefresh = FALSE;

     if(!msg)
          return(0);

     strncpy(oldflags, msg->FlagsText, sizeof(oldflags) - 1);

     if(msg->Index.Size > (1024 * 1024))
     {
          sprintf(tempstr, "%.1lf", msg->Index.Size / (1024.0 * 1024.0));
          meas = LocaleString(ETXT_MB);
     }
     else if(msg->Index.Size > 1024)
     {
          sprintf(tempstr, "%.1lf", msg->Index.Size / 1024.0);
          meas = LocaleString(ETXT_KB);
     }
     else
     {
          sprintf(tempstr, "%ld", msg->Index.Size);
          meas = NULL;
     }
     strncpy(msg->SizeText, tempstr, sizeof(msg->SizeText) - 1);
     if(meas)
          strncat(msg->SizeText, meas, sizeof(msg->SizeText) - 1);

     BuildDateString(msg);
     if(msg->Folder->Data.Type & FTF_POSTPONED)
          *msg->Index.DateStr = '\0';

     strcpy(msg->FlagsText, "");
     if(!(msg->Folder->Data.Type & (FTF_OUTGOING | FTF_SENT | FTF_POSTPONED)))
     {
          if(msg->Index.Flags & MIEF_NEW)
               strncat(msg->FlagsText, "N", sizeof(msg->FlagsText) - 1);
          if(msg->Index.Flags & MIEF_UNREAD)
               strncat(msg->FlagsText, "U", sizeof(msg->FlagsText) - 1);
          if(msg->Index.Flags & MIEF_REPLIED)
               strncat(msg->FlagsText, "R", sizeof(msg->FlagsText) - 1);
          if(msg->Index.Flags & MIEF_FORWARDED)
               strncat(msg->FlagsText, "F", sizeof(msg->FlagsText) - 1);
          if(msg->Index.Flags & MIEF_BOUNCED)
               strncat(msg->FlagsText, "B", sizeof(msg->FlagsText) - 1);
     }
     if(msg->Index.Flags & MIEF_DELETED)
          strncat(msg->FlagsText, "D", sizeof(msg->FlagsText) - 1);
     if(msg->Index.Flags & MIEF_DONTDELETE)
          strncat(msg->FlagsText, "P", sizeof(msg->FlagsText) - 1);
     if(msg->Index.Flags & MIEF_NOTIFY)
          strncat(msg->FlagsText, "I", sizeof(msg->FlagsText) - 1);
     if(msg->Index.Flags & MIEF_ATTACHMENTS)
          strncat(msg->FlagsText, "A", sizeof(msg->FlagsText) - 1);
     if(msg->Index.Flags & MIEF_SUSPEND)
     {
          if(msg->Folder->Data.Type & FTF_OUTGOING)
               strncat(msg->FlagsText, "S", sizeof(msg->FlagsText) - 1);
     }
     if(msg->Index.Flags & MIEF_PERSONAL)
          pen = prefs->PersonalPen;
     if(msg->Index.Flags & MIEF_PRIORITY)
          pen = prefs->PriorityPen;

     if(strcasecmp(oldflags, msg->FlagsText))
          needsrefresh = TRUE;

     if(attached)
     {
          LBEditNode(msg->Folder->ListBrowser, msg->Folder->Window, NULL, (struct Node *)msg,
                    LBNA_Flags,         LBFLG_CUSTOMPENS,
                    LBNA_Column,        0,
                         LBNCA_Text,         msg->Index.From,
                         LBNCA_FGPen,        pen,
                    LBNA_Column,        1,
                         LBNCA_Text,         msg->Index.DateStr,
                         LBNCA_FGPen,        pen,
                    LBNA_Column,        2,
                         LBNCA_Text,         msg->SizeText,
                         LBNCA_FGPen,        pen,
                    LBNA_Column,        3,
                         LBNCA_Text,         msg->FlagsText,
                         LBNCA_FGPen,        pen,
                    LBNA_Column,        4,
                         LBNCA_Text,         msg->Index.Subject,
                         LBNCA_FGPen,        pen,
                    TAG_END);
     }
     else
     {
          SetListBrowserNodeAttrs((struct Node *)msg,
                    LBNA_Flags,         LBFLG_CUSTOMPENS,
                    LBNA_Column,        0,
                         LBNCA_Text,         msg->Index.From,
                         LBNCA_FGPen,        pen,
                    LBNA_Column,        1,
                         LBNCA_Text,         msg->Index.DateStr,
                         LBNCA_FGPen,        pen,
                    LBNA_Column,        2,
                         LBNCA_Text,         msg->SizeText,
                         LBNCA_FGPen,        pen,
                    LBNA_Column,        3,
                         LBNCA_Text,         msg->FlagsText,
                         LBNCA_FGPen,        pen,
                    LBNA_Column,        4,
                         LBNCA_Text,         msg->Index.Subject,
                         LBNCA_FGPen,        pen,
                    TAG_END);
     }

     return(needsrefresh);
#endif
	return FALSE;
}

gboolean mailmsg_get_parts(MailMsg *msg)
{
     HdrNode *mver = NULL, *content = NULL, *cte = NULL, *cdisp = NULL;
     gboolean success = FALSE;
     gchar text[WORKLEN], *fname;
     FILE *fp;

     if(!msg ||
        !msg->Folder)
          return FALSE;

     msg->PartCount = 0;

	fname = mailmsg_get_full_path(msg);

     if(fp = fopen(fname, "r"))
     {
          if(!msg->Headers)
          {
               if(!(msg->Headers = mailmsg_get_header(GMHT_FILE, fp, NULL)))
               {
               	fclose(fp);
                    return FALSE;
               }
          }

          if(mver = header_find_tag("MIME-Version", msg->Headers, 0))
          {
               if(!strstr(mver->Data, "1.0"))
               {    /* MIME version we don't understand */
               	gchar *errstr;
               	
               	errstr = g_strdup_printf(_("This message is not the correct\n"
               						"MIME version.  Eucalyptus was expecting\n"
               						"version 1.0, and found verion %s.\n\n"
               						"Do you want to try to display it anyway?"),
               						mver->Data);
               	
               	if(euc_request(msg->Window,
               			_("Eucalyptus warning"),
               			_(errstr),
               			euc_ok_cancel_buttons, 1,
               			0))
               	{
               		g_free(errstr);
                         goto out;
                    }
                    
                    g_free(errstr);
               }
          }

          if(content = header_find_tag("Content-Type", msg->Headers, 0))
          {
               if(-1 == (msg->ContentTypeCode = content_type_verify(content->Data)))
                    msg->ContentTypeCode = CONTENTTYPE_APPLICATION;
          }
          else
               msg->ContentTypeCode = CONTENTTYPE_TEXT;
          if(cte = header_find_tag("Content-Transfer-Encoding", msg->Headers, CHTF_CASECONV))
               msg->ContentEncoding = content_transfer_encoding_verify(cte->Data);
          cdisp = header_find_tag("Content-Disposition", msg->Headers, 0);

          if(mailmsg_is_signed(msg))
               msg->ContentFlags |= MMCF_HASSIGNATURE;
          if(mailmsg_is_encrypted(msg))
               msg->ContentFlags |= MMCF_ISENCRYPTED;

		if(!g_list_length(msg->Parts.List))
		{
			list_init(&msg->Parts);

			switch(msg->ContentTypeCode & CONTENTTYPE_MASK)
			{
				MsgPart *mp;

				case CONTENTTYPE_MULTIPART:
					mailmsg_process_multipart(msg, content->Data, fp);
					break;
				case CONTENTTYPE_MESSAGE:
				default:  /* other types */
					if(mp = msgpart_new())
					{
						HdrNode *content;

						/* init some things */
						g_free(mp->Name);
						mp->Name = g_strdup(_("Text"));

						if(content = header_find_tag("Content-Type", msg->Headers, 0))
						{
							gchar *str;
							
							if(-1 == (mp->Type = content_type_verify(content->Data)))
							{
								mp->Type = CONTENTTYPE_APPLICATION;
							}
							if(str = content_type_find_param(content->Data, "name=", TRUE))
							{
								header_remove_quotes(str);
								g_free(mp->Name);
								mp->Name = g_strdup(str);
								g_free(str);
							}
							if(str = content_type_find_param(content->Data, "charset=", TRUE))
							{
								header_remove_quotes(str);
								header_remove_whitespace(str, 0);
								if(strcasecmp(str, "us-ascii") &&
								   strcasecmp(str, "iso-8859-1"))
								{    /* unknown character set
									/* FIXME */
								}

								g_free(str);
							}
						}
						else
							mp->Type = CONTENTTYPE_TEXT;

						if(content = header_find_tag("Content-Disposition", msg->Headers, 0))
						{
							gchar *c, *e;

							if(c = stristr(content->Data, "name="))
							{
								gboolean semi = FALSE;

								if(e = strchr(c, ';'))
								{
									*e = '\0';
									semi = TRUE;
								}

								g_free(mp->Name);
								mp->Name = g_strdup(c);
								header_remove_quotes(mp->Name);

								if(semi)
									*e = ';';
							}
						}

						list_append(&msg->Parts, mp);
						msg->PartCount = 1;
					}
					break;
			}
		}

		out:
          fclose(fp);
     }

	if(g_list_length(msg->Parts.List))
		success = TRUE;

     return success;
}

void mailmsg_free_contents(MailMsg *msg)
{
     MsgPart *mp;
     GList *node;

	if(!msg)
		return;

     /* free msg header info */
     mailmsg_free_header(msg->Headers);
     msg->Headers = NULL;

     /* delete temporary files */
     while(node = g_list_first(msg->Parts.List))
     {
     	mp = (MsgPart *)node->data;
     	msg->Parts.List = list_remove(&msg->Parts, mp);

          if(mp->Filename && *mp->Filename)
          {
			unlink(mp->Filename);
          }

          msgpart_free(mp);
     }
}

void mailmsg_free_parts(MailMsg *msg)
{
}

void mailmsg_sort_headers(MailMsg *msg)
{
     gboolean done = FALSE;
     gchar *b, *sc;

     sc = g_strdup(prefs.HeaderOrder);

     while(!done)
     {
          HdrNode *node;

          if(b = strrchr(sc, ';'))
          {
               *b = '\0';
               b++;
          }
          else
          {
               b = sc;
               done = TRUE;
          }

          if(node = header_find_tag(b, msg->Headers, 0))
          {
          	msg->Headers->List = list_remove(msg->Headers, node);
          	msg->Headers->List = g_list_prepend(msg->Headers->List, node);
          }
     }
     
     g_free(sc);

     sc = g_strdup(prefs.HideHeaders);

	done = FALSE;
     while(!done)
     {
          HdrNode *node;

          if(b = strrchr(sc, ';'))
          {
               *b = '\0';
               b++;
          }
          else
          {
               b = sc;
               done = TRUE;
          }

          while(node = header_find_tag(b, msg->Headers, 0))
          {
          	msg->Headers->List = list_remove(msg->Headers, node);
          	
          	g_free(node->Tag);
          	g_free(node->Data);
          	g_free(node);
          }
     }

     g_free(sc);
}

#if 0
void RemoveMsg(MailMsg *msg, BOOL silent)
{
     Folder *folder;

     if(!msg)
          return;

     if(!(folder = msg->Folder))
          return;

     if(folder->ListBrowser)
          SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
                    LISTBROWSER_Labels,      ~0,
                    TAG_END);
     DeleteFile(msg->Index.Filename);
     if(folder->Data.Type & FTF_OUTGOING)
     {
          UBYTE name[31];

          strncpy(name, msg->Index.Filename, sizeof(name) - 1);
          memset(name, '_', 3);
          DeleteFile(name);
          name[2] = 'a';
          DeleteFile(name);
          name[2] = 'i';
          DeleteFile(name);
     }
     Remove((struct Node *)msg);
     AdjustMsgCount(folder, 1, msg->Index.Size, msg->Index.Flags, -1);
     FreeMailMsg(msg);
     if(folder->ListBrowser)
          if(SetGadgetAttrs(folder->ListBrowser, folder->Window, NULL,
                    LISTBROWSER_Labels,      &folder->Msgs,
                    LISTBROWSER_AutoFit,     TRUE,
                    TAG_END))
               if(!silent && folder->Window)
                    RefreshGList(folder->ListBrowser, folder->Window, NULL, 1);
}
#endif

gboolean mailmsg_process_multipart(MailMsg *msg, const gchar *content_type, FILE *fp)
{
     MsgPart *mp;
     glong part_count = 0;
     gchar *boundary = "", text[WORKLEN];
     gboolean msgdone, success = TRUE;

	if(!msg ||
	   !content_type ||
	   !fp)
	   	return FALSE;

     if(!(boundary = content_type_find_param(content_type, "boundary=", TRUE)))
          return FALSE;

     header_remove_whitespace(boundary, 0);
     header_remove_quotes(boundary);

     /* get opening boundary string */
     while(fgets(text, sizeof(text) - 1, fp))
     {
          if(!strncmp(text, "--", 2) &&
             !strncmp(text + 2, boundary, strlen(boundary)))
               break;
     }

     msgdone = FALSE;
     while(!msgdone)
     {
          gchar *md;

          if(mp = msgpart_new())
          {
               /* init some things */
               g_free(mp->Name);
               mp->Name = g_strdup_printf(_("<Part %d>"), part_count + 1);
               g_free(mp->Boundary);
               mp->Boundary = g_strdup(boundary);

               mp->Number = part_count;

               list_append(&msg->Parts, mp);
               
               part_count++;
               msg->PartCount++;

               /* get msg part headers */
               if(mp->Headers = mailmsg_get_header(GMHT_FILE, fp, NULL))
               {
                    HdrNode *content;

                    if(content = header_find_tag("Content-Type", mp->Headers, 0))
                    {
                    	gchar *str;

                         if(-1 == (mp->Type = content_type_verify(content->Data)))
                              mp->Type = CONTENTTYPE_APPLICATION;
                              
                         if(str = content_type_find_param(content->Data, "name=", TRUE))
                         {
                         	g_free(mp->Name);
                              mp->Name = g_strdup(str);
						g_free(str);
                              header_remove_quotes(mp->Name);
                         }

                         if((mp->Type & CONTENTTYPE_MASK) == CONTENTTYPE_MULTIPART)
                         {
                              msg->Parts.List = list_remove(&msg->Parts, mp);
                              msg->PartCount--;

                              mailmsg_process_multipart(msg, content->Data, fp);

                              msgpart_free(mp);

						goto get_body;
                         }
                    }
                    else
                         mp->Type = CONTENTTYPE_TEXT;

                    if(content = header_find_tag("Content-Disposition", mp->Headers, 0))
                    {
                         gchar *c, *e;

                         if(!(c = stristr(content->Data, "name=")))
						c = stristr(content->Data, "filename=");

					if(c)
                         {
						gchar *eq;
                              gboolean rs = FALSE;

                              if(e = strchr(c, ';'))
                              {
                                   *e = '\0';
                                   rs = TRUE;
                              }

						eq = strchr(c, '=');

                              g_free(mp->Name);
                              mp->Name = g_strdup(eq + 1);
                              header_remove_quotes(mp->Name);

                              if(rs)
                                   *e = ';';
                         }
                    }
               }
               else
                    mp->Type = CONTENTTYPE_TEXT;
          }
          else
          {
          	success = FALSE;
               break;
          }

          /* get msg part body */
get_body:
          while(md = fgets(text, sizeof(text) - 1, fp))
          {
               /* check for boundary */
               if(!strncmp(text, "--", 2) &&
                  !strncmp(text + 2, boundary, strlen(boundary)))
               {
                    if(!strncmp(text + 2 + strlen(boundary), "--", 2))
                         msgdone = TRUE;
                    break;
               }
          }

          if(!md)   /* reached end of file */
               break;
     }

	return success;
}

#if 0
LONG UpdateIndexEntry(Folder *folder, MailIndexEntry *mie)
{
     LONG error = 0;

     if(!folder ||
        !mie)
          return(0);

     if(mie->Position > 0)
     {
          BPTR fp, dir;

          if(dir = CurrentDir(folder->DirLock))
          {
               if(fp = Open(".index", MODE_READWRITE))
               {
                    if(-1 != Seek(fp, mie->Position, OFFSET_BEGINNING))
                    {
                         Write(fp, mie, sizeof(MailIndexEntry));
                    }
                    else
                         error = err_POSITIONNOTKNOWN;

                    Close(fp);
               }
               else
                    error = err_UNABLETOOPENFILE;

               CurrentDir(dir);
          }
          else
               error = err_UNABLETOLOCKDIR;
     }
     else
          error = err_INVALIDPOSITION;

     return(error);
}

#endif
