/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: pop.c
 *   	 POP client function server
 *
 * Copyright  1997-2000 Isengard Developments
 * All rights reserved.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "eucalyptus.h"
#include "md5.h"

struct POPControl pop_control;

void *pop_thread(void *data)
{
	Server *server = (Server *)data;
	ProgressMsg *pmsg;
	
	/* Send a visual update.
	 */
	if(pmsg = g_malloc0(sizeof(ProgressMsg)))
	{
		pmsg->msg.type = MSG_TYPE_PROGRESS;
		pmsg->msg.subtype = MSG_SUBTYPE_PROGRESS_OPEN;
		pmsg->who = MSG_WHO_POP;
		pmsg->total = server->Size;
fprintf(stderr, "server->Size = %d\n", server->Size);
		pmsg->current = 0;
		pmsg->server = server;
		strncpy(pmsg->id, "", sizeof(pmsg->id) - 1);
		/* text */

		pthread_mutex_lock(&euc.q.Mutex);
		list_append(&euc.q.Msgs, pmsg);
		pthread_mutex_unlock(&euc.q.Mutex);
	}

	switch(server->Type)
	{
		case PST_INTERNAL:
			/* Connect.
			 */
			if(!pop_server_connect(server))
				break;
			
			/* Get headers, if requested.
			 */
			if(!pop_server_get_headers(server))
				break;
			 
			/* Wait for a go-ahead from the user, if requested,
			 * but only for so long, then disconnect.
			 */
			
			 
			/* Get messages.
			 */
			pop_server_get_messages(server);

			break;
			
		case PST_COMMAND:
			{
				glong x = 0;
				gchar command[2048];

				while(server->Command[x])
				{
					switch(server->Command[x])
					{
						gchar work[5];

						case '%':
							switch(server->Command[++x])
							{
								case '%':
									strncat(command, "%", sizeof(command) - 1);
									break;
								case 'h': // host
									break;
								case 'p': // port
									break;
							}
							break;
						default:
							sprintf(work, "%c", server->Command[x]);
							strncat(command, work, sizeof(command) - 1);
							break;
					}

					x++;
				}

				switch(x = gnome_execute_shell(NULL, command))
				{
					struct Folder *dfolder;

					case -1:
#if 0
						IsenRequest(euc.Screen,
								ISENREQUEST_Text,   	LocaleString(ETXT_UNABLETOEXECUTECMD),
								TAG_END);
#endif
						break;

					default:
						break;
				}
			}
			break;
	}
	
	/* Send a visual update.
	 */
	if(pmsg = g_malloc0(sizeof(ProgressMsg)))
	{
		pmsg->msg.type = MSG_TYPE_PROGRESS;
		pmsg->msg.subtype = MSG_SUBTYPE_PROGRESS_CLOSE;
		pmsg->who = MSG_WHO_POP;
		pmsg->server = server;

		pthread_mutex_lock(&euc.q.Mutex);
		list_append(&euc.q.Msgs, pmsg);
		pthread_mutex_unlock(&euc.q.Mutex);
	}

	/* Clean up.
	 */
	server->Thread = 0;
	pthread_attr_destroy(&server->ThreadAttr);
	server->Running = FALSE;

	return NULL;
}

gboolean pop_server_get_headers(Server *server)
{
	ServerEntry *entry;
	glong val, i;
	gboolean success = TRUE;

	/* Check if this node is connected.
	 */
	if(!server->Connected)
	{
		return FALSE;
	}

	/* Disconnect if there aren't any messages.
	 */
	if(server->Count == 0)
	{
		return FALSE;
	}

	/* Update the status.
	 */
	pop_server_status(server, _("Retrieving message headers..."), 0);

	/* Enable Stop button.
	 */
	gtk_widget_set_sensitive(server->w.Stop, TRUE);

	/* Iterate through the messages on the server.
	 */
	for(i=1; i<=server->Count; i++)
	{
		server->Current = i;
	
		if(entry = pop_server_entry_new())
		{
			entry->Server = server;
			entry->Number = server->Current;

			sprintf(server->out_buf, "UIDL %ld\r\n", server->Current);
			switch(val = pop_server_send(server, server->out_buf, server->in_buf, sizeof(server->in_buf)))
			{
				case 0:   // good
					strncpy(entry->UIDL, server->in_buf, sizeof(entry->UIDL) - 1);
					break;
				default:
					break;
			}

			sprintf(server->out_buf, "LIST %ld\r\n", server->Current);
			switch(val = pop_server_send(server, server->out_buf, server->in_buf, sizeof(server->in_buf)))
			{
				gchar *c;

				case 0:   // good
					if(c = strchr(server->in_buf, ' '))
					{
						entry->Size = atol(c + 1);
//						server->Size += entry->Size;
					}

					if(server->SizeLimit &&
					   (entry->Size > (server->SizeLimit * 1024)))
					{
						entry->LargeMsg = TRUE;
						entry->Selected = FALSE;
					}
					break;
				default:
					pop_server_status(server, _("Unable to get message information"), -1);
					server->State = PSS_STOPPING;
					break;
			}

			if(!server->DontRetrieveHeaders)
			{
				sprintf(server->out_buf, "TOP %ld 0\r\n", server->Current);
				switch(val = pop_server_send(server, server->out_buf, server->in_buf, sizeof(server->in_buf)))
				{
					struct List *hdrs;

					case 0:   // good
						if(hdrs = mailmsg_get_header(GMHT_SOCKET, (gpointer)server->Socket, NULL))
						{
							HdrNode *hnode;
							Filter *fnode;

							if(hnode = header_find_tag("From", hdrs, 0))
							{
								parse_address(hnode->Data, entry->From, sizeof(entry->From), NULL, 0, FALSE);
								if(!strlen(entry->From))
									parse_address(hnode->Data, NULL, 0, entry->From, sizeof(entry->From), FALSE);
							}

							if(hnode = header_find_tag("Date", hdrs, 0))
							{
								gchar *c;

								strncpy(entry->Date, hnode->Data, sizeof(entry->Date) - 1);
								if(c = strchr(entry->Date, ':'))
								{
									while(c != entry->Date)
									{
										if(*c == ' ')
										{
											*c = '\0';
											break;
										}

										c--;
									}
								}
							}

							if(hnode = header_find_tag("Subject", hdrs, 0))
							{
								strncpy(entry->Subject, hnode->Data, sizeof(entry->Subject) - 1);
							}

							/* Determine disposition, if necessary.
							 */
							if(!server->IgnoreFilters)
							{
								gboolean matched = FALSE;

	#if 0
								ObtainSemaphore(&euc.Locks.Config);
								for(fnode = (Filter *)euc.Filters.lh_Head;
								    fnode->Node.ln_Succ && !matched;
								    fnode = (Filter *)fnode->Node.ln_Succ)
								{
									glong l, error = 0;
									gchar *token;

									if(!(l = strlen(fnode->Pattern)))
										continue;

									if(fnode->Disabled)
										continue;

									matched = FALSE;
									if(token = (gchar *)AllocVec(l * 2 + 5, MEMF_CLEAR))
									{
										if(-1 != ParsePatternNoCase(fnode->Pattern, token, l * 2 + 4))
										{
											if(hnode = header_find_tag(fnode->Tag, hdrs, 0))
											{
												if(MatchPatternNoCase(token, hnode->Data))
												{
													matched = TRUE;

													switch(fnode->Type)
													{
														case FNT_DELETE:
															entry->Disposition = PEDT_DELETE;
															strncpy(entry->Status, LocaleString(ETXT_POP_MSGSTATUS_DELETE), sizeof(entry->Status) - 1);
															break;
	/*
														case FNT_FORWARD:
															break;
														case FNT_REPLY:
															break;
	*/
														case FNT_MOVE:
														case FNT_COPY:
														default:
															entry->Disposition = PEDT_MOVE;
															strncpy(entry->DispAction, fnode->Action, sizeof(entry->DispAction) - 1);
															sprintf(entry->Status, LocaleString(ETXT_POP_MSGSTATUS_MOVE), entry->DispAction);
															break;
														case FNT_EXECUTE:
															entry->Disposition = PEDT_EXECUTE;
															strncpy(entry->DispAction, fnode->Action, sizeof(entry->DispAction) - 1);
															sprintf(entry->Status, LocaleString(ETXT_POP_MSGSTATUS_EXECUTE), entry->DispAction);
															break;
													}
												}
											}
										}
										else
											error = err_INVALIDPATTERN;

										FreeVec(token);
										token = NULL;
									}
									else
										error = err_INSUFFICIENTMEMORY;
								}
								ReleaseSemaphore(&euc.Locks.Config);
	#endif
							}

							mailmsg_free_header(hdrs);
						}
						break;
					default:
						strncpy(entry->From, "n/a", sizeof(entry->From) - 1);
						strncpy(entry->Date, "n/a", sizeof(entry->Date) - 1);
						strncpy(entry->Subject, "n/a", sizeof(entry->Subject) - 1);
						break;
				}
			}

			/* Add to list.
			 */
			list_append(&server->Messages, entry);

			/* Update window.
			 * FIXME
			 */
		}
		else
			success = FALSE;
	}

	return success;
}

gboolean pop_server_get_messages(Server *server)
{
	ServerEntry *entry;
	GList *node;
	glong val;

	/* If this server is not connected, we've passed the
	 * top msg count, or there are no msgs, skip this
	 * server and move on to the next.
	 */
	if(!server->Connected ||
	   (server->Count == 0))
		return FALSE;

	for(node = g_list_first(server->Messages.List); node; node = g_list_next(node))
	{
		   entry = (ServerEntry *)node->data;

		if(!entry->Selected ||
		   entry->Done)
			continue;

#if 0	// FIXME
		if(!server->AutoRetrieve &&
		   (control->Flags & PCF_USINGSUMMARY))
		{
			glong disp[] = {ETXT_POP_DISP_MOVE, ETXT_POP_DISP_DELETE, ETXT_POP_DISP_SKIP, };

			sprintf(server->in_buf, LocaleString(ETXT_POP_WHATTODOWITHMSG),
					server->Current, server->Count,
					entry->Size,
					entry->From, entry->Date, entry->Subject,
					LocaleString(disp[entry->Disposition]));

			switch(IsenRequest(euc.Screen,
					ISENREQUEST_Text,   	server->in_buf,
					ISENREQUEST_Button, 	LocaleString(ETXT_REQ_RETRIEVE_SKIP_DELETE_AUTO_ABORT),
					TAG_END))
			{
				case 1:   // retrieve
				default:
					entry->Disposition = PEDT_MOVE;
					break;
				case 2:   // skip
					entry->Disposition = PEDT_SKIP;
					break;
				case 3:   // delete
					entry->Disposition = PEDT_DELETE;
					break;
				case 4:   // auto
					entry->Disposition = PEDT_MOVE;
					server->AutoRetrieve = TRUE;
					break;
				case 0:   // abort
					control->Current = 0;
					control->State = PSS_STOPPING;
					Signal(euc.POPProcess, SIG_CHECK);
					return(0);
			}
		}
		else if(entry->LargeMsg && !entry->Selected)
		{    // msg too large
			glong disp[] = {ETXT_POP_DISP_MOVE, ETXT_POP_DISP_DELETE, ETXT_POP_DISP_SKIP, };

			switch(server->SizeLimitAction)
			{
				case SLAT_ASK:
					sprintf(server->in_buf, LocaleString(ETXT_POP_LARGEMSG),
							server->Current, server->Count,
							entry->Size,
							entry->From, entry->Subject, entry->Date,
							LocaleString(disp[entry->Disposition]),
							server->SizeLimit);

					switch(IsenRequest(euc.Screen,
							ISENREQUEST_Beep,   	TRUE,
							ISENREQUEST_Text,   	server->in_buf,
							ISENREQUEST_Button, 	LocaleString(ETXT_REQ_RETRIEVE_SKIP_DELETE_AUTO_ABORT),
							TAG_END))
					{
						case 1:   // retrieve
						default:
							entry->Disposition = PEDT_MOVE;
							break;
						case 2:   // skip
							entry->Disposition = PEDT_SKIP;
							break;
						case 3:   // delete
							entry->Disposition = PEDT_DELETE;
							break;
						case 4:   // auto
							entry->Disposition = PEDT_MOVE;
							server->AutoRetrieve = TRUE;
							break;
						case 0:   // abort
							control->Current = 0;
							server->State = PSS_STOPPING;
							Signal(euc.POPProcess, SIG_CHECK);
							return(0);
					}
					break;
				case SLAT_DELETE:
					entry->Disposition = PEDT_DELETE;
					break;
				case SLAT_SKIP:
					entry->Disposition = PEDT_SKIP;
					break;
			}
		}
#endif

		switch(entry->Disposition)
		{
			case PEDT_SKIP:
				break;
			case PEDT_EXECUTE:
#if 0
				if(strlen(entry->DispAction))
				{
					BPTR fp;

					if(fp = Open("CON:0/0/640/200/Eucalyptus Output/AUTO/SIMPLE", MODE_OLDFILE))
					{
						glong e;

						switch(e = SystemTags(entry->DispAction,
								SYS_Input,		fp,
								SYS_Output,    	NULL,
								SYS_Asynch,    	TRUE,
								TAG_END))
						{
							case -1:
								Close(fp);
								break;
							case 0:
								break;
							default:
								break;
						}
					}
				}
#endif
			case PEDT_MOVE:
	//   	case PEDT_FORWARD:
			default:
				pop_server_fetch_msg(server, entry, entry->Number, FALSE);
				break;
			case PEDT_DELETE:
#if 0
// FIXME			UpdateFuelGauge(control, LocaleString(ETXT_POP_GAUGE), server->Current, server->Count, control->Level = (server->Level += entry->Size), control->Size);

				sprintf(server->out_buf, "DELE %ld\r\n", server->Current);
				switch(val = pop_server_send(server, server->out_buf, server->in_buf, sizeof(server->in_buf)))
				{
					case 0:   // good
						pop_server_status(server, LocaleString(ETXT_POP_SVRSTATUS_DELETING));
						break;
					default:
						pop_server_status(server, LocaleString(ETXT_POP_SVRSTATUS_UNABLETODELETE));
						break;
				}
#endif
				break;
		}

//		SetPOPEntryStatus(entry, LocaleString(ETXT_POP_MSGSTATUS_DONE), FALSE);
		entry->Done = TRUE;

		server->Current++;
	}

	server->State = PSS_STOPPING;

	return(0);
}

gboolean pop_server_fetch_msg(Server *server, ServerEntry *entry, glong msgnum, gboolean justfetch)
{
	Folder *folder;
	glong val;
	gboolean success = TRUE;

	// get folder
	folder = folder_find_by_name(entry->DispAction);

	sprintf(server->out_buf, "RETR %ld\r\n", msgnum);
	switch(val = pop_server_send(server, server->out_buf, server->in_buf, sizeof(server->in_buf)))
	{
		gchar fname[FILENAME_MAX], tmpname[FILENAME_MAX], status[201];

		case 0:   // good
			g_snprintf(status, sizeof(status) - 1, _("Retrieving message %d of %d..."), msgnum, server->Count);
			pop_server_status(server, status, -1);

			if(!folder)
				folder = folder_get_special(FTF_INCOMING);

			if(folder)
			{
				FILE *fp;
				
				/* Lock this folder, so we don't get interference
				 * from the main thread or any other POP thread
				 * trying to access it.
				 */
				pthread_mutex_lock(&folder->Mutex);

				/* Get a message file.
				 */
				folder_get_next_msg_num(folder, "pop", fname, sizeof(fname));
				// retrieve message
				if(fp = fopen(fname, "w"))
				{
					NewMsg *nmsg;
					gchar *c;
					
					while(pop_read_line(server->Socket, server->in_buf, sizeof(server->in_buf) - 1) > 0)
					{
						gchar *buf;

						buf = server->in_buf;
						if(buf[0] == '.')
						{
							if((buf[1] == '\r') &&
							   (buf[2] == '\n'))
								break;

							buf++;
						}

						server->Level += strlen(buf);
fprintf(stderr, "server->Level = %d\n", server->Level);
						pop_server_status(server, NULL, server->Level);

						fputs(buf, fp);
					}

					fclose(fp);

					strncpy(tmpname, fname, sizeof(tmpname));
					c = strstr(tmpname, G_DIR_SEPARATOR_S "pop");
					memcpy(c + 1, "new", 3);
					rename(fname, tmpname);

					if(nmsg = g_malloc0(sizeof(NewMsg)))
					{
						nmsg->msg.type = MSG_TYPE_NEW;
						nmsg->folder = folder;
						strncpy(nmsg->filename, tmpname, sizeof(nmsg->filename) - 1);

						pthread_mutex_lock(&euc.q.Mutex);
						list_append(&euc.q.Msgs, nmsg);
						pthread_mutex_unlock(&euc.q.Mutex);
					}

					folder->Data.Flags |= (FFF_NEWMSGS | FFF_UPDATEMAIN);
//						SetFolderIndexValid(folder, FALSE);
					server->GotMsgs = TRUE;

					euc.Flags |= EGF_UPDATEFOLDERS;
//						Signal(euc.ThisProcess, SIG_UPDATE);
				}
				else
					success = FALSE;

				pthread_mutex_unlock(&folder->Mutex);
			}
			else
				success = FALSE;

			if(!justfetch)
			{
				if(success &&
				   server->DeleteOnServer)
				{
#if 0
					sprintf(server->out_buf, "DELE %ld\r\n", msgnum);
					switch(val = pop_server_send(server, server->out_buf, server->in_buf, sizeof(server->in_buf)))
					{
						case 0:   // good
							pop_server_status(server, _("Deleting message..."));
							break;
						default:
							pop_server_status(server, _("Unable to delete message."));
							break;
					}
#endif
				}
			}
			break;
		default:
			pop_server_status(server, _("Unable to retrieve message."), -1);
			break;
	}

	return success;
}

void pop_server_status(Server *server, const gchar *text, glong level)
{
	ProgressMsg *pmsg;

	if(!server)
		return;

	if(pmsg = g_malloc0(sizeof(ProgressMsg)))
	{
		pmsg->msg.type = MSG_TYPE_PROGRESS;
		pmsg->who = MSG_WHO_POP;
		pmsg->total = server->Size;
fprintf(stderr, "server->Size = %d\n", server->Size);
		pmsg->current = level;
		pmsg->server = server;
		strncpy(pmsg->id, "", sizeof(pmsg->id) - 1);
		if(text)
		{
			strncpy(pmsg->text, text, sizeof(pmsg->text) - 1);
			strncpy(server->t.Status, text, sizeof(server->t.Status) - 1);
		}

		pthread_mutex_lock(&euc.q.Mutex);
		list_append(&euc.q.Msgs, pmsg);
		pthread_mutex_unlock(&euc.q.Mutex);
	}
}

/* Miscellaneous.
 */
glong pop_read_line(glong sock, gchar *buf, glong buf_len)
{
	glong pos = 0;
	gchar tbuf[5];

	if((sock < 0) ||
	   !buf)
		return(-1);

	memset(buf, 0, buf_len);
	for(;;)
	{
		if(recv(sock, tbuf, 1, 0) < 1)
			return(-1);

		if(pos >= buf_len)
			break;

		if((buf[pos] = tbuf[0]) == '\n')
			break;
		pos++;
	}

	return((glong)strlen(buf));
}

/* Callbacks.
 */
void pop_get_new_mail_cb(GtkWidget *widget, gpointer data)
{
	pop_start_threads();
}

