#ifndef EUCALYPTUS_POP_H
#define EUCALYPTUS_POP_H

/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Header: pop.h
 *   	 Header info for pop.c
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <sys/socket.h>
#include <netinet/in.h>

struct POPControl
{
	struct List		 Servers;

	ServerEntry 		*LastSelected;

	gint				 Timer;
	glong 			 Flags;
};

// Flags
#define PCF_UPDATELIST   	(1L << 0)
#define PCF_REMOVELIST   	(1L << 1)
#define PCF_OPENWINDOW   	(1L << 2)
#define PCF_CLOSEWINDOW  	(1L << 3)
#define PCF_SEMAPHORELOCKED   (1L << 4)
#define PCF_LISTREMOVED  	(1L << 5)
#define PCF_INITIALWARN  	(1L << 6)
#define PCF_USINGFULL    	(1L << 7)
#define PCF_USINGSUMMARY 	(1L << 8)
#define PCF_LOGGING 		(1L << 9)
#define PCF_KILLED  		(1L << 10)
#define PCF_STOPPING		(1L << 11)
#define PCF_STOP			(1L << 12)

// States
#define PSS_WAITING 		(0)
#define PSS_RETRIEVINGHEADERS (1)
#define PSS_RETRIEVING   	(2)
#define PSS_WAITINGUSER  	(3)
#define PSS_STOPPING		(4)

struct _Server
{
	gchar			 ID[41];
	struct {
		GtkWidget	  	*Window,
				  	*Layout,
					*MsgList,
					*Status,
					*Retrieve, *Stop, *Close,
					*Progress,
					*Delete, *Fetch,
					*Disposition,
					*DispAction,
					*SelectDispAction,
					*MarkMessage,
					*AutoRetrieve;
	} w;
	struct {
		GtkObject		*SizeLimit,
					*Port;
	} adj;
	pthread_t			 Thread;
	pthread_attr_t		 ThreadAttr;
	glong 			 Port,    	// column 2 (settings)
					 Socket,
					 Flags,
					 Count,
					 Current, 	// ordinal # of message to retrieve (starts at 1)
					 Size,
					 Level,
					 SizeLimit;	// (settings)
	gchar			 Host[101],    // column 1 (settings)
					 Username[51],  // column 3 (settings)
					 Password[51],	// (settings)
					 SessionPass[51],
					 UIDL[81],
					 Command[301],		// column 3/4 (settings)
					 in_buf[1025],
					 out_buf[1025];
     struct {
		gchar		 Status[101],   // column 5
					 PortText[10],
					 Title[101];
	} t;
	gchar			 Protocol,	// column 4 (settings)
					 NOOPs,
					 SizeLimitAction,	// (settings)
					 State,
					 Type;		// column 0 (settings)
	gboolean			 IgnoreFilters,// (settings)
					 Connected,
					 Authenticated,
					 Disabled,	// (settings)
					 AutoRetrieve,	// (settings)
					 DontRetrieveHeaders,	// (settings)
					 DeleteOnServer,	// (settings)
					 GotMsgs,
					 Running;
	struct sockaddr_in   SockAddr;
	struct List		 Messages;
};

// Flags
#define PISF_TOPNOTSUPPORTED  	(1L << 0)
#define PISF_NOMOREMSGS  		(1L << 1)
#define PISF_BADDISCONNECT    	(1L << 2)

// Server Protocols
#define PST_POP3  	(0)
#define PST_IMAP4 	(1)

struct _ServerEntry
{
	Server			*Server;
	glong 			 Number,
					 Size;    	// column 3
	gchar			 From[101],    // column 0
					 Date[31],	// column 1
					 Subject[301], // column 2
					 Status[301],  // column 4; where msg is going / what server is doing
					 UIDL[81],
					 DispAction[101];
	gchar			 Disposition;
	gboolean			 LargeMsg,
					 Selected,
					 Done;    	// pop client is done with this node (retrieved/deleted)
};

struct _ServerInfo
{
	gchar			 Info[101];
};

/* Node.ln_Types for Server* nodes */
#define PSNT_SERVER 	(0)
#define PSNT_MESSAGE	(1)
#define PSNT_INFO   	(2)

#define PEDT_MOVE   	(0)
#define PEDT_DELETE 	(1)
#define PEDT_REPLY  	(2)  /* not used */
#define PEDT_FORWARD	(3)  /* not used */
#define PEDT_BOUNCE 	(4)  /* not used */
#define PEDT_SKIP   	(5)  
#define PEDT_EXECUTE	(6)  

// types for GetNode()
#define GN_SERVER   	(0)
#define GN_MESSAGE  	(1)
#define GN_INFO		(2)

#endif /* EUCALYPTUS_POP_H */
