#ifndef EUCALYPTUS_PREFS_H
#define EUCALYPTUS_PREFS_H

/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Header: prefs.h
 *   	 Settings and Settings panel control structures
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include "list.h"

struct Prefs
{
	GdkColor 			 PersonalPen,	/* Messages/Display/Colors */
					 PriorityPen,	/* Messages/Display/Colors */
					 ImportantPen;
	glong 			 SMTPPort,	/* Messages/Sending/Sending */
					 SendInterval,	/* Messages/Sending/Misc. */
					 GetInterval;	/* Messages/Receiving/Misc. */
	gint				 MainWindowLeft, MainWindowTop,	/* General/Display/Window */
					 MainWindowWidth, MainWindowHeight,	/* General/Display/Window */
					 FolderWindowLeft, FolderWindowTop,	/* General/Folders/Window */
					 FolderWindowWidth, FolderWindowHeight,	/* General/Folders/Window */
					 DeleteSentMail,	/* General/Folders/Maintenance */
					 MsgWindowLeft, MsgWindowTop,	/* Messages/Display/Window */
					 MsgWindowWidth, MsgWindowHeight,	/* Messages/Display/Window */
					 SMTPWindowLeft, SMTPWindowTop,	/* Messages/Sending/Window */
					 SMTPWindowWidth, SMTPWindowHeight,	/* Messages/Sending/Window */
					 POPWindowLeft, POPWindowTop,	/* Messages/Receiving/Window */
					 POPWindowWidth, POPWindowHeight,	/* Messages/Receiving/Window */
					 ComposeWindowLeft, ComposeWindowTop,	/* Compose/Window */
					 ComposeWindowWidth, ComposeWindowHeight,	/* Compose/Window */
					 AddrWindowLeft, AddrWindowTop,	/* Address book/Window */
					 AddrWindowWidth, AddrWindowHeight, /* Address book/Window */
					 AddressBookSort;	/* Address book/Sorting */
	gchar			 MsgFont[201],	/* Messages/Display/Reader */
					 HeaderOrder[151],	/* Messages/Display/Reader */
					 HideHeaders[151],	/* Messages/Display/Reader */
					 TabWidth,	/* Messages/Display/Parser */
				      DisplayFilter[FILENAME_MAX],	/* Messages/Display/Parser */
					 SecurityProgram,	/* Messages/Security/Security program */
			 		 SMTPServer[101],	/* Messages/Sending/Sending */
					 SMTPCommand[301],	/* Messages/Sending/Sending */
					 SMTPType,		/* Messages/Sending/Sending */
			 		 POPWindowType,	/* Messages/Receiving/Misc. */
					 NoMailAction[FILENAME_MAX],	/* Messages/Receiving/Misc. */
					 NewMailAction[FILENAME_MAX],	/* Messages/Receiving/Misc. */
					 QuotePrefix[11],	/* Compose/Reply/Forward */
					 ReplyHeader[101],	/* Compose/Reply/Forward */
					 ForwardHeader[101],	/* Compose/Reply/Forward */
					 MaxLineLength,	/* Compose/Reply/Forward */
					 EOLType,	/* Compose/Misc. */
					 ReplyDefault,	/* Compose/Reply/Forward */
					 ForwardDefault;	/* Compose/Reply/Forward */
	gboolean			 AskBeforeQuitting,	/* General/General */
				      ExpungeDeleted,	/* General/Folders/Maintenance */
					 OpenFirstNew,	/* Messages/General */
					 DisableReturnReceipt,	/* Messages/General */
					 StripCodes,	/* Messages/Display/Reader */
					 WrapText,	/* Messages/Display/Reader */
					 ShowOnlyHeaders,	/* Messages/Display/Reader */
				      InitialMailSend,	/* Messages/Sending/Misc. */
					 InitialMailGet,	/* Messages/Receiving/Misc. */
					 UseTagline,	/* Compose/Misc. */
					 UseSignature,	/* Compose/Misc. */
					 CloseMsgOnReply,	/* Compose/Reply/Forward */
					 UseQuotedPrintable,	/* Compose/Misc. */
					 SignMsgs,	/* Compose/Security/Security */
					 EncryptMsgs,	/* Compose/Security/Security */
				      KeepAddressesSorted,	/* Address book/Sorting */
					 SortGroupsAs;	/* Address book/Sorting */
};

/* AddressBookSort types */
#define ABST_FIRSTNAME   	(0)
#define ABST_LASTNAME    	(1)
#define ABST_ADDRESS		(2)

/* SortGroupsAs */
#define SGAT_FIRST  		(0)
#define SGAT_MIXED  		(1)
#define SGAT_LAST   		(2)

/* EOLTypes */
#define EOLT_CRLF   	(0)
#define EOLT_CRONLY 	(1)
#define EOLT_LFONLY 	(2)

struct User
{
	gchar			 RealName[101],
					 Address[101],
					 ReplyTo[101],
					 Organization[101],
					 Signature[FILENAME_MAX],
					 Taglines[FILENAME_MAX],
					 AddressBook[FILENAME_MAX],
					 MailDir[301],
					 Mailcap[FILENAME_MAX],
					 Filters[FILENAME_MAX];
};

/* POP/SMTPTypes */
#define PST_INTERNAL		(0)  /* use internal */
#define PST_COMMAND 		(1)  /* command to execute */

/* POPWindowTypes */
#define PWTT_FULL   		(0)
#define PWTT_SUMMARY		(1)

/* POPWindowStates */
#define PWST_CLOSED 		(0)
#define PWST_ICONIFIED   	(1)
#define PWST_OPEN   		(2)

/* SMTPWindowStates */
#define SWST_CLOSED 		(0)
#define SWST_ICONIFIED   	(1)
#define SWST_OPEN   		(2)

/* ReplyDefault types */
#define RDT_SENDER  		(0)
#define RDT_SENDERNOTEXT 	(1)
#define RDT_ALL			(2)
#define RDT_ALLNOTEXT    	(3)

/* ForwardDefault types */
#define FDT_NONE    		(0)
#define FDT_BRIEF   		(1)
#define FDT_FULL    		(2)

/* SizeLimitAction types */
#define SLAT_ASK    		(0)
#define SLAT_SKIP   		(1)
#define SLAT_DELETE 		(2)

/* Control structures.
 */
struct PrefsControl
{
	GtkWidget		  	*Window,
				     *List,
					*Layout,
	 			 	*PersonalPen,	/* Messages/Display */
					*PriorityPen,	/* Messages/Display */
	   	 			*SMTPPort,	/* Messages/Sending */
					*SendInterval,	/* Messages/Sending */
					*GetInterval,	/* Messages/Receiving */
					*MainWindowLeft, 
					*MainWindowTop,
					*MainWindowWidth, 
					*MainWindowHeight,
					*SnapshotMain,
					*POPWindowLeft, 
					*POPWindowTop,
					*POPWindowWidth, 
					*POPWindowHeight,
					*SnapshotPOP,
					*SMTPWindowLeft, 
					*SMTPWindowTop,
					*SMTPWindowWidth, 
					*SMTPWindowHeight,
					*SnapshotSMTP,
					*ComposeWindowLeft, 
					*ComposeWindowTop,
					*ComposeWindowWidth, 
					*ComposeWindowHeight,
					*SnapshotCompose,
					*MsgWindowLeft, 
					*MsgWindowTop,
					*MsgWindowWidth, 
					*MsgWindowHeight,
					*SnapshotMsg,
					*FolderWindowLeft, 
					*FolderWindowTop,
					*FolderWindowWidth, 
					*FolderWindowHeight,
					*SnapshotFolder,
					*AddrWindowLeft, 
					*AddrWindowTop,
					*AddrWindowWidth, 
					*AddrWindowHeight, 
					*SnapshotAddr,
					*DeleteSentMail,	/* General/Folders */
					*AddressBookSort,	/* Address book */
			 		*MsgFont,	/* Messages/Display */
					*TabWidth,	/* Messages/Display */
					*MaxLineLength,	/* Messages/Display */
				     *DisplayFilter,	/* Messages/Display */
					*HeaderOrder,	/* Messages/Display */
					*HideHeaders,	/* Messages/Display */
			 		*SecurityProgram,	/* Messages/Security */
			 		*SMTPServer,	/* Messages/Sending */
					*SMTPCommand,	/* Messages/Sending */
					*SMTPType,		/* Messages/Sending */
				     *InitialMailSend,	/* Messages/Sending */
					*InitialMailGet,	/* Messages/Receiving */
					*ServerList,
					*NewPOP, *EditPOP, *DelPOP,
					*DictList,
					*NewDict, *EditDict, *DelDict,
			 		*POPWindowType,	/* Messages/Receiving */
					*NoMailAction,	/* Messages/Receiving */
					*NewMailAction,	/* Messages/Receiving */
					*QuotePrefix,	/* Compose */
					*ReplyHeader,	/* Compose */
					*ForwardHeader,	/* Compose */
					*EOLType,	/* Compose */
					*ReplyDefault,	/* Compose */
					*ForwardDefault,	/* Compose */
				 	*AskBeforeQuitting,	/* General */
				     *ExpungeDeleted,	/* General/Folders */
					*OpenFirstNew,	/* Messages */
					*DisableReturnReceipt,	/* Messages */
					*StripCodes,	/* Messages/Display */
					*WrapText,	/* Messages/Display */
					*ShowOnlyHeaders,	/* Messages/Display */
					*UseTagline,	/* Compose */
					*UseSignature,	/* Compose */
					*CloseMsgOnReply,	/* Compose */
					*UseQuotedPrintable,	/* Compose */
					*SignMsgs,	/* Compose/Security */
					*EncryptMsgs,	/* Compose/Security */
				     *KeepAddressesSorted,	/* Address book */
					*SortGroupsAs,	/* Address book */
					*PreviewPane;
	struct {
		GtkObject		*TabWidth,
					*SendInterval,
					*GetInterval,
					*SMTPPort,
					*DeleteSentMail,
					*MaxLineLength,
					*MainWindowLeft, *MainWindowTop,
					*MainWindowWidth, *MainWindowHeight,
					*FolderWindowLeft, *FolderWindowTop,
					*FolderWindowWidth, *FolderWindowHeight,
					*MsgWindowLeft, *MsgWindowTop,
					*MsgWindowWidth, *MsgWindowHeight,
					*ComposeWindowLeft, *ComposeWindowTop,
					*ComposeWindowWidth, *ComposeWindowHeight,
					*AddrWindowLeft, *AddrWindowTop,
					*AddrWindowWidth, *AddrWindowHeight,
					*POPWindowLeft, *POPWindowTop,
					*POPWindowWidth, *POPWindowHeight,
					*SMTPWindowLeft, *SMTPWindowTop,
					*SMTPWindowWidth, *SMTPWindowHeight;
	} adj;
	struct Prefs	 	*Prefs,
					*Backup;
	Server			*CurrentServer;
	Dictionary		*CurrentDict;
	struct List		 Servers,
				      Dictionaries;
};

/* Flags */
#define GPF_DISPLAYCHANGE		(1L << 0)

struct EucDruid
{
	GtkWidget			*Window,
					*StartPage, *EndPage,
					*CurrentPage,
					*Druid,
					*RealName, *Organization,
					*Address,
					*SMTPServerName, *SMTPServerPort,
					*POPServerName, *POPServerPort,
					*POPUsername, *POPPassword;
	GtkObject			*SMTPAdjustment,
					*POPAdjustment;
};

#endif /* EUCALYPTUS_PREFS_H */
