/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: prefs_compose.c
 *   	 Prefs layout code
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <string.h>

#include "eucalyptus.h"

GtkWidget *prefs_setup_compose(void)
{
	GtkWidget *page = NULL, *frame, *table;

	page = gtk_vbox_new(FALSE, 0);

	frame = gtk_frame_new(_("Misc."));
	{
		table = gtk_table_new(1, 2, FALSE);
		{
			   guint top = 0;

			   prefs_control.UseTagline = gtk_check_button_new_with_label(_("Use tagline?"));
			   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.UseTagline), prefs_control.Prefs->UseTagline);
			   gtk_signal_connect(GTK_OBJECT(prefs_control.UseTagline), "toggled",
						    GTK_SIGNAL_FUNC(prefs_setting_usetagline_cb), NULL);
			   prefs_layout_table_item(table, prefs_control.UseTagline, NULL, &top, 0);

			   prefs_control.UseSignature = gtk_check_button_new_with_label(_("Use signature?"));
			   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.UseSignature), prefs_control.Prefs->UseSignature);
			   gtk_signal_connect(GTK_OBJECT(prefs_control.UseSignature), "toggled",
						    GTK_SIGNAL_FUNC(prefs_setting_usesignature_cb), NULL);
			   prefs_layout_table_item(table, prefs_control.UseSignature, NULL, &top, 0);

			   prefs_control.UseQuotedPrintable = gtk_check_button_new_with_label(_("Use quoted-printable encoding?"));
			   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.UseQuotedPrintable), prefs_control.Prefs->UseQuotedPrintable);
			   gtk_signal_connect(GTK_OBJECT(prefs_control.UseQuotedPrintable), "toggled",
						    GTK_SIGNAL_FUNC(prefs_setting_usequotedprintable_cb), NULL);
			   prefs_layout_table_item(table, prefs_control.UseQuotedPrintable, NULL, &top, 0);

			prefs_control.EOLType = gtk_option_menu_new();
			{
				GtkWidget *menu, *item;
				GSList *group = NULL;
				glong i;
				gchar *types[] = { N_("CR/LF"), N_("CR only"), N_("LF only"), NULL, };
				
				menu = gtk_menu_new();

				for(i=0; types[i]; i++)
				{
					item = gtk_radio_menu_item_new_with_label(group, _(types[i]));
					group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(item));
					gtk_signal_connect(GTK_OBJECT(item), "activate",
							GTK_SIGNAL_FUNC(prefs_setting_eoltype_cb), GINT_TO_POINTER(i));
					gtk_menu_append(GTK_MENU(menu), item);
				}

				gtk_option_menu_set_menu(GTK_OPTION_MENU(prefs_control.EOLType), menu);
			}
			gtk_option_menu_set_history(GTK_OPTION_MENU(prefs_control.EOLType), prefs_control.Prefs->EOLType);
			prefs_layout_table_item(table, prefs_control.EOLType, _("End-of-line type"), &top, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	frame = gtk_frame_new(_("Window"));
	{
		table = gtk_table_new(3, 4, FALSE);
		{
			guint top = 0;

			prefs_control.ComposeWindowLeft = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.ComposeWindowLeft), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->ComposeWindowLeft);
			gtk_signal_connect(GTK_OBJECT(prefs_control.ComposeWindowLeft), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_composewindowleft_cb), NULL);
			prefs_layout_table_item(table, prefs_control.ComposeWindowLeft, _("Left"), &top, 0);
			top--;

			prefs_control.ComposeWindowTop = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.ComposeWindowTop), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->ComposeWindowTop);
			gtk_signal_connect(GTK_OBJECT(prefs_control.ComposeWindowTop), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_composewindowtop_cb), NULL);
			prefs_layout_table_item(table, prefs_control.ComposeWindowTop, _("Top"), &top, 2);

			prefs_control.ComposeWindowWidth = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.ComposeWindowWidth), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->ComposeWindowWidth);
			gtk_signal_connect(GTK_OBJECT(prefs_control.ComposeWindowWidth), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_composewindowwidth_cb), NULL);
			prefs_layout_table_item(table, prefs_control.ComposeWindowWidth, _("Width"), &top, 0);
			top--;

			prefs_control.ComposeWindowHeight = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.ComposeWindowHeight), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->ComposeWindowHeight);
			gtk_signal_connect(GTK_OBJECT(prefs_control.ComposeWindowHeight), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_composewindowheight_cb), NULL);
			prefs_layout_table_item(table, prefs_control.ComposeWindowHeight, _("Height"), &top, 2);

			prefs_control.SnapshotCompose = gtk_button_new_with_label(_("Snapshot"));
			gtk_signal_connect(GTK_OBJECT(prefs_control.SnapshotCompose), "clicked",
					GTK_SIGNAL_FUNC(prefs_setting_snapshotcompose_cb), NULL);
			gtk_table_attach(GTK_TABLE(table), prefs_control.SnapshotCompose,
					1, 2, top, top + 1,
					GTK_FILL, GTK_FILL,
					0, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);


	return page;
}

GtkWidget *prefs_setup_compose_replyforward(void)
{
	GtkWidget *page = NULL, *frame, *table;

	page = gtk_vbox_new(FALSE, 0);

	frame = gtk_frame_new(_("Reply/Forward"));
	{
		table = gtk_table_new(1, 2, FALSE);
		{
			   glong i;
			   guint top = 0;
			   gchar *prefixes[] = { ">", "|", ":", NULL, };
			   gchar *rhdrs[] = { N_(""), NULL, };
			   gchar *fhdrs[] = { N_(""), NULL, };

			   prefs_control.QuotePrefix = gnome_entry_new("prefs:quote-prefix");
			   gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(prefs_control.QuotePrefix))), prefs_control.Prefs->QuotePrefix);
			   for(i=0; prefixes[i]; i++)
					 gnome_entry_append_history(GNOME_ENTRY(prefs_control.QuotePrefix), FALSE, prefixes[i]);
			   gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(prefs_control.QuotePrefix))), "changed",
						    GTK_SIGNAL_FUNC(prefs_setting_quoteprefix_cb), NULL);
			   prefs_layout_table_item(table, prefs_control.QuotePrefix, _("Quote prefix"), &top, 0);

			   prefs_control.ReplyHeader = gnome_entry_new("prefs:reply-header");
			   gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(prefs_control.ReplyHeader))), prefs_control.Prefs->ReplyHeader);
			   for(i=0; rhdrs[i]; i++)
					 gnome_entry_append_history(GNOME_ENTRY(prefs_control.ReplyHeader), FALSE, rhdrs[i]);
			   gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(prefs_control.ReplyHeader))), "changed",
						    GTK_SIGNAL_FUNC(prefs_setting_replyheader_cb), NULL);
			   prefs_layout_table_item(table, prefs_control.ReplyHeader, _("Reply header"), &top, 0);

			   prefs_control.ForwardHeader = gnome_entry_new("prefs:forward-header");
			   gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(prefs_control.ForwardHeader))), prefs_control.Prefs->ForwardHeader);
			   for(i=0; fhdrs[i]; i++)
					 gnome_entry_append_history(GNOME_ENTRY(prefs_control.ForwardHeader), FALSE, fhdrs[i]);
			   gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(prefs_control.ForwardHeader))), "changed",
						    GTK_SIGNAL_FUNC(prefs_setting_forwardheader_cb), NULL);
			   prefs_layout_table_item(table, prefs_control.ForwardHeader, _("Forward header"), &top, 0);

			   prefs_control.MaxLineLength = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.MaxLineLength), 1, 0);
			   gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MaxLineLength), prefs_control.Prefs->MaxLineLength);
			   gtk_signal_connect(GTK_OBJECT(prefs_control.MaxLineLength), "changed",
						    GTK_SIGNAL_FUNC(prefs_setting_maxlinelength_cb), NULL);
			   prefs_layout_table_item(table, prefs_control.MaxLineLength, _("Max line length"), &top, 0);

			prefs_control.ReplyDefault = gtk_option_menu_new();
			{
				GtkWidget *menu, *item;
				GSList *group = NULL;
				gchar *types[] = { N_("Sender"), N_("Sender, no text"), N_("All"), N_("All, no text"), NULL, };
				
				menu = gtk_menu_new();

				for(i=0; types[i]; i++)
				{
					item = gtk_radio_menu_item_new_with_label(group, _(types[i]));
					group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(item));
					gtk_signal_connect(GTK_OBJECT(item), "activate",
							GTK_SIGNAL_FUNC(prefs_setting_replydefault_cb), GINT_TO_POINTER(i));
					gtk_menu_append(GTK_MENU(menu), item);
				}

				gtk_option_menu_set_menu(GTK_OPTION_MENU(prefs_control.ReplyDefault), menu);
			}		
			gtk_option_menu_set_history(GTK_OPTION_MENU(prefs_control.ReplyDefault), prefs_control.Prefs->ReplyDefault);
			prefs_layout_table_item(table, prefs_control.ReplyDefault, _("Reply default"), &top, 0);

			prefs_control.ForwardDefault = gtk_option_menu_new();
			{
				GtkWidget *menu, *item;
				GSList *group = NULL;
				gchar *types[] = { N_("None"), N_("Brief"), N_("Full"), NULL, };
				
				menu = gtk_menu_new();

				for(i=0; types[i]; i++)
				{
					item = gtk_radio_menu_item_new_with_label(group, _(types[i]));
					group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(item));
					gtk_signal_connect(GTK_OBJECT(item), "activate",
							GTK_SIGNAL_FUNC(prefs_setting_forwarddefault_cb), GINT_TO_POINTER(i));
					gtk_menu_append(GTK_MENU(menu), item);
				}

				gtk_option_menu_set_menu(GTK_OPTION_MENU(prefs_control.ForwardDefault), menu);
			}		
			gtk_option_menu_set_history(GTK_OPTION_MENU(prefs_control.ForwardDefault), prefs_control.Prefs->ForwardDefault);
			prefs_layout_table_item(table, prefs_control.ForwardDefault, _("Forward default"), &top, 0);

			prefs_control.CloseMsgOnReply = gtk_check_button_new_with_label(_("Close message on reply?"));
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.CloseMsgOnReply), prefs_control.Prefs->CloseMsgOnReply);
			gtk_signal_connect(GTK_OBJECT(prefs_control.CloseMsgOnReply), "toggled",
						 GTK_SIGNAL_FUNC(prefs_setting_closemsgonreply_cb), NULL);
			prefs_layout_table_item(table, prefs_control.CloseMsgOnReply, NULL, &top, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	return page;
}

GtkWidget *prefs_setup_compose_security(void)
{
	GtkWidget *page = NULL, *frame, *table;

	page = gtk_vbox_new(FALSE, 0);

	frame = gtk_frame_new(_("Security"));
	{
		table = gtk_table_new(1, 2, FALSE);
		{
			   guint top = 0;

			   prefs_control.SignMsgs = gtk_check_button_new_with_label(_("Sign messages?"));
			   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.SignMsgs), prefs_control.Prefs->SignMsgs);
			   gtk_signal_connect(GTK_OBJECT(prefs_control.SignMsgs), "toggled",
						    GTK_SIGNAL_FUNC(prefs_setting_signmsgs_cb), NULL);
			   prefs_layout_table_item(table, prefs_control.SignMsgs, NULL, &top, 0);

			   prefs_control.EncryptMsgs = gtk_check_button_new_with_label(_("Encrypt messages?"));
			   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.EncryptMsgs), prefs_control.Prefs->EncryptMsgs);
			   gtk_signal_connect(GTK_OBJECT(prefs_control.EncryptMsgs), "toggled",
						    GTK_SIGNAL_FUNC(prefs_setting_encryptmsgs_cb), NULL);
			   prefs_layout_table_item(table, prefs_control.EncryptMsgs, NULL, &top, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	return page;
}

GtkWidget *prefs_setup_compose_spelling(void)
{
	GtkWidget *page = NULL, *frame, *table;

	page = gtk_vbox_new(FALSE, 0);

	frame = gtk_frame_new(_("Dictionaries"));
	{
		table = gtk_table_new(1, 2, FALSE);
		{
			   prefs_control.DictList = gtk_clist_new(3);
			   gtk_signal_connect(GTK_OBJECT(prefs_control.DictList), "select-row",
						    GTK_SIGNAL_FUNC(prefs_setting_selectdictserver_cb), NULL);
			   gtk_table_attach(GTK_TABLE(table), prefs_control.DictList,
						    0, 1, 0, 4,
						    GTK_FILL | GTK_EXPAND,
						    GTK_FILL | GTK_EXPAND,
						    0, 0);

			   prefs_control.NewDict = gtk_button_new_with_label(_("New"));
			   gtk_signal_connect(GTK_OBJECT(prefs_control.NewDict), "clicked",
						    GTK_SIGNAL_FUNC(prefs_setting_newdict_cb), NULL);
			   gtk_table_attach(GTK_TABLE(table), prefs_control.NewDict,
						    1, 2, 0, 1,
						    GTK_FILL, GTK_FILL,
						    0, 0);

			   prefs_control.EditDict = gtk_button_new_with_label(_("Edit"));
			   gtk_signal_connect(GTK_OBJECT(prefs_control.EditDict), "clicked",
						    GTK_SIGNAL_FUNC(prefs_setting_editdict_cb), NULL);
			   gtk_table_attach(GTK_TABLE(table), prefs_control.EditDict,
						    1, 2, 1, 2,
						    GTK_FILL, GTK_FILL,
						    0, 0);

			   prefs_control.DelDict = gtk_button_new_with_label(_("Del"));
			   gtk_signal_connect(GTK_OBJECT(prefs_control.DelDict), "clicked",
						    GTK_SIGNAL_FUNC(prefs_setting_deldict_cb), NULL);
			   gtk_table_attach(GTK_TABLE(table), prefs_control.DelDict,
						    1, 2, 2, 3,
						    GTK_FILL, GTK_FILL,
						    0, 0);

		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	return page;
}

void prefs_populate_dict_list(void)
{
	   GList *node;

	   gtk_clist_freeze(GTK_CLIST(prefs_control.DictList));
	   gtk_clist_clear(GTK_CLIST(prefs_control.DictList));

	   for(node = g_list_first(prefs_control.Dictionaries.List); node; node = g_list_next(node))
	   {
			 Dictionary *dict = (Dictionary *)node->data;
			 gchar *cols[3];

			 g_snprintf(dict->PortText, sizeof(dict->PortText), "%d", dict->Port);
			 cols[0] = dict->Host;
			 cols[1] = dict->PortText;
			 cols[2] = dict->Username;

			 gtk_clist_append(GTK_CLIST(prefs_control.DictList), cols);
	   }

	   gtk_clist_thaw(GTK_CLIST(prefs_control.DictList));
}

void prefs_edit_dictionary(Dictionary *dict)
{
	   if(!dict)
			 return;
}

/* Callbacks.
 */
void prefs_setting_composewindowleft_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->ComposeWindowLeft = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_composewindowtop_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->ComposeWindowTop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_composewindowwidth_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->ComposeWindowWidth = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_composewindowheight_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->ComposeWindowHeight = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_snapshotcompose_cb(GtkWidget *widget, gpointer data)
{
	   GList *node;
	   GdkWindow *window = NULL;

	for(node = g_list_first(euc.Composes.List); node; node = g_list_next(node))
	{
		Composition *comp;

		comp = (Composition *)node->data;
		if(comp->Window)
		{
			window = comp->Window->window;
			break;
		}
	}

	if(window)
	{
		   gint x, y, w, h;

		gdk_window_get_position(window, &x, &y);
		gdk_window_get_size(window, &w, &h);
	
		prefs_control.Prefs->ComposeWindowLeft = x;
		prefs_control.Prefs->ComposeWindowTop = y;
		prefs_control.Prefs->ComposeWindowWidth = w;
		prefs_control.Prefs->ComposeWindowHeight = h;

		gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.ComposeWindowLeft), x);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.ComposeWindowTop), y);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.ComposeWindowWidth), w);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.ComposeWindowHeight), h);
	}

}

void prefs_setting_usetagline_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->UseTagline = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_setting_usesignature_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->UseSignature = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_setting_usequotedprintable_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->UseQuotedPrintable = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_setting_eoltype_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->EOLType = GPOINTER_TO_INT(data);
}

void prefs_setting_quoteprefix_cb(GtkWidget *widget, gpointer data)
{
	   gchar *text;

	   text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	   strncpy(prefs_control.Prefs->QuotePrefix, text, sizeof(prefs_control.Prefs->QuotePrefix) - 1);
	   g_free(text);
}

void prefs_setting_replyheader_cb(GtkWidget *widget, gpointer data)
{
	   gchar *text;

	   text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	   strncpy(prefs_control.Prefs->ReplyHeader, text, sizeof(prefs_control.Prefs->ReplyHeader) - 1);
	   g_free(text);
}

void prefs_setting_forwardheader_cb(GtkWidget *widget, gpointer data)
{
	   gchar *text;

	   text = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
	   strncpy(prefs_control.Prefs->ForwardHeader, text, sizeof(prefs_control.Prefs->ForwardHeader) - 1);
	   g_free(text);
}

void prefs_setting_maxlinelength_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->MaxLineLength = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_replydefault_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->ReplyDefault = (gchar)GPOINTER_TO_INT(data);
}

void prefs_setting_forwarddefault_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->ForwardDefault = (gchar)GPOINTER_TO_INT(data);
}

void prefs_setting_closemsgonreply_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->CloseMsgOnReply = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_setting_selectdictserver_cb(GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer data)
{
	   prefs_control.CurrentDict = g_list_nth_data(prefs_control.Dictionaries.List, row);

	   gtk_widget_set_sensitive(prefs_control.EditDict, (gboolean)prefs_control.CurrentDict);
	   gtk_widget_set_sensitive(prefs_control.DelDict, (gboolean)prefs_control.CurrentDict);
}

void prefs_setting_newdict_cb(GtkWidget *widget, gpointer data)
{
	   Dictionary *dict;

	   if(dict = compose_dictionary_new())
	   {
			 glong count;
			 
			 list_append(&prefs_control.Dictionaries, dict);
			 count = g_list_length(prefs_control.Dictionaries.List);

			 prefs_populate_dict_list();

			 gtk_clist_select_row(GTK_CLIST(prefs_control.DictList), count - 1, 0);
	   }
}

void prefs_setting_editdict_cb(GtkWidget *widget, gpointer data)
{
	   prefs_edit_dictionary(prefs_control.CurrentDict);
}

void prefs_setting_deldict_cb(GtkWidget *widget, gpointer data)
{
	   if(!prefs_control.CurrentDict)
			 return;

	   list_remove(&prefs_control.Dictionaries, prefs_control.CurrentDict);
	   compose_dictionary_free(prefs_control.CurrentDict);

	   prefs_control.CurrentDict = NULL;

	   prefs_populate_dict_list();

	   gtk_widget_set_sensitive(prefs_control.EditDict, (gboolean)prefs_control.CurrentDict);
	   gtk_widget_set_sensitive(prefs_control.DelDict, (gboolean)prefs_control.CurrentDict);
}

void prefs_setting_signmsgs_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->SignMsgs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

void prefs_setting_encryptmsgs_cb(GtkWidget *widget, gpointer data)
{
	   prefs_control.Prefs->EncryptMsgs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}

