/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: prefs_general.c
 *   	 Prefs handling code for general settings panel
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>

#include "eucalyptus.h"

void prefs_general_set_defaults(void)
{
}

GtkWidget *prefs_setup_general(void)
{
	GtkWidget *page = NULL, *frame, *vbox, *table;

	page = gtk_vbox_new(FALSE, 0);

	frame = gtk_frame_new(_("General"));
	{
		vbox = gtk_vbox_new(FALSE, 0);
		{
			prefs_control.AskBeforeQuitting = gtk_check_button_new_with_label(_("Ask before quitting?"));
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_control.AskBeforeQuitting), prefs_control.Prefs->AskBeforeQuitting);
			gtk_signal_connect(GTK_OBJECT(prefs_control.AskBeforeQuitting), "toggled",
					GTK_SIGNAL_FUNC(prefs_setting_askbeforequitting_cb), NULL);
			gtk_box_pack_start(GTK_BOX(vbox), prefs_control.AskBeforeQuitting, FALSE, TRUE, 2);
		}
		gtk_container_add(GTK_CONTAINER(frame), vbox);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	frame = gtk_frame_new(_("Main window position"));
	{
		table = gtk_table_new(3, 4, FALSE);
		{
			guint top = 0;

			prefs_control.MainWindowLeft = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.MainWindowLeft), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->MainWindowLeft);
			gtk_signal_connect(GTK_OBJECT(prefs_control.MainWindowLeft), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_mainwindowleft_cb), NULL);
			prefs_layout_table_item(table, prefs_control.MainWindowLeft, _("Left"), &top, 0);
			top--;

			prefs_control.MainWindowTop = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.MainWindowTop), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->MainWindowTop);
			gtk_signal_connect(GTK_OBJECT(prefs_control.MainWindowTop), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_mainwindowtop_cb), NULL);
			prefs_layout_table_item(table, prefs_control.MainWindowTop, _("Top"), &top, 2);

			prefs_control.MainWindowWidth = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.MainWindowWidth), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->MainWindowWidth);
			gtk_signal_connect(GTK_OBJECT(prefs_control.MainWindowWidth), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_mainwindowwidth_cb), NULL);
			prefs_layout_table_item(table, prefs_control.MainWindowWidth, _("Width"), &top, 0);
			top--;

			prefs_control.MainWindowHeight = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.MainWindowHeight), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->MainWindowHeight);
			gtk_signal_connect(GTK_OBJECT(prefs_control.MainWindowHeight), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_mainwindowheight_cb), NULL);
			prefs_layout_table_item(table, prefs_control.MainWindowHeight, _("Height"), &top, 2);

			prefs_control.SnapshotMain = gtk_button_new_with_label(_("Snapshot"));
			gtk_signal_connect(GTK_OBJECT(prefs_control.SnapshotMain), "clicked",
					GTK_SIGNAL_FUNC(prefs_setting_snapshotmain_cb), NULL);
			gtk_table_attach(GTK_TABLE(table), prefs_control.SnapshotMain,
					1, 2, top, top + 1,
					GTK_FILL, GTK_FILL,
					0, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

	return page;
}

GtkWidget *prefs_setup_general_folders(void)
{
	GtkWidget *page = NULL, *frame, *table;

	page = gtk_vbox_new(FALSE, 0);

	frame = gtk_frame_new(_("Messages"));
	{
		table = gtk_table_new(1, 2, FALSE);
		{
			guint top = 0;

			prefs_control.DeleteSentMail = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.DeleteSentMail), 1, 0);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.DeleteSentMail), prefs_control.Prefs->DeleteSentMail);
			gtk_signal_connect(GTK_OBJECT(prefs_control.DeleteSentMail), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_deletesentmail_cb), NULL);
			prefs_layout_table_item(table, prefs_control.DeleteSentMail, _("Delete mail older than (days)"), &top, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);

#if 0
	frame = gtk_frame_new(_("Window position"));
	{
		table = gtk_table_new(3, 4, FALSE);
		{
			guint top = 0;

			prefs_control.FolderWindowLeft = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.FolderWindowLeft), 1, 0);
			gtk_widget_set_sensitive(prefs_control.FolderWindowLeft, !euc.IntegratedDisplay);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->FolderWindowLeft);
			gtk_signal_connect(GTK_OBJECT(prefs_control.FolderWindowLeft), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_folderwindowleft_cb), NULL);
			prefs_layout_table_item(table, prefs_control.FolderWindowLeft, _("Left"), &top, 0);
			top--;

			prefs_control.FolderWindowTop = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.FolderWindowTop), 1, 0);
			gtk_widget_set_sensitive(prefs_control.FolderWindowTop, !euc.IntegratedDisplay);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->FolderWindowTop);
			gtk_signal_connect(GTK_OBJECT(prefs_control.FolderWindowTop), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_folderwindowtop_cb), NULL);
			prefs_layout_table_item(table, prefs_control.FolderWindowTop, _("Top"), &top, 2);

			prefs_control.FolderWindowWidth = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.FolderWindowTop), 1, 0);
			gtk_widget_set_sensitive(prefs_control.FolderWindowWidth, !euc.IntegratedDisplay);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->FolderWindowWidth);
			gtk_signal_connect(GTK_OBJECT(prefs_control.FolderWindowWidth), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_folderwindowwidth_cb), NULL);
			prefs_layout_table_item(table, prefs_control.FolderWindowWidth, _("Width"), &top, 0);
			top--;

			prefs_control.FolderWindowHeight = gtk_spin_button_new(GTK_ADJUSTMENT(prefs_control.adj.FolderWindowHeight), 1, 0);
			gtk_widget_set_sensitive(prefs_control.FolderWindowHeight, !euc.IntegratedDisplay);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), prefs_control.Prefs->FolderWindowHeight);
			gtk_signal_connect(GTK_OBJECT(prefs_control.FolderWindowHeight), "changed",
					GTK_SIGNAL_FUNC(prefs_setting_folderwindowheight_cb), NULL);
			prefs_layout_table_item(table, prefs_control.FolderWindowHeight, _("Height"), &top, 2);

			prefs_control.SnapshotFolder = gtk_button_new_with_label(_("Snapshot"));
			gtk_widget_set_sensitive(prefs_control.SnapshotFolder, !euc.IntegratedDisplay);
			gtk_signal_connect(GTK_OBJECT(prefs_control.SnapshotFolder), "clicked",
					GTK_SIGNAL_FUNC(prefs_setting_snapshotfolder_cb), NULL);
			gtk_table_attach(GTK_TABLE(table), prefs_control.SnapshotFolder,
					1, 2, top, top + 1,
					GTK_FILL, GTK_FILL,
					0, 0);
		}
		gtk_container_add(GTK_CONTAINER(frame), table);
	}
	gtk_box_pack_start(GTK_BOX(page), frame, FALSE, TRUE, 2);
#endif

	return page;
}

/* Widget callbacks.
 */
void prefs_setting_askbeforequitting_cb(GtkWidget *widget, gpointer data)
{
	prefs.AskBeforeQuitting = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}
void prefs_setting_mainwindowleft_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->MainWindowLeft = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_mainwindowtop_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->MainWindowTop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_mainwindowwidth_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->MainWindowWidth = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_mainwindowheight_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->MainWindowHeight = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void prefs_setting_snapshotmain_cb(GtkWidget *widget, gpointer data)
{	
	GdkWindow *window = NULL;
	gint x, y, w, h;
	
	window = euc.Window->window;
	gdk_window_get_position(window, &x, &y);
	gdk_window_get_size(window, &w, &h);

	prefs_control.Prefs->MainWindowLeft = x;
	prefs_control.Prefs->MainWindowTop = y;
	prefs_control.Prefs->MainWindowWidth = w;
	prefs_control.Prefs->MainWindowHeight = h;

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowLeft), x);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowTop), y);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowWidth), w);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(prefs_control.MainWindowHeight), h);
}

void prefs_setting_deletesentmail_cb(GtkWidget *widget, gpointer data)
{
	prefs_control.Prefs->DeleteSentMail = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
}


