/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: prefsinit.c
 *   	 First-time configuration code
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include <glib.h>
#include <libgnomeui/gnome-druid-page-standard.h>

#include "eucalyptus.h"

/* This function builds a GnomeDruid that will guide
 * the user through an initial configuration.
 */
glong prefs_new_config(void)
{
	struct EucDruid *euc_druid;
	GtkWidget *window, *druid, *start_page, *page, *end_page;
	GdkImlibImage *logo = NULL, *watermark = NULL;

	errno = 0;

	if(euc_druid = g_malloc0(sizeof(struct EucDruid)))
	{
		glong i;
		gboolean done = FALSE;
		
		/* Set up images.
		 */
		logo = gdk_imlib_load_image("eucalyptus-logo.png");
		watermark = gdk_imlib_load_image("eucalyptus-watermark.png");

		/* Create the Druid's window.
		 */		
		window = euc_druid->Window = gnome_app_new(APPNAME, _("Set up Eucalyptus"));

		/* Create the Druid.
		 */
		druid = euc_druid->Druid = gnome_druid_new();

		/* Connect signals.
		 */
		gtk_signal_connect(GTK_OBJECT(druid), "cancel",
				GTK_SIGNAL_FUNC(prefs_druid_cancel), euc_druid);

		/* Build start page.
		 */	
		start_page = euc_druid->StartPage = gnome_druid_page_start_new_with_vals(_("Eucalyptus"),
				_("Welcome to Eucalyptus!\n"
				  "\n"
				  "It seems that this is your first time using\n"
				  "Eucalyptus.  This Druid will guide you through\n"
				  "some initial setup, so you can start using\n"
				  "Eucalyptus right away."),
				logo, watermark);
		gnome_druid_append_page(GNOME_DRUID(druid), GNOME_DRUID_PAGE(start_page));
		
		/* Build config pages.
		 */
		for(i=0; !done; i++)
		{
			GtkWidget *label, *table;
			guint top = 0;
			
			switch(i)
			{
				/* User information.
				 */
				case 0:
					page = gnome_druid_page_standard_new_with_vals(_("User information"), logo);
					{
						table = gtk_table_new(3, 2, FALSE);
						
						euc_druid->RealName = gtk_entry_new();
						prefs_layout_table_item(table, euc_druid->RealName, _("Real name"), &top, 0);
						
						euc_druid->Organization = gtk_entry_new();
						prefs_layout_table_item(table, euc_druid->Organization, _("Organization"), &top, 0);
						
						euc_druid->Address = gtk_entry_new();
						prefs_layout_table_item(table, euc_druid->Address, _("Email address"), &top, 0);
						
						gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(page)->vbox), table, FALSE, TRUE, GNOME_PAD_SMALL);
					}
					gnome_druid_append_page(GNOME_DRUID(druid), GNOME_DRUID_PAGE(page));
					
					gtk_widget_show_all(page);
					break;

				/* SMTP server.
				 */
				case 1:
					page = gnome_druid_page_standard_new_with_vals(_("SMTP Server"), logo);
					{
						GtkWidget *label;
						
						table = gtk_table_new(2, 2, FALSE);

						label = gtk_label_new(_("Enter the address for your outgoing mail server"));
						prefs_layout_table_item(table, label, NULL, &top, 0);
						
						euc_druid->SMTPServerName = gtk_entry_new();
						prefs_layout_table_item(table, euc_druid->SMTPServerName, _("Server name"), &top, 0);
						
						euc_druid->SMTPAdjustment = gtk_adjustment_new(25, 0, 65535, 1, 1, 10);
						euc_druid->SMTPServerPort = gtk_spin_button_new(GTK_ADJUSTMENT(euc_druid->SMTPAdjustment), 1, 0);
						prefs_layout_table_item(table, euc_druid->SMTPServerPort, _("Port"), &top, 0);
						
						gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(page)->vbox), table, FALSE, TRUE, GNOME_PAD_SMALL);
					}
					gnome_druid_append_page(GNOME_DRUID(druid), GNOME_DRUID_PAGE(page));
					
					gtk_widget_show_all(page);
					break;

				/* POP server(s).
				 */
				case 2:
					page = gnome_druid_page_standard_new_with_vals(_("POP Server"), logo);
					{
						table = gtk_table_new(3, 2, FALSE);
						
						label = gtk_label_new(_("Enter the address for your incoming mail server"));
						prefs_layout_table_item(table, label, NULL, &top, 0);
						
						euc_druid->POPServerName = gtk_entry_new();
						prefs_layout_table_item(table, euc_druid->POPServerName, _("Server name"), &top, 0);
						
						euc_druid->POPAdjustment = gtk_adjustment_new(110, 0, 65535, 1, 1, 10);
						euc_druid->POPServerPort = gtk_spin_button_new(GTK_ADJUSTMENT(euc_druid->POPAdjustment), 1, 0);
						prefs_layout_table_item(table, euc_druid->POPServerPort, _("Port"), &top, 0);
						
						euc_druid->POPUsername = gtk_entry_new();
						prefs_layout_table_item(table, euc_druid->POPUsername, _("Username"), &top, 0);
						
						euc_druid->POPPassword = gtk_entry_new();
						gtk_entry_set_visibility(GTK_ENTRY(euc_druid->POPPassword), FALSE);
						prefs_layout_table_item(table, euc_druid->POPPassword, _("Password"), &top, 0);
						
						label = gtk_label_new(_("(You can add more servers from\nthe \'Settings\' menu."));
						prefs_layout_table_item(table, label, NULL, &top, 0);
						
						gtk_box_pack_start(GTK_BOX(GNOME_DRUID_PAGE_STANDARD(page)->vbox), table, FALSE, TRUE, GNOME_PAD_SMALL);
					}
					gnome_druid_append_page(GNOME_DRUID(druid), GNOME_DRUID_PAGE(page));
					
					gtk_widget_show_all(page);
					break;
					
				default:
					done = TRUE;
					break;
			}
		}

		/* Build end page.
		 */
		end_page = euc_druid->EndPage = gnome_druid_page_start_new_with_vals(_("Eucalyptus"),
				_("Now that that's done, click \"Finish\" to save\n"
				  "your settings and get started using Eucalyptus."),
				logo, watermark);
		gnome_druid_append_page(GNOME_DRUID(druid), GNOME_DRUID_PAGE(end_page));
		
		gtk_signal_connect(GTK_OBJECT(end_page), "prepare",
				GTK_SIGNAL_FUNC(prefs_druid_finish_prepare), euc_druid);
		gtk_signal_connect(GTK_OBJECT(end_page), "back",
				GTK_SIGNAL_FUNC(prefs_druid_finish_back), euc_druid);
		gtk_signal_connect(GTK_OBJECT(end_page), "finish",
				GTK_SIGNAL_FUNC(prefs_druid_finish), euc_druid);

		/* Put it in the window and show it.
		 */
		gnome_app_set_contents(GNOME_APP(window), druid);
		gtk_widget_show_all(window);
		
		euc.Flags |= EGF_INITCONFIG;
	}
	else
		errno = ENOMEM;
	
	return errno;
}

gboolean prefs_druid_cancel(GnomeDruid *druid, struct EucDruid *euc_druid)
{
	/* User aborted initial setup of the program.
	 * FIXME: Should we continue with the program,
	 * or exit? (We'll quit for now.)
	 */
	gtk_main_quit();

	/* Clean up.
	 */
	gtk_widget_destroy(euc_druid->Window);

	g_free(euc_druid);
	
	return FALSE;
}

void prefs_druid_finish_prepare(GnomeDruidPage *page, gpointer arg1, struct EucDruid *euc_druid)
{
	gnome_druid_set_buttons_sensitive(GNOME_DRUID(euc_druid->Druid), TRUE, TRUE, TRUE);
	gnome_druid_set_show_finish(GNOME_DRUID(euc_druid->Druid), TRUE);
}

gboolean prefs_druid_finish_back(GnomeDruidPage *page, gpointer arg1, struct EucDruid *euc_druid)
{
	gnome_druid_set_show_finish(GNOME_DRUID(euc_druid->Druid), FALSE);
	
	return FALSE;
}

void prefs_druid_finish(GnomeDruidPage *page, gpointer arg1, struct EucDruid *euc_druid)
{
	Folder *folder;
	Server *server;
	glong port;
	gchar *str;
	
	/* Get what's in the widgets.
	 */
	str = gtk_editable_get_chars(GTK_EDITABLE(euc_druid->RealName), 0, -1);
	strncpy(uprefs.RealName, str, sizeof(uprefs.RealName) - 1);
	g_free(str);
	
	str = gtk_editable_get_chars(GTK_EDITABLE(euc_druid->Address), 0, -1);
	strncpy(uprefs.Address, str, sizeof(uprefs.Address) - 1);
	g_free(str);

	str = gtk_editable_get_chars(GTK_EDITABLE(euc_druid->Organization), 0, -1);
	strncpy(uprefs.Organization, str, sizeof(uprefs.Organization) - 1);
	g_free(str);

	str = gtk_editable_get_chars(GTK_EDITABLE(euc_druid->SMTPServerName), 0, -1);
	strncpy(prefs.SMTPServer, str, sizeof(prefs.SMTPServer) - 1);
	g_free(str);
	
	prefs.SMTPPort = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(euc_druid->SMTPServerPort));

	/* Create a POP server object.
	 */
	str = gtk_editable_get_chars(GTK_EDITABLE(euc_druid->POPServerName), 0, -1);
	if(*str && (server = pop_server_new(PST_POP3)))
	{
		strncpy(server->Host, str, sizeof(server->Host) - 1);
		server->Port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(euc_druid->POPServerPort));
		g_free(str);
		
		str = gtk_editable_get_chars(GTK_EDITABLE(euc_druid->POPUsername), 0, -1);
		strncpy(server->Username, str, sizeof(server->Username) - 1);
		g_free(str);
		
		str = gtk_editable_get_chars(GTK_EDITABLE(euc_druid->POPPassword), 0, -1);
		strncpy(server->Password, str, sizeof(server->Password) - 1);

		list_append(&euc.Servers, server);
	}
	g_free(str);

	/* Save it.
	 */
	if(!prefs_save())
	{
		/* Should return an error message.
		 */
		euc_request(NULL,
				_("Eucalyptus error"),
				_("Unable to save settings!"),
				euc_ok_button, 0,
				0);
		 
		return;
	}

	/* Clean up.
	 */
	gtk_widget_destroy(euc_druid->Window);

	g_free(euc_druid);
	
	/* Open the main window.
	 */
	euc_main_window();

	euc.Flags &= ~EGF_INITCONFIG;
}

