/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: prefsuser.c
 *   	 Prefs handling code for user settings panel
 *
 * Copyright  1997-2000 Isengard Developments
 * All rights reserved.
 */

#include <string.h>

#include "eucalyptus.h"

struct
{
	GtkWidget		  	*Window,
					*DefaultSig,
					*Taglines,
					*Directory,
				  	*Layout,
					*Name,
					*Address,
					*Organization,
					*EditSig,
					*EditTaglines,
					*AddressBook,
					*MailDir,
					*Mailcap,
					*Filters,
					*ReplyTo;
	struct User		 Prefs;
} uprefs_control;

void prefs_user_set_defaults(void)
{
	memset(&uprefs, 0, sizeof(struct User));
}

void edit_user_prefs_cb(GtkWidget *widget, gpointer data)
{
	prefs_user_open_window();
}

gboolean prefs_user_open_window(void)
{
	GtkWidget *vbox, *hbox, *label, *table;
	
	if(uprefs_control.Window)
	{
		gtk_widget_show_all(uprefs_control.Window);
		return TRUE;
	}

	memcpy(&uprefs_control.Prefs, &uprefs, sizeof(struct User));
	
	/* Create the window.
	 */
	uprefs_control.Window = gnome_dialog_new(_("User information"),
			GNOME_STOCK_BUTTON_OK,
			GNOME_STOCK_BUTTON_CANCEL,
			NULL);
	
	/* Fill it.
	 */
	table = gtk_table_new(8, 2, FALSE);
	{
		guint top = 0;

		uprefs_control.Name = gnome_entry_new("real-name");
		gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(uprefs_control.Name))), uprefs.RealName);
		prefs_layout_table_item(table, uprefs_control.Name, _("Real name"), &top, 0);

		uprefs_control.Address = gnome_entry_new("address");
		gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(uprefs_control.Address))), uprefs.Address);
		prefs_layout_table_item(table, uprefs_control.Address, _("Address"), &top, 0);
		
		uprefs_control.ReplyTo = gnome_entry_new("reply-to");
		gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(uprefs_control.ReplyTo))), uprefs.ReplyTo);
		prefs_layout_table_item(table, uprefs_control.ReplyTo, _("Reply-to"), &top, 0);
		
		uprefs_control.Organization = gnome_entry_new("organization");
		gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(uprefs_control.Organization))), uprefs.Organization);
		prefs_layout_table_item(table, uprefs_control.Organization, _("Organization"), &top, 0);
		
		uprefs_control.AddressBook = gnome_file_entry_new("addr-book", _("Select address book file"));
		gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(uprefs_control.AddressBook))), uprefs.AddressBook);
		prefs_layout_table_item(table, uprefs_control.AddressBook, _("Address book"), &top, 0);

		hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
		{
			uprefs_control.DefaultSig = gnome_file_entry_new("default-sig", _("Select signature file"));
			gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(uprefs_control.DefaultSig))), uprefs.Signature);
			gtk_box_pack_start(GTK_BOX(hbox), uprefs_control.DefaultSig, TRUE, TRUE, 0);

			uprefs_control.EditSig = gtk_button_new_with_label(_("Edit"));
			gtk_box_pack_start(GTK_BOX(hbox), uprefs_control.EditSig, FALSE, FALSE, 0);
		}
		prefs_layout_table_item(table, hbox, _("Signature"), &top, 0);
		
		hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
		{
			uprefs_control.Taglines = gnome_file_entry_new("taglines", _("Select taglines file"));
			gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(uprefs_control.Taglines))), uprefs.Taglines);
			gtk_box_pack_start(GTK_BOX(hbox), uprefs_control.Taglines, TRUE, TRUE, 0);

			uprefs_control.EditTaglines = gtk_button_new_with_label(_("Edit"));
			gtk_box_pack_start(GTK_BOX(hbox), uprefs_control.EditTaglines, FALSE, FALSE, 0);
		}
		prefs_layout_table_item(table, hbox, _("Taglines"), &top, 0);
	}
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(uprefs_control.Window)->vbox), table, FALSE, TRUE, 0);

	gnome_dialog_set_close(GNOME_DIALOG(uprefs_control.Window), TRUE);

	/* Set signals.
	 */
	gnome_dialog_button_connect(GNOME_DIALOG(uprefs_control.Window), 0,
			GTK_SIGNAL_FUNC(prefs_user_okay_cb), NULL);
	gnome_dialog_button_connect_object(GNOME_DIALOG(uprefs_control.Window), 0,
			GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(uprefs_control.Window));
	gnome_dialog_button_connect_object(GNOME_DIALOG(uprefs_control.Window), 1,
			GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(uprefs_control.Window));

	gtk_signal_connect(GTK_OBJECT(uprefs_control.Window), "destroy",
			GTK_SIGNAL_FUNC(prefs_user_destroy_cb), NULL);

	/* Open the window.
	 */	
	gtk_widget_show_all(uprefs_control.Window);
}

/* Callbacks.
 */
void prefs_user_okay_cb(GnomeDialog *dialog, gint button, gpointer data)
{
	memcpy(&uprefs, &uprefs_control.Prefs, sizeof(struct User));
}

void prefs_user_destroy_cb(GnomeDialog *dialog, gpointer data)
{
	uprefs_control.Window = NULL;
}

