/*
 * Eucalyptus
 * by Paul A. Schifferer
 *
 * Module: smtp.c
 *   	 SMTP client function server
 *
 * Copyright  1997-9 Isengard Developments
 * All rights reserved.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <netinet/in.h>
#include <netdb.h>
#include <gtk/gtk.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "eucalyptus.h"

struct SMTPControl smtp_control;

struct List *smtp_parse_addressees(gchar *addr, gint16 flags)
{
	struct List *list = NULL;
	gchar *work;
	
	work = g_strdup(addr);

	if(list = g_malloc0(sizeof(struct List)))
	{
		gchar *p, *s;
		gboolean escaped = FALSE, quoting = FALSE, bracket = 0, done = FALSE;

		list_init(list);

		s = p = work;

		while(!done)
		{
			switch(*p)
			{
				case ':':
					if(!escaped &&
					   !quoting)
						s = p + 1;
					escaped = FALSE;
					break;
				case 0:
					done = TRUE;
				case ';':
				case ',':
					if(!escaped ||
					   !(*p))
					{
						glong l;

						if(*p &&
						   (quoting ||
						    bracket))
							break;
						*p = '\0';
						if((l = strlen(s) + 1) > 1)
						{
							gchar *str;
							
							str = g_malloc0(l);

							if(flags & PA_FULL)
								strncpy(str, s, l - 1);
							else
								parse_address(s, NULL, 0, str, l, FALSE);

							list_append(list, str);
						}

						s = p + 1;
					}
					escaped = FALSE;
					break;
				case '\"':
					if(!escaped)
						quoting = !quoting;
					escaped = FALSE;
					break;
				case '<':
					if(!escaped)
						bracket = 1;
					escaped = FALSE;
					break;
				case '>':
					if(!escaped)
						bracket = 0;
					escaped = FALSE;
					break;
				case '\\':
					escaped = TRUE;
					break;
			}

			p++;
		}

		if(!g_list_length(list->List))
		{
			g_free(list);
			list = NULL;
		}
	}

	return(list);
}

void smtp_free_addressees(struct List *list)
{
	GList *node;

	if(!list)
		return;

	while(node = g_list_first(list->List))
	{
		gchar *str = (gchar *)node->data;
		
		list->List = list_remove(list, str);
		g_free(str);
	}

	g_free(list);
}


